#ifndef _FbxSdk_h
#define _FbxSdk_h

/**************************************************************************************

 Copyright  2000 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

/**
  *	\mainpage FBX SDK Index
  * <p>
  * \image html fbx.jpg
  * <p>
  *	\section welcome Welcome to the FBX SDK
  *	The FBX SDK is a C++ software development kit (SDK) that allows you 
  *	to import and export 3D scenes in Alias FBX file format. The FBX SDK 
  * reads FBX files produced by FiLMBOX version 2.5 and later and writes FBX 
  * files compatible with MotionBuilder version 6.0. 
  *	<p>
  *	\section features Features
  * The following 3D scene elements and settings are supported by the FBX SDK:
  * <ul><li>Mesh
  *     <li>Nurb
  *     <li>Patch
  *     <li>Texture mapping over patch, nurb and mesh
  *     <li>Material mapping over patch, nurb and mesh
  *     <li>Multiple cameras
  *     <li>Global camera settings
  *     <li>Camera switcher
  *     <li>Multiple lights and gobos
  *     <li>Global light and fog settings
  *     <li>Marker
  *     <li>Skeleton segments (root, limb and limb node)
  *     <li>Link constraint on control points of a patch, nurb or mesh
  *     <li>Shape constraint on control points of a patch, nurb or mesh
  *     <li>Multiple takes of animation
  *     <li>Global time settings
  *     <li>Pivots</ul>
  * <p>
  *	\section otherdocumentation Other Documentation
  * Apart from this reference guide, an overview document and various examples 
  * are provided to show how to use the FBX SDK. 
  * <p>
  */

#ifndef K_PLUGIN
    #define K_PLUGIN
#endif

#ifndef K_FBXSDK
    #define K_FBXSDK
#endif

#ifndef K_FBXSDK_INTERFACE
    #define K_FBXSDK_INTERFACE
#endif

#include <kaydara.h>
#include <kaydaradef.h>


#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif


#include <kfbxio/kfbxio.h>
#include <kfbxio/kfbxexporter.h>
#include <kfbxio/kfbximporter.h>

#include <kfbxmath/kfbxmatrix.h>
#include <kfbxmath/kfbxquaternion.h>
#include <kfbxmath/kfbxvector2.h>
#include <kfbxmath/kfbxvector4.h>
#include <kfbxmath/kfbxxmatrix.h>

#include <kfbxplugins/kfbxsdkmanager.h>
#include <kfbxplugins/kfbxscene.h>
#include <kfbxplugins/kfbxgloballightsettings.h>
#include <kfbxplugins/kfbxglobalcamerasettings.h>
#include <kfbxplugins/kfbxglobaltimesettings.h>
#include <kfbxplugins/kfbxcolor.h>
#include <kfbxplugins/kfbxnode.h>
#include <kfbxplugins/kfbxnodeattribute.h>
#include <kfbxplugins/kfbxmarker.h>
#include <kfbxplugins/kfbxcamera.h>
#include <kfbxplugins/kfbxcameraswitcher.h>
#include <kfbxplugins/kfbxlight.h>
#include <kfbxplugins/kfbxopticalreference.h>
#include <kfbxplugins/kfbxskeleton.h>
#include <kfbxplugins/kfbxgeometry.h>
#include <kfbxplugins/kfbxgeometrybase.h>
#include <kfbxplugins/kfbxgeometryconverter.h>
#include <kfbxplugins/kfbxmesh.h>
#include <kfbxplugins/kfbxnurb.h>
#include <kfbxplugins/kfbxpatch.h>
#include <kfbxplugins/kfbxtexture.h>
#include <kfbxplugins/kfbxmaterial.h>
#include <kfbxplugins/kfbxlink.h>
#include <kfbxplugins/kfbxshape.h>
#include <kfbxplugins/kfbxtakenode.h>
#include <kfbxplugins/kfbxtakeinfo.h>
#include <kfbxplugins/kfbxpose.h>
#include <kfbxplugins/kfbxnull.h>
#include <kfbxplugins/kfbxthumbnail.h>
#include <kfbxplugins/kfbxsceneinfo.h>
#include <kfbxplugins/kfbxuserproperty.h>
#include <kfbxplugins/kfbxvideo.h>
#include <kfbxplugins/kfbxgenericnode.h>

#include <kfbxcharacter/kfbxcharacter.h>
#include <kfbxcharacter/kfbxcharacterpose.h>
#include <kfbxcharacter/kfbxcontrolset.h>
#include <kfbxcharacter/kfbxfilterset.h>

#include <kfcurve/kfcurve.h>
#include <kfcurve/kfcurvenode.h>
#include <kfcurve/kfcurveutils.h>

#include <klib/karrayul.h>
#include <klib/kerror.h>
#include <klib/kmemory.h>
#include <klib/kstring.h>
#include <klib/ktime.h>

#endif // #ifndef _FbxSdk_h
