/*!  \file kfbxcharacter.h
 */

#ifndef _KFbxCharacter_h
#define _KFbxCharacter_h

/**************************************************************************************

 Copyright  2002 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxmath/kfbxvector4.h>

#include <klib/kstring.h>
#include <klib/kerror.h>

#include <kfbxplugins/kfbxconstraint.h>

#include <kfbxplugins/kfbxnamespacedef.h>
#include <kfbxplugins/kfbxnodelimits.h>
namespace FBXSDK_NAMESPACE {


class KFbxNode;
class KFbxControlSet;
class KFbxFilterSet;
class KFbxSdkManager;
class KFbxCharacterProperties;

/** \enum ECharacterInputType           Character input type.
      * - \e eCharacterInputActor       Not supported.
      * - \e eCharacterInputCharacter   The character's input is another character.
      * - \e eCharacterInputMarkerSet   The character's input is a control rig.
      * - \e eCharacterOutputMarkerSet  Not supported.
      * - \e eCharacterInputStance      The character's input is the stance pose.
      */
typedef enum 
{ 
	eCharacterInputActor = 0, 
	eCharacterInputCharacter = 1, 
	eCharacterInputMarkerSet = 2, 
	eCharacterOutputMarkerSet = 3, 
	eCharacterInputStance = 4 
} ECharacterInputType;


typedef enum  
{	
	eCharacterHips = 0,             // Required
    eCharacterLeftHip,              // Required
    eCharacterLeftKnee,             // Required
	eCharacterLeftAnkle,            // Required
	eCharacterLeftFoot,
	eCharacterRightHip,             // Required
	eCharacterRightKnee,            // Required
	eCharacterRightAnkle,           // Required
	eCharacterRightFoot,
	eCharacterWaist,				// Spine0, Required
	eCharacterChest,		        // Spine1
	eCharacterLeftCollar,
	eCharacterLeftShoulder,         // Required
	eCharacterLeftElbow,            // Required
	eCharacterLeftWrist,            // Required
	eCharacterRightCollar,
	eCharacterRightShoulder,        // Required
	eCharacterRightElbow,           // Required
	eCharacterRightWrist,           // Required
	eCharacterNeck,
	eCharacterHead,                 // Required
	eCharacterLeftHipRoll,
	eCharacterLeftKneeRoll,
	eCharacterRightHipRoll,
	eCharacterRightKneeRoll,
	eCharacterLeftShoulderRoll,
	eCharacterLeftElbowRoll,
	eCharacterRightShoulderRoll,
	eCharacterRightElbowRoll,
	eCharacterSpine2,
	eCharacterSpine3,
	eCharacterSpine4,
	eCharacterSpine5,
	eCharacterSpine6,
	eCharacterSpine7,
	eCharacterSpine8,
	eCharacterSpine9,
	eCharacterLeftThumbA,
	eCharacterLeftThumbB,
	eCharacterLeftThumbC,
	eCharacterLeftIndexA,
	eCharacterLeftIndexB,
	eCharacterLeftIndexC,
	eCharacterLeftMiddleA,
	eCharacterLeftMiddleB,
	eCharacterLeftMiddleC,
	eCharacterLeftRingA,
	eCharacterLeftRingB,
	eCharacterLeftRingC,
	eCharacterLeftPinkyA,
	eCharacterLeftPinkyB,
	eCharacterLeftPinkyC,
	eCharacterRightThumbA,
	eCharacterRightThumbB,
	eCharacterRightThumbC,
	eCharacterRightIndexA,
	eCharacterRightIndexB,
	eCharacterRightIndexC,
	eCharacterRightMiddleA,
	eCharacterRightMiddleB,
	eCharacterRightMiddleC,
	eCharacterRightRingA,
	eCharacterRightRingB,
	eCharacterRightRingC,
	eCharacterRightPinkyA,
	eCharacterRightPinkyB,
	eCharacterRightPinkyC,
	eCharacterReference,
	eCharacterLeftFloor,
	eCharacterRightFloor,
	eCharacterHipsTranslation,
	eCharacterProps0,
	eCharacterProps1,
	eCharacterProps2,
	eCharacterProps3,
	eCharacterProps4,

	// Added in 4.0x as patch.

	eCharacterGameModeParentLeftHipRoll,
	eCharacterGameModeParentLeftKnee,
	eCharacterGameModeParentLeftKneeRoll,
	eCharacterGameModeParentRightHipRoll,
	eCharacterGameModeParentRightKnee,
	eCharacterGameModeParentRightKneeRoll,
	eCharacterGameModeParentLeftShoulderRoll,	
	eCharacterGameModeParentLeftElbow,	
	eCharacterGameModeParentLeftElbowRoll,	
	eCharacterGameModeParentRightShoulderRoll,
	eCharacterGameModeParentRightElbow,		
	eCharacterGameModeParentRightElbowRoll,	


	// Elements added for Dominus.
	// From this point, the enum doesn't have to be synchronized with the HumanIK library.

	eCharacterLeftHandFloor,
	eCharacterRightHandFloor,

	eCharacterLeftHand,
	eCharacterRightHand,
	
	eCharacterNeck1,
	eCharacterNeck2,
	eCharacterNeck3,
	eCharacterNeck4,
	eCharacterNeck5,
	eCharacterNeck6,
	eCharacterNeck7,
	eCharacterNeck8,
	eCharacterNeck9,

	eCharacterLeftInHandThumb,
	eCharacterLeftThumbD,
	
	eCharacterLeftInHandIndex,
	eCharacterLeftIndexD,
	
	eCharacterLeftInHandMiddle,
	eCharacterLeftMiddleD,
	
	eCharacterLeftInHandRing,
	eCharacterLeftRingD,
	
	eCharacterLeftInHandPinky,
	eCharacterLeftPinkyD,

	eCharacterLeftInHandExtraFinger,
	eCharacterLeftExtraFingerA,
	eCharacterLeftExtraFingerB,
	eCharacterLeftExtraFingerC,
	eCharacterLeftExtraFingerD,

	eCharacterRightInHandThumb,
	eCharacterRightThumbD,
	
	eCharacterRightInHandIndex,
	eCharacterRightIndexD,
	
	eCharacterRightInHandMiddle,
	eCharacterRightMiddleD,
	
	eCharacterRightInHandRing,
	eCharacterRightRingD,
	
	eCharacterRightInHandPinky,
	eCharacterRightPinkyD,

	eCharacterRightInHandExtraFinger,
	eCharacterRightExtraFingerA,
	eCharacterRightExtraFingerB,
	eCharacterRightExtraFingerC,
	eCharacterRightExtraFingerD,

	eCharacterLeftInFootThumb,
	eCharacterLeftFootThumbA,
	eCharacterLeftFootThumbB,
	eCharacterLeftFootThumbC,
	eCharacterLeftFootThumbD,
	eCharacterLeftInFootIndex,
	eCharacterLeftFootIndexA,
	eCharacterLeftFootIndexB,
	eCharacterLeftFootIndexC,
	eCharacterLeftFootIndexD,
	eCharacterLeftInFootMiddle,
	eCharacterLeftFootMiddleA,
	eCharacterLeftFootMiddleB,
	eCharacterLeftFootMiddleC,
	eCharacterLeftFootMiddleD,
	eCharacterLeftInFootRing,
	eCharacterLeftFootRingA,
	eCharacterLeftFootRingB,
	eCharacterLeftFootRingC,
	eCharacterLeftFootRingD,
	eCharacterLeftInFootPinky,
	eCharacterLeftFootPinkyA,
	eCharacterLeftFootPinkyB,
	eCharacterLeftFootPinkyC,
	eCharacterLeftFootPinkyD,
	eCharacterLeftInFootExtraFinger,
	eCharacterLeftFootExtraFingerA,
	eCharacterLeftFootExtraFingerB,
	eCharacterLeftFootExtraFingerC,
	eCharacterLeftFootExtraFingerD,

	eCharacterRightInFootThumb,
	eCharacterRightFootThumbA,
	eCharacterRightFootThumbB,
	eCharacterRightFootThumbC,
	eCharacterRightFootThumbD,
	eCharacterRightInFootIndex,
	eCharacterRightFootIndexA,
	eCharacterRightFootIndexB,
	eCharacterRightFootIndexC,
	eCharacterRightFootIndexD,
	eCharacterRightInFootMiddle,
	eCharacterRightFootMiddleA,
	eCharacterRightFootMiddleB,
	eCharacterRightFootMiddleC,
	eCharacterRightFootMiddleD,
	eCharacterRightInFootRing,
	eCharacterRightFootRingA,
	eCharacterRightFootRingB,
	eCharacterRightFootRingC,
	eCharacterRightFootRingD,
	eCharacterRightInFootPinky,
	eCharacterRightFootPinkyA,
	eCharacterRightFootPinkyB,
	eCharacterRightFootPinkyC,
	eCharacterRightFootPinkyD,
	eCharacterRightInFootExtraFinger,
	eCharacterRightFootExtraFingerA,
	eCharacterRightFootExtraFingerB,
	eCharacterRightFootExtraFingerC,
	eCharacterRightFootExtraFingerD,

	eCharacterLastNodeId
} ECharacterNodeId;


typedef enum 
{
	eCharacterGroup_Base,
	eCharacterGroup_Auxiliary,
	eCharacterGroup_Spine,
	eCharacterGroup_Roll,
	eCharacterGroup_Special,
	eCharacterGroup_LeftHand,
	eCharacterGroup_RightHand,
	eCharacterGroup_Props,
	eCharacterGroup_GameModeParent,

	// Added for 4.5 

	eCharacterGroup_Neck,
	eCharacterGroup_LeftFoot,
	eCharacterGroup_RightFoot,

	eLastCharacterGroupId
} ECharacterGroupId;


typedef enum 
{
	eCharacterLink,
	eControlSetLink,
	eControlSetEffector,
	eControlSetEffectorAux
} ECharacterLinkType;

/** \class KFbxCharacterLink
  *
  * \brief This class represents a link between a given character's node and the associated node in the hierarchy. It also contains
  *  the characterization matrix (offset) for the linked character's node.
  */
class KFBX_DLL KFbxCharacterLink
{

public:

	KFbxCharacterLink();
	KFbxCharacterLink(const KFbxCharacterLink& pCharacterLink);
	KFbxCharacterLink& operator=(const KFbxCharacterLink& pCharacterLink);
	void Reset();

    //! The character's node in hierarchy linked to this character link.
	KFbxNode* mNode;

    //! A template name is a naming convention that is used to automatically map
    //! the nodes of other skeletons that use the same naming convention for automatic characterization.
	KString mTemplateName;

    //! Get offset position of this character link.
	KFbxVector4 mOffsetT;
    //! Get offset rotation of this character link.
	KFbxVector4 mOffsetR;
    //! Get offset scale of this character link.
	KFbxVector4 mOffsetS;
	//! Get the parent offset rotation of this character link
	KFbxVector4 mParentROffset;

	//! True if this character link has a defined rotation space
	bool		mHasRotSpace;
	//! Get the rotation limits of this character link
	KFbxLimits  mRLimits;
	//! Get the PreRotation of this character link
	KFbxVector4 mPreRotation;
	//! Get the PostRotation of this character link
	KFbxVector4 mPostRotation;
	//! Get the rotation order of this character link
	int			mRotOrder;
	//! Get the axis length of this character link
	double		mAxisLen;


///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

	void *mPropertyLink;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

/** \class KFbxCharacter
  *
  * \brief This class contains all methods to setup an exported character or query information on an imported character.
  *
  *  There are some methods to manipulate the KFbxCharacterLink, KFbxControlSet, and KFbxFilterSet.
  *  
  *  The most important part of a KFbxCharacter is the KFbxCharacterLink. There is one KFbxCharacterLink for each characterized node.
  *  This class contains the associated KFbxNode to the node represented by this link and also the characterization offset of this node. 
  *  For more information see KFbxCharacterLink class documentation. All the character's parameter can be accessed by getting a reference
  *  to the KFbxFilterSet with the function KFbxCharacter::GetFilterSet(), and the control set can be accessed by getting a reference
  *  to the KFbxControlSet with the function KFbxCharacter::GetControlSet().
  *
  */
class KFBX_DLL KFbxCharacter : public KFbxConstraint
{

public:

	//! Reset to default values. 
    //!     - Input type will be set to eCharacterInputStance.
    //!     - Input object will be set to NULL.
    //!     - Each Character link will be reset.
    //!     - The control set will be reset.
	void Reset();

	/** Set input type and index.
	  * \param pInput           Input type.
	  * \param pInputObject     Pointer to input character if input type equals eCharacterInputCharacter, otherwise NULL.
      */
	void SetInput(ECharacterInputType pInputType, KFbxObject* pInputObject = NULL);

	//! Get input type.
	ECharacterInputType GetInputType();

	/** Get input actor or character.
	  * \return     
      *             - Pointer to input character if input type equals eCharacterInputCharacter.
	  *               In this case, the returned pointer can be casted to a pointer of type KFbxCharacter.
	  *             - Null pointer if input object has not been set or if input type is not set to eCharacterInputCharacter.
	  */
	KFbxObject* GetInputObject();

	/** Associate a character link to a given character node ID. If a character link already exists for this character node ID
      * it will be removed.
      * \param pCharacterNodeId     Character node ID.
	  * \param pCharacterLink       Character link.
	  * \return \c true on success, \c false otherwise.
	  */
	bool SetCharacterLink(ECharacterNodeId pCharacterNodeId, const KFbxCharacterLink& pCharacterLink, bool pUpdateObjectList = true);

	/** Get a character link associated with a given character node ID.
      * \param pCharacterNodeId     ID of character node requested.
	  * \param pCharacterLink       Optional pointer to receive character link if function succeeds.
	  * \return \c true on success, \c false otherwise.
	  */
	bool GetCharacterLink(ECharacterNodeId pCharacterNodeId, KFbxCharacterLink* pCharacterLink = NULL);

	/** Get control set associated with the character.
      * \return                     Return the control set associated with the character.
      */
	KFbxControlSet& GetControlSet();

	/** Get filter set associated with the character.
      * \return                     Return the filter set associated with the character.
      */
	KFbxFilterSet& GetFilterSet();

	//! Get number of elements in a given character group.
	static int GetCharacterGroupCount(ECharacterGroupId pCharacterGroupId);

	//! Get character node ID of an element in a given character group.
	static ECharacterNodeId GetCharacterGroupElementByIndex(ECharacterGroupId pCharacterGroupId, int pIndex);

    
	//! Get character node name of an element in a given character group.
	static char* GetCharacterGroupNameByIndex(ECharacterGroupId pCharacterGroupId, int pIndex);

	//! Get character node version of an element in a given character group.
	static int GetCharacterGroupVersionByIndex(ECharacterGroupId pCharacterGroupId, int pIndex);

	//! Find the character group index associated with a given character node name.
	static bool FindCharacterGroupIndexByName(char* pName, bool pForceGroupId, ECharacterGroupId& pCharacterGroupId, int& pIndex);

	/** Get character node group and index of a given character node ID.
	  * \remarks Only works for a character node ID part of a group.
	  */
	static bool GetCharacterGroupIndexByElement(ECharacterNodeId pCharacterNodeId, ECharacterGroupId& pCharacterGroupId, int& pIndex);

	/** Get character node version of a given character node ID.
	  * \remarks Only works for a character node ID part of a group.
	  */
	static bool GetCharacterGroupVersionByElement(ECharacterNodeId pCharacterNodeId, int& pVersion);

  	/** Get character node name associated with a given character node ID.
      * \param pCharacterNodeId     Character node ID to get name.
      * \param pName                return the node name throught this parameter if node ID is found. Pointer will point to internal data,
      *                             so you don't need to allocate a string buffer before calling this function.
      * \return                     return true if a name exists for the given node ID.
	  */
	static bool GetCharacterNodeNameFromNodeId(ECharacterNodeId pCharacterNodeId, char*& pName);

	/** Get the character node ID associated with a given character node name.
      * \param pName                Character node name to get node ID.
      * \param pCharacterNodeId     return the node ID through this parameter if node name is found.
      * \return                     return true if a node ID exists for the given node name.
	  */
	static bool GetCharacterNodeIdFromNodeName(char* pName, ECharacterNodeId& pCharacterNodeId);

	typedef enum 
	{
		eINTERNAL_ERROR,
		eERROR_COUNT
	} EError; 


///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

	static int GetClassID();
	virtual KObject*  GetPropertyHandler();

public:

	virtual KFbxObject::ENameSpace GetNameSpace() const; 
	KFbxCharacter& operator=(KFbxCharacter& pCharacter); 

	KFbxCharacter(KFbxSdkManager& pManager, char* pName);
	~KFbxCharacter();

	void Destruct();

	virtual EConstraintType GetConstraintType();
	virtual KString			GetTypeName();

	KStringList	GetTypeFlags();

	// KFbxTakeNodeContainer virtual functions
	virtual void AddChannels(KFbxTakeNode *pTakeNode);
	virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);

	KFbxCharacterProperties* mPH;
	
	bool GetFilterSetNameFromPropertyNameOrModeAndIndex( char const*  &pFilterSetName, char const* pPropertyName, int pIndex )const ;
	bool GetPropertyNameAndModeNameAndIndexFromFilterSetName(char const* &pPropertyName,char const* &pPropertyModeName,int &pIndex, char const* pFilterSetName) const;	
	
	KError mError;

	KString mName;	
	KFbxCharacterLink mCharacterLink[eCharacterLastNodeId];
	KFbxControlSet* mControlSet;
	KFbxFilterSet* mFilterSet;

	friend class KFbxScene;
	friend class KFbxNode;
	friend class KFbxReaderFbx;
	friend class KFbxCharacterProperties;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:
	
};

typedef KFbxCharacter* HKFbxCharacter;

} // namespace
#endif // #ifndef _KFbxCharacter_h


