#ifndef _KFbxControlSet_h
#define _KFbxControlSet_h

/**************************************************************************************

 Copyright  2002 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxcharacter/kfbxcharacter.h>

#include <kfbxmath/kfbxxmatrix.h>

#include <klib/kstring.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxNode;
class KFbxSdkManager;
class KFbxControlSetPlug;
class KFbxControlSetPlugProperties;

typedef enum 
{
	eEffectorHips = 0,
	eEffectorLeftAnkle,
	eEffectorRightAnkle,
	eEffectorLeftWrist,
	eEffectorRightWrist,
	eEffectorLeftKnee,
	eEffectorRightKnee,
	eEffectorLeftElbow,
	eEffectorRightElbow,
	eEffectorChestOrigin,
	eEffectorChestEnd,
	eEffectorLeftFoot,
	eEffectorRightFoot,
	eEffectorLeftShoulder,
	eEffectorRightShoulder,
	eEffectorHead,
	eEffectorLeftHip,
	eEffectorRightHip,

	// Added for 4.5 

	eEffectorLeftHand,
	eEffectorRightHand,

	eEffectorLeftHandThumb,
	eEffectorLeftHandIndex,
	eEffectorLeftHandMiddle,
	eEffectorLeftHandRing,
	eEffectorLeftHandPinky,
	eEffectorLeftHandExtraFinger,

	eEffectorRightHandThumb,
	eEffectorRightHandIndex,
	eEffectorRightHandMiddle,
	eEffectorRightHandRing,
	eEffectorRightHandPinky,
	eEffectorRightHandExtraFinger,

	eEffectorLeftFootThumb,
	eEffectorLeftFootIndex,
	eEffectorLeftFootMiddle,
	eEffectorLeftFootRing,
	eEffectorLeftFootPinky,
	eEffectorLeftFootExtraFinger,

	eEffectorRightFootThumb,
	eEffectorRightFootIndex,
	eEffectorRightFootMiddle,
	eEffectorRightFootRing,
	eEffectorRightFootPinky,
	eEffectorRightFootExtraFinger,

	eEffectorLastNodeId
} EEffectorNodeId;

typedef enum 
{	
	eEffectorSetDefault = 0, 
	eEffectorSetAux1, 
	eEffectorSetAux2, 
	eEffectorSetAux3, 
	eEffectorSetAux4, 
	eEffectorSetAux5, 
	eEffectorSetAux6, 
	eEffectorSetAux7, 
	eEffectorSetAux8, 
	eEffectorSetAux9, 
	eEffectorSetAux10, 
	eEffectorSetAux11, 
	eEffectorSetAux12, 
	eEffectorSetAux13, 
	eEffectorSetAux14, 
	eLastEffectorSetId
} EEffectorSetId;


/** \class KFbxControlSetLink
  *
  * \brief This class represents a link between a given character's FK node and the associated node in the hierarchy.
  *
  */
class KFBX_DLL KFbxControlSetLink
{

public:

	KFbxControlSetLink();
	KFbxControlSetLink(const KFbxControlSetLink& pControlSetLink);
	KFbxControlSetLink& operator=(const KFbxControlSetLink& pControlSetLink);

	/** Reset to default values.
	  *
	  * Member mNode is set to NULL and member mTemplateName is cleared.
	  */
	void Reset();

	//! The character's node in a hierarchy linked to this cotnrol set link.
	KFbxNode* mNode;

	//! A template name is a naming convention that is used to automatically map
    //! the nodes of other skeletons that use the same naming convention for automatic characterization.
	KString mTemplateName;
};

/** \class KFbxEffector
  *
  * \brief This class represents a link between a given character's effector and the associated node in the hierarchy.
  *
  */
class KFBX_DLL KFbxEffector
{

public:

	KFbxEffector();
	KFbxEffector& operator=(KFbxEffector& pEffector);

	/** Reset to default values.
	  *
	  *		- mNode is set to NULL.
	  *		- mTemplateName is cleared.
	  *		- mShow is set to true.
	  */
	void Reset();

	//! The character's node in a hierarchy linked to this cotnrol set link.
	KFbxNode* mNode;

	//! Does this effector is visible.
	bool mShow;

	// These members are for backward compatibility and should not be used. These properties are now published through class KFbxControlSetPlug.
#ifndef DOXYGEN_SHOULD_SKIP_THIS	
	bool mTActive;
	bool mRActive;
	bool mCandidateTActive;
	bool mCandidateRActive;
#endif
};

/** \class KFbxControlSet
  *
  * \brief This class contains all methods to setup an exported control rig or query information on an imported control rig.
  *
  *  There are some methods to manipulate the KFbxEffector and KFbxControlSetLink.
  *  
  *  The KFbxControlSet class contains FK rig and IK rig animation. The IK rig is represented by a list of effectors which can be accessed with
  *  the KFbxEffector class with function KFbxControlSet::SetEffector() and KFbxControlSet::GetEffector(). The FK rig is represented by a list of nodes which can be 
  *  accessed with the KFbxControlSetLink class with function KFbxControlSet::SetControlSetLink() and KFbxControlSet::GetControlSetLink().
  *
  */
class KFBX_DLL KFbxControlSet
{

public:

	/** Reset to default values.
	  * Reset all the effector and control set links.
	  */
	void Reset();

	//! Set type.
	enum EType
	{
		eNone,
		eFkIk,
		eIkOnly
	};

	//! Set type.
	void SetType(EType pType);

	//! Get type.
	EType GetType();

	void SetUseAxis(bool pUseAxis);
	bool GetUseAxis();

	//! Set lock transform flag.
	void SetLockTransform(bool pLockTransform);

	//! Get lock transform flag.
	bool GetLockTransform();

	//! Set lock 3D pick flag.
	void SetLock3DPick(bool pLock3DPick);

	//! Get lock 3D pick flag.
	bool GetLock3DPick();

	/** Associate a control set link to a given character node ID.
      * \param pCharacterNodeId Character node ID.
	  * \param pControlSetLink Control set link.
	  * \return \c true on success, \c false otherwise.
	  * \remarks It is useless to set a control set link related either to
	  * eCharacterReference, eCharacterLeftFloor, eCharacterRightFloor,
	  * eCharacterLeftHandFloor, eCharacterRightHandFloor,
	  * eCharacterProps0, eCharacterProps1, eCharacterProps2, 
	  * eCharacterProps3 or eCharacterProps4. None of these are part of
	  * a control set.
	  */
	bool SetControlSetLink(ECharacterNodeId pCharacterNodeId, const KFbxControlSetLink& pControlSetLink);

	/** Get a control set link associated with a given character node ID.
      * \param pCharacterNodeId ID of character node requested.
	  * \param pControlSetLink Optional pointer to receive control set link if function succeeds.
	  * \return \c true on success, \c false otherwise.
	  * \remarks It is useless to get a character link related either to
	  * eCharacterReference, eCharacterLeftFloor, eCharacterRightFloor,
	  * eCharacterLeftHandFloor, eCharacterRightHandFloor,
	  * eCharacterProps0, eCharacterProps1, eCharacterProps2, 
	  * eCharacterProps3 or eCharacterProps4. None of these are part of
	  * a control set.
	  */
	bool GetControlSetLink(ECharacterNodeId pCharacterNodeId, KFbxControlSetLink* pControlSetLink = NULL);

	/** Associate an effector node to a given effector node ID.
      * \param pEffectorNodeId Effector node ID.
	  * \param pEffector Associated effector.
	  * \return \c true on success, \c false otherwise.
	  */
	bool SetEffector(EEffectorNodeId pEffectorNodeId, KFbxEffector pEffector);

	/** Get an effector associated with a given effector node ID.
      * \param pEffectorNodeId ID of effector node requested.
	  * \param pEffector Optional pointer to receive effector if function succeeds.
	  * \return \c true on success, \c false otherwise.
	  */
	bool GetEffector(EEffectorNodeId pEffectorNodeId, KFbxEffector* pEffector = NULL);
	
	/** Associate an auxiliary effector node to a given effector node ID.
      * \param pEffectorNodeId Effector node ID.
	  * \param pNode Associated node.
	  * \return \c true on success, \c false otherwise.
	  */
	bool SetEffectorAux(EEffectorNodeId pEffectorNodeId, KFbxNode* pNode,EEffectorSetId pEffectorSetId = eEffectorSetAux1);

	/** Get an auxiliary effector associated with a given effector node ID.
      * \param pEffectorNodeId ID of effector node requested.
	  * \param pNode Optional pointer to receive node if function succeeds.
	  * \return \c true on success, \c false otherwise.
	  */
	bool GetEffectorAux(EEffectorNodeId pEffectorNodeId, KFbxNode** pNode = NULL,EEffectorSetId pEffectorSetId = eEffectorSetAux1);

	//! Get name associated with effector node ID.
	static char* GetEffectorNodeName(EEffectorNodeId pEffectorNodeId);

	/** Get ID associated with effector node name.
	  * \remarks If no effector node with given name exists, -1 is returned.
	  */
	static EEffectorNodeId GetEffectorNodeId(char* pEffectorNodeName);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS	

private:

	KFbxControlSet();
	~KFbxControlSet();

	KFbxCharacter* mCharacter;
	EType mType;
	bool mUseAxis;
	bool mLockTransform;
	bool mLock3DPick;
	KFbxControlSetLink mControlSetLink[eCharacterLastNodeId]; // Except floor node IDs!
	KFbxEffector mEffector[eEffectorLastNodeId];
	KFbxNode* mEffectorAux[eEffectorLastNodeId][eLastEffectorSetId-1];

	friend class KFbxCharacter;
	friend class KFbxNode;

public:

	void FromPlug(KFbxControlSetPlug *pPlug);
	void ToPlug(KFbxControlSetPlug *pPlug);

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};


class KFBX_DLL KFbxControlSetPlug : public KFbxTakeNodeContainer
{

	public:
		/**
		  * \name Error Management
		  * The same error object is shared among instances of this class.
		  */
		//@{

		/** Retrieve error object.
		  *	\return Reference to error object.
		  */
		KError& GetError();

		//! Error identifiers.
		typedef enum 
		{
			eERROR,
			eERROR_COUNT
		} EError;	

		/** Get last error code.
		  *	\return Last error code.
		  */
		EError GetLastErrorID();

		/** Get last error string.
		  *	\return Textual description of the last error.
		  */
		char* GetLastErrorString();

		//@}

		virtual KFbxObject::ENameSpace GetNameSpace() const;
		virtual KObject*  GetPropertyHandler();

	protected:
		KFbxControlSetPlug(KFbxSdkManager& pManager, char* pName);
		virtual ~KFbxControlSetPlug();

		void        Destruct();

		KString		GetTypeName();
		KStringList	GetTypeFlags();

		virtual void AddChannels(KFbxTakeNode *pTakeNode);
		virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);

		KFbxControlSetPlugProperties* mPH;

		KError mError;

		friend class KFbxReaderFbx;
		friend class KFbxWriterFbx;
		friend class KFbxWriterFbx6;
		friend class KFbxSdkManager;
		friend class KFbxScene;
};

typedef KFbxControlSetPlug* HKFbxControlSetPlug;

} // namespace
#endif // #ifndef _KFbxControlSet_h


