#ifndef _KFbxExporter_h
#define _KFbxExporter_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/


#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kfbxio/kfbxio.h>

#include <klib/kstring.h>
#include <klib/karrayul.h>

#ifdef KARCH_DEV_MACOSX_CFM
	#include <CFURL.h>
	#include <Files.h>
#endif

class IFbxObject;
class KStreamOptions;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxScene;
class KFbxExporterImp;
class KStreamOptionsFbx;


/** Class to export SDK objects into an FBX file.
  * \nosubgrouping
  *	Typical way to use:
  *		- create an exporter
  *		- initialize it with a file name
  *	    - set numerous states, take information, defining how the exporter will behave
  *		- call KFbxExporter::Export() with the entity to export
  */
class KFBX_DLL KFbxExporter : public KFbxIO
{

public:

	/** 
	  * \name Export Functions
	  */
	//@{

	/** Initialize object.
	  *	\param pFileName Name of file to access.
	  *	\return \c true on success, \c false otherwise.
	  * To identify the error that occured, call KFbxIo::GetLastErrorID().
	  */
	virtual bool Initialize(const char *pFileName);

#ifdef KARCH_DEV_MACOSX_CFM
    virtual bool Initialize(const FSSpec &pMacFileSpec);
    virtual bool Initialize(const FSRef &pMacFileRef);
    virtual bool Initialize(const CFURLRef &pMacURL);
#endif

    /** Get file export options settings.
	  *	\return Pointer to file export options or NULL on failure.
	  * \remarks Caller gets ownership of the returned structure.
      */
	KStreamOptions* GetExportOptions();	
    
	/** Export the scene to the currently created file.
      * \param pScene Scene to export.
	  * \param pStreamOptions Pointer to file export options.
	  *	\return \c true on success, \c false otherwise.
	  * To identify the error, call KFbxIo::GetLastErrorID().
      */
    bool Export(KFbxScene& pScene, KStreamOptions* pStreamOptions = NULL);

    /** Release the file export options. 
	  * \param pStreamOptions Pointer to file export options.
	  */
	void ReleaseExportOptions(KStreamOptions* pStreamOptions);

	//@}

	/** Set the password.
	  * All subsequently exported files are protected with the given password.
	  * A \c NULL password string disables password protection.
      * \param pPassword Password string.
	  *	\remarks This function must be called after KFbxImporter::Initialize().
      */
    void SetPassword(char* pPassword);

	/** 
	  * \name Export State Functions
	  * \remarks Only some parts of the scene can be exported by setting the export states.
	  */
	//@{

	/** \enum EExportState Export section identifiers.
	  - \e eEXPORT_MODEL
	  - \e eEXPORT_MATERIAL
	  - \e eEXPORT_TEXTURE
	  - \e eEXPORT_TEXTURE_AS_EMBEDDED
	  - \e eEXPORT_LINK
	  - \e eEXPORT_SHAPE
	  - \e eEXPORT_GOBO
	  - \e eEXPORT_ANIMATION
	  - \e eEXPORT_CHARACTER
	  - \e eEXPORT_GLOBAL_SETTINGS
	  - \e eEXPORT_PIVOT
	  */
	typedef enum 
	{
		eEXPORT_MODEL,
		eEXPORT_MATERIAL,
		eEXPORT_TEXTURE,
		eEXPORT_TEXTURE_AS_EMBEDDED,
		eEXPORT_LINK,
		eEXPORT_SHAPE,
		eEXPORT_GOBO,
		eEXPORT_ANIMATION,
		eEXPORT_CHARACTER,
		eEXPORT_GLOBAL_SETTINGS,
		eEXPORT_PIVOT,
		eEXPORT_STATE_COUNT
	} EExportState;

    /** Set the value of an export state.
	  * \param pState Export section identifier.
	  * \param pValue State of the flag. If set to \c true, the corresponding section
	  * is processed for export. Otherwise it is skipped.
	  */
    void SetState(EExportState pState, bool pValue);

    /** Get the value of an export state.
	  * \param pState Export section identifier.
	  * \return The current value of the Export section identifier.
	  */
    bool GetState(EExportState pState);

	/**	Disable password protection and reset all the export states to \c true, except eEXPORT_TEXTURE_AS_EMBEDDED 
	  * which is set to \c false.
	  */
    void ResetConfiguration();


#ifndef K_FBXSDK
	#include <kfbxio/kfbxexporterinhouse.h>
#else
	typedef enum 
	{
		eFBX_BINARY,
		eFBX_ASCII,
		eFBX_ENCRYPTED,
		eFBX_50_BINARY,
		eFBX_50_ASCII,
		eAUTOCAD_DXF,		
		e3D_STUDIO_3DS,
		eALIAS_OBJ,
		eFILE_FORMAT_COUNT
	} EFileFormat;
#endif

	/** Set the exported file format.
	  *	\param pFileFormat	File format identifier.
	  */
	void SetFileFormat(EFileFormat pFileFormat);

	/** Get the format of the exported file.
	  *	\return File format identifier.
	  */
	EFileFormat GetFileFormat();

	/** Return the number of exportable file formats.
	  *	\return Number of supported exportable file formats.
	  * \remark Multiple identifiers for the same format are counted as 
	  * a file format. For instance, eFBX_BINARY, eFBX_ASCII and eFBX_ENCRIPTED
	  * count for three file formats.
	  */
	static int GetFileFormatCount() { return eFILE_FORMAT_COUNT; }

	/** Get the textual description of the file format that can be used in the file selector.
	  *	\param pFileFormat	File format identifier.
	  *	\return Pointer to the textual description of the format.
	  */
	static char* GetFileFormatDescription(EFileFormat pFileFormat);

	/** Get the file extension in a readable format.
	  *	\param pFileFormat	File format identifier.
	  *	\return Pointer to the character representation of the file extension.
	  */
	static char* GetFileFormatExtension(EFileFormat pFileFormat);

	/** Set the file mode to binary.
	  * This method is provided for convenience. The same results can be achieved
	  * using the SetFileFormat directly. In fact, if the passed argument is \c true,
	  * this method is equivalent to the call SetFileFormat(eFBX_BINARY) and if the
	  * passed argument is \c false, the call is equivalent to SetFileFormat(eFBX_ASCII).
	  * 
	  * \param pBinary Set the file mode to binary.
	  */
	void SetFileModeBinary(bool pBinary);

	/** Get file mode.
	  * \return \c true if file mode is set to binary.
	  */
	bool IsFileModeBinary();
	//@}


///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

    /** Get file export option settings.
      * \param pFbxObject Target FBX file.
	  *	\return Pointer to stream export options or NULL on failure.
	  * \remarks Caller gets ownership of the returned structure.
      */
	KStreamOptions* GetExportOptions(IFbxObject* pFbxObject);	

    /** Export the scene to a FBX file.
      * \param pScene Scene to export.
	  * \param pStreamOptions Pointer to stream export options, not publicly available yet.
      * \param pFbxObject Target FBX file.
	  *	\return \c true on success, \c false otherwise.
	  * In the last case, call KFbxIo::GetLastErrorID() to identify the error.
      */
	bool Export(KFbxScene& pScene, KStreamOptions* pStreamOptions, IFbxObject* pFbxObject);

protected:

	KFbxExporter(KFbxSdkManager& pManager);
	virtual ~KFbxExporter();

	void Reset();

	bool FileCreatePart1();
	bool FileCreatePart2();
	void FileClose();

	void SetExportOptionsInfo(KStreamOptionsFbx* pStreamOptionsFbx);

	KFbxExporterImp* mImp;

	KString mPassword;
	bool mExportStates[eEXPORT_STATE_COUNT];

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxExporter_h


