#ifndef _KFbxQuaternion_h
#define _KFbxQuaternion_h

/**************************************************************************************

 Copyright  2000 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/



#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

/**	FBX SDK quaternion class.
  * \nosubgrouping
  */
class KFBX_DLL KFbxQuaternion
{

public:

	/**
	  * \name Constructors and Destructor
	  */
	//@{

	//! Constructor.
	KFbxQuaternion();

	//! Copy constructor.
	KFbxQuaternion(const KFbxQuaternion& pV);

	/** Constructor.
	  * \param pX The X component.
	  * \param pY The Y component.
	  * \param pZ The Z component.
	  * \param pW The W component.
	  */
	KFbxQuaternion(double pX, double pY, double pZ, double pW = 1.0);
	
	//! Destructor.
	~KFbxQuaternion();

	//@}

	/**
	  * \name Access
	  */
	//@{
	
	//! Assignment operation.
	KFbxQuaternion& operator=(const KFbxQuaternion& pQuaternion);
	
	/** Accessor.
	  * \param pIndex The index of the component to access.
	  * \remark The parameter is not checked for value out of bounds. The valid
	  * range is [0,3].
	  * \return The reference to the indexed component.
	  */
	double& operator[](int pIndex);

	/** Get a vector element.
	  * \param pIndex The index of the component to access.
	  * \remark The parameter is not checked for values out of bounds. The valid
	  * range is [0,3].
	  * \return The value of the indexed component.
	  */
	double GetAt(int pIndex);	

	/** Set a vector element.
      * \param pIndex The index of the component to set.
	  * \param pValue The new value to set the component.
	  * \remark The index parameter is not checked for values out of bounds. The valid
	  * range is [0,3].
	  */
	void SetAt(int pIndex, double pValue);

	/** Set vector.
	  * \param pX The X component value.
  	  * \param pY The Y component value.
	  * \param pZ The Z component value.
	  * \param pW The W component value.
	  */
	void Set(double pX, double pY, double pZ, double pW = 1.0);
	
	//@}

	/**
	  * \name Scalar Operations
	  */
	//@{
	
	/** Add a value to all vector components.
	  * \param pValue The value to add to each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxQuaternion operator+(double pValue);

	/** Subtract a value from all vector components.
	  * \param pValue The value to subtract from each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxQuaternion operator-(double pValue);

	/** Multiply a value to all vector components.
	  * \param pValue The value multiplying each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxQuaternion operator*(double pValue);

	/**	Divide all vector components by a value.
	  * \param pValue The value dividing each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxQuaternion operator/(double pValue);

	/** Add a value to all vector components.
	  * \param pValue The value to add to each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator+=(double pValue);

	/** Subtract a value from all vector components.
	  * \param pValue The value to subtract from each component of the vector.
  	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator-=(double pValue);

	/** Multiply a value to all vector elements.
	  * \param pValue The value multiplying each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator*=(double pValue);

	/**	Divide all vector elements by a value.
	  * \param pValue The value dividing each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator/=(double pValue);
	
	//@}

	/**
	  * \name Vector Operations
	  */
	//@{

	/**	Unary minus operator.
	  * \return The quaternion that is the negation of \this (each component is multiplied by -1).
	  */
	KFbxQuaternion operator-();
	
	/** Add two vectors together.
	  * \param pQuaternion Quaternion to add.
	  * \remark The values in pQuaternion are not checked.
	  * \return The quaternion v' = this + pQuaternion.
	  */
	KFbxQuaternion operator+(KFbxQuaternion& pQuaternion);

	/** Subtract a quaternion from another quaternion.
	  * \param pQuaternion Quaternion to subtract.
	  * \remark The values in pQuaternion are not checked.
	  * \return The quaternion v' = this - pQuaternion.
	  */
	KFbxQuaternion operator-(KFbxQuaternion& pQuaternion);

	/** Memberwise multiplication of two vectors.
	  * \param pQuaternion Multiplying quaternion.
	  * \remark The values in pQuaternion are not checked.
	  * \return The quaternion v' = this * pQuaternion.
	  */
	KFbxQuaternion operator*(KFbxQuaternion& pQuaternion);

	/** Memberwise division of a quaternion with another quaternion.
	  * \param pQuaternion Dividing quaternion.
	  * \remark The values in pQuaternion are not checked.
	  * \return The quaternion v[i]' = this[i] / pQuaternion[i].
	  */
	KFbxQuaternion operator/(KFbxQuaternion& pQuaternion);

	/** Add two quaternions together.
	  * \param pQuaternion Quaternion to add.
	  * \remark The values in pQuaternion are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator+=(KFbxQuaternion& pQuaternion);

	/** Subtract a quaternion from another vector.
	  * \param pQuaternion Quaternion to subtract.
	  * \remark The values in pQuaternion are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator-=(KFbxQuaternion& pQuaternion);

	/** Memberwise multiplication of two quaternions.
	  * \param pQuaternion Multiplying quaternion.
	  * \remark The values in pQuaternion are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator*=(KFbxQuaternion& pQuaternion);
	
	/** Memberwise division of a quaternion with another quaternion.
	  * \param pQuaternion Dividing quaternion.
	  * \remark The values in pQuaternion are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxQuaternion& operator/=(KFbxQuaternion& pQuaternion);
	
	//@}

	/**
	  * \name Boolean Operations
	  */
	//@{

	/**	Equivalence operator.
	  * \param pV The quaternion to be compared to \e this.
	  * \return \c true if the two quaternions are equal (each element is within a 1.0e-6 tolerance)
	  * and \c false otherwise.
	  */
	bool operator==(KFbxQuaternion& pV);
	
	/**	Non equivalence operator.
	  * \param pV The quaternion to be compared to \e this.
	  * \return \c false if the two quaternions are equal (each element is within a 1.0e-6 tolerance)
	  * and \c true otherwise.
	  */
	bool operator!=(KFbxQuaternion& pV);
	
	//@}

	/**
	  * \name Casting
	  */
	//@{

	//! Cast the vector in a double pointer.
	operator double* ();
	
	//@}

	// Vector data.
	double mData[4];

};

} // namespace
#endif // #ifndef _KFbxQuaternion_h


