#ifndef _KFbxVector2_h
#define _KFbxVector2_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/


#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>
#include <klib/karrayul.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

/**	FBX SDK 2-elements vector class.
  * \nosubgrouping
  */
class KFBX_DLL KFbxVector2
{

public:

	/**
	  * \name Constructors and Destructor
	  */
	//@{

	//! Constructor.
	KFbxVector2();

	//! Copy constructor.
	KFbxVector2(const KFbxVector2& pVector2);

	/** Constructor.
	  *	\param pX X component.
	  *	\param pY Y component.
	  */
	KFbxVector2(double pX, double pY);
	
	//! Destructor.
	~KFbxVector2();

	//@}

	/**
	  * \name Access
	  */
	//@{
	
	//! Assignment operation.
	KFbxVector2& operator=(const KFbxVector2& pVector2);
	
	/** Accessor.
	  * \param pIndex The index of the component to access.
	  * \remark The parameter is not checked for values out of bounds. The valid
	  * values are 0 and 1.
	  * \return The reference to the indexed component.
	  */
	double& operator[](int pIndex);

	/** Get a vector element.
	  * \param pIndex The index of the component to access.
	  * \remark The parameter is not checked for values out of bounds. The valid
	  * values are 0 and 1.
	  * \return The value of the indexed component.
	  */
	double GetAt(int pIndex);	

	/** Set a vector element.
      * \param pIndex The index of the component to set.
	  * \param pValue The new value to set the component.
	  * \remark The index parameter is not checked for values out of bounds. The valid
	  * values are 0 and 1.
	  */
	void SetAt(int pIndex, double pValue);

	/** Set vector.
	  * \param pX The X component value.
  	  * \param pY The Y component value.
	  */
	void Set(double pX, double pY);
	
	//@}

	/**
	  * \name Scalar Operations
	  */
	//@{

	/** Add a value to all vector components.
	  * \param pValue The value to add to each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxVector2 operator+(double pValue);

	/** Subtract a value from all vector components.
	  * \param pValue The value to subtract from each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxVector2 operator-(double pValue);

	/** Multiply a value to all vector components.
	  * \param pValue The value multiplying each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxVector2 operator*(double pValue);

	/**	Divide all vector components by a value.
	  * \param pValue The value dividing each component of the vector.
	  * \remark The passed value is not checked.
	  * \return New vector.
	  */
	KFbxVector2 operator/(double pValue);

	/** Add a value to all vector components.
	  * \param pValue The value to add to each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator+=(double pValue);

	/** Subtract a value from all vector components.
	  * \param pValue The value to subtract from each component of the vector.
  	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator-=(double pValue);

	/** Multiply a value to all vector elements.
	  * \param pValue The value multiplying each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator*=(double pValue);

	/**	Divide all vector elements by a value.
	  * \param pValue The value dividing each component of the vector.
	  * \remark The passed value is not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator/=(double pValue);
	
	//@}

	/**
	  * \name Vector Operations
	  */
	//@{

	/**	Unary minus operator.
	  * \return The vector that is the negation of \this.
	  */
	KFbxVector2 operator-();
	
	/** Add two vectors together.
	  * \param pVector Vector to add.
	  * \remark The values in pVector are not checked.
	  * \return The vector v' = this + pVector.
	  */
	KFbxVector2 operator+(KFbxVector2& pVector);

	/** Subtract a vector from another vector.
	  * \param pVector Vector to subtract.
	  * \remark The values in pVector are not checked.
	  * \return The vector v' = this - pVector.
	  */
	KFbxVector2 operator-(KFbxVector2& pVector);

	/** Memberwise multiplication of two vectors.
	  * \param pVector2 Multiplying vector.
	  * \remark The values in pVector are not checked.
	  * \return The vector v' = this * pVector.
	  */
	KFbxVector2 operator*(KFbxVector2& pVector);

	/** Memberwise division of a vector with another vector.
	  * \param pVector Dividing vector.
	  * \remark The values in pVector are not checked.
	  * \return The vector v[i]' = this[i] / pVector[i].
	  */
	KFbxVector2 operator/(KFbxVector2& pVector);

	/** Add two vectors together.
	  * \param pVector Vector to add.
	  * \remark The values in pVector are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator+=(KFbxVector2& pVector);

	/** Subtract a vector from another vector.
	  * \param pVector Vector to subtract.
	  * \remark The values in pVector are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator-=(KFbxVector2& pVector);

	/** Memberwise multiplication of two vectors.
	  * \param pVector Multiplying vector.
	  * \remark The values in pVector are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator*=(KFbxVector2& pVector);
	
	/** Memberwise division of a vector with another vector.
	  * \param pVector Dividing vector.
	  * \remark The values in pVector are not checked.
	  * \return \e this updated with the operation result.
	  */
	KFbxVector2& operator/=(KFbxVector2& pVector);

	/** Calculate the dot product of two vectors.
      * \remarks Being considered as a XYZ vector with a weight,
	  * only the first 3 elements are considered in this operation.
	  * \param pVector The second vector.
	  * \return The dot product value.
	  */
	double DotProduct(KFbxVector2& pVector);
	
	//@}

	/**
	  * \name Boolean Operations
	  */
	//@{

	/**	Equivalence operator.
	  * \param pVector The vector to be compared to \e this.
	  * \return \c true if the two vectors are equal (each element is within a 1.0e-6 tolerance)
	  * and \c false otherwise.
	  */
	bool operator==(KFbxVector2& pVector);
	
	/**	Non-equivalence operator.
	  * \param pVector The vector to be compared to \e this.
	  * \return \c false if the two vectors are equal (each element is within a 1.0e-6 tolerance)
	  * and \c true otherwise.
	  */
	bool operator!=(KFbxVector2& pVector);
	
	//@}

	/**
	  * \name Length
	  */
	//@{

	/** Get the vector's length.
	  * \return The mathematical length of the vector.
	  */
	double Length();

	/** Get the vector's length squared.
	  * \return The mathematical square length of the vector.
	  */
	double SquareLength();

	/** Find the distance between 2 vectors.
	  * \param pVector The second vector.
	  * \return The mathematical distance between the two vectors.
	  */
	double Distance(KFbxVector2& pVector);

	//! Normalize the vector, length set to 1.
	void Normalize();

	//@}

	/**
	  * \name Casting
	  */
	//@{

	//! Cast the vector in a double pointer.
	operator double* ();

	//! Cast the vector in a const double pointer.
	operator const double* ();

	operator const double* const();

	operator const double* () const;

	operator const double* const () const;

	//@}

	// Vector data.
	double mData[2];

};

typedef  class KFBX_DLL KArrayTemplate<KFbxVector2>				KArrayKFbxVector2;

} // namespace
#endif // #ifndef _KFbxVector2_h


