#ifndef _KFbxCameraSwitcher_h
#define _KFbxCameraSwitcher_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/karrayul.h>

#include <kfbxplugins/kfbxnodeattribute.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxTakeNode;
class KFbxSdkManager;

/**	This node attribute contains the properties of a camera switcher.
  * \nosubgrouping
  */
class KFBX_DLL KFbxCameraSwitcher : public KFbxNodeAttribute
{

public:

	//! Return the type of node attribute which is EAttributeType::eCAMERA_SWITCHER.
	virtual EAttributeType GetAttributeType();

	/**
	  * \name Default Animation Values
	  * \remarks This set of functions provides direct access to default
	  * animation values specific to a camera switcher. The default animation 
	  * values are found in the default take node of the associated node.
	  * Hence, these functions only work if the camera switcher has been 
	  * associated with a node.
	  *
	  * \par
	  * Camera indices start at 1. Out of range indices are clamped between 1 and the 
	  * number of cameras in the scene. The index of a camera refers to its order of 
	  * appearance when searching the node tree depth first. 
	  */
	//@{

	/** Get default camera index.
	  * \return Camera index. The return value is an integer between 1 and the number
	  * of cameras in the scene, or 0 if there are no default camera set in the camera switcher.
	  */
	int GetDefaultCameraIndex();

	/** Set default camera index.
	  * \param pIndex Id of the camera to set as default. This parameter has an integer
	  * scale from 1 to the number of cameras in the scene. Its default value is 1 if
	  * there is at least one camera in the camera switcher, 0 if there are none.
	  * No validation checks are made.
	  */
	void SetDefaultCameraIndex(int pIndex);

	//@}

	//! Assignment operator.
    KFbxCameraSwitcher& operator=(KFbxCameraSwitcher& pCameraIndex);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxCameraSwitcher(KFbxSdkManager& pManager);
	virtual ~KFbxCameraSwitcher();

	/** Add channels specific to a camera switcher attribute.
      * \param pTakeNode Take node to add specialized channels to.
	  */
	virtual void AddChannels(KFbxTakeNode *pTakeNode);

	/** Remove channels specific to a camera switcher attribute.
      * \param pTakeNode Take node to remove specialized channels from.
	  */
	virtual void RemoveChannels(KFbxTakeNode *pTakeNode);

	/**
	  *	Used to retrieve the KProperty list from an attribute
	  */
	virtual void FillPropertyList(void* pPropertyList);

	KString		GetTypeName();
	KStringList	GetTypeFlags();

    void AddCameraName(char* pCameraName);
    char* GetCameraName(kUInt pIndex);
    kUInt GetCameraNameCount();

    KArrayTemplate<KString*> mCameraNameList;


	friend class KFbxWriterFbx6;
	friend class KFbxReaderFbx;
	friend class KFbxSdkManager;
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxCameraSwitcher_h


