#ifndef _KFbxDeformer_h
#define _KFbxDeformer_h

/**************************************************************************************

 Copyright  2004 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxtakenodecontainer.h>
#include <kfbxplugins/kfbxlink.h>
#include <kfbxmath/kfbxmatrix.h>

#include <klib/kerror.h>

class KObject;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager; 
class KFbxDeformerProperties;

class KFBX_DLL KFbxDeformer : public KFbxTakeNodeContainer
{

	public:
		/**	Set multi layer state flag.
		  * \param pMultiLayer If \c true, multi-layering is enabled.
		  */
		void SetMultiLayer(bool pMultiLayer);

		/**	Get multilayer state.
		  * \return The state of the multy-layer flag.
	      */
		bool GetMultiLayer();

		/** \enum EDeformerType Deformer types.
		  * - \e eUNDEFINED
		  * - \e eSKIN
		  * - \e eCLUSTER
		  */
		typedef enum 
		{
			eUNDEFINED,
			eSKIN,
			eCLUSTER,
			eDEFORMER_COUNT
		} EDeformerType;	

		/** Get the type of the deformer.
		  * \return Deformer type identifier.
		  */
		EDeformerType  GetDeformerType();

		/**
		  * \name Error Management
		  */
		//@{

		/** Retrieve error object.
		  *	\return Reference to error object.
		  */
		KError& GetError();

		/** \enum EError Error identifiers.
		  * - \e eERROR
		  */
		typedef enum 
		{
			eERROR,
			eERROR_COUNT
		} EError;	

		/** Get last error code.
		  *	\return Last error code.
		  */
		EError GetLastErrorID();

		/** Get last error string.
		  *	\return Textual description of the last error.
		  */
		char* GetLastErrorString();

		//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

	protected:
		KFbxDeformer(KFbxSdkManager& pManager, char* pName, char* pTypeName);
		virtual ~KFbxDeformer();

		void        Destruct();

		KString		GetTypeName();
		KStringList	GetTypeFlags();

		KFbxDeformer& operator=(KFbxLink& pLink);

		virtual KFbxObject::ENameSpace GetNameSpace() const;

		static int GetClassID();

		// Entity manipulation
		bool SetEntity(KFbxEntity* pEntity);
		KFbxEntity* GetEntity();
		KFbxEntity* mEntity;
		void AddChannels(KFbxTakeNode *pTakeNode);

		// Properties Handler
		virtual KObject*          GetPropertyHandler();
		virtual KFbxUserProperty* CreateKFbxUserProperty(KProperty* pProperty);
		
		KFbxDeformerProperties* mPH;

		// Local
		KError mError;
		bool   mMultiLayer;
		EDeformerType mDeformerType;

		// Skin deformer
		double mLinkDeformAccuracy;

		// 	Cluster deformer
		KFbxLink::ELinkMode     mLinkMode;
		KString                 mUserDataID;
		KString                 mUserData;
		KArrayTemplate<int>    mControlPointIndices;
		KArrayTemplate<double> mControlPointWeights;
		KFbxMatrix              mTransform;
		KFbxMatrix              mTransformLink;
		KFbxMatrix              mTransformAssociate;
		KFbxMatrix              mTransformParent;
		bool                    mIsTransformParentSet;

		friend class KFbxReaderFbx;
		friend class KFbxWriterFbx;
		friend class KFbxWriterFbx6;
		friend class KFbxSdkManager;
		friend class KFbxEntity;
		friend class KFbxScene;
		friend class KFbxLink;
};

typedef KFbxDeformer* HKFbxDeformer;

} // namespace


#endif // _KFbxDeformer_h
