#ifndef _KFbxGeometryBase_h
#define _KFbxGeometryBase_h

/**************************************************************************************

 Copyright  2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/karrayul.h>

#include <kfbxmath/kfbxvector4.h>

#include <kfbxplugins/kfbxlayercontainer.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;

/** KFbxGeometryBase is the base class for managing Control Points. 
  * Use the KFbxGeometryBase class to manage control points for
  * mesh, nurbs, patch and normals (on Layer 0).
  * \nosubgrouping
  */
class KFBX_DLL KFbxGeometryBase : public KFbxLayerContainer
{

public:

	/**
	  * \name Control Points and Normals Management.
	  */
	//@{

	/** Allocate memory space for the array of control points.
	  * \param pCount The number of control points.
	  * \remarks Any previously allocated array of control points will be cleared.
      */
	void InitControlPoints(int pCount);

	/** Allocate memory space for the array of normals.
	  * If \c pCount is specified, the array will have the same size as \c pCount otherwise
	  * the array will have the same length as the array of control points. 
	  * This function must be called after function \c KFbxLayerContainer::InitControlPoints().
	  * \param pCount The desired size for the normal array.
      */
	void InitNormals(int pCount = 0 );

	/** Sets the control point and the normal values for a given index.
	  * \param pCtrlPoint The value of the control point.
	  * \param pNormal The value of the normal.
	  * \param pIndex The index of the control point/normal to be modified.
	  * \remarks If the arrays are not big enough to store the values at the given index, their 
	  * size will be increased.
	  */
	void SetControlPointAt(KFbxVector4 &pCtrlPoint , KFbxVector4 &pNormal , int pIndex);

	/** Get the number of control points.
	  * \return The number of control points allocated in the geometry.
	  */
	int GetControlPointsCount();


	/** Get a pointer to the array of control points.
	  * \returns Pointer to array of control points or \c NULL if the array
	  * has not been allocated yet. 
	  * \remarks Use the function \c KFbxGeometryBase::InitControlPoints() to allocate
	  * the array.
	  */
	KFbxVector4* GetControlPoints();

	/** Get a pointer to the array of normals.
	  * \returns Pointer to array of normals or \c NULL if the array
	  * hasn't been allocated yet.
	  * \remarks Use the function \c KFbxGeometryBase::InitNormals() to allocate
	  * the array.
      */
	KFbxVector4* GetNormals();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS


protected:

	KFbxGeometryBase(KFbxSdkManager& pManager);
	virtual ~KFbxGeometryBase();

	void Copy(KFbxGeometryBase& pGeometryBase);

	KArrayTemplate<KFbxVector4> mControlPoints;

	friend class KFbxGeometryConverter;
	friend class KFbxWriterFbx6;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxGeometryBase_h


