#ifndef _KFbxGeometryConverter_h
#define _KFbxGeometryConverter_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/karrayul.h>
#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxNode;
class KFbxLayerContainer;
class KFbxGeometryBase;
class KFbxGeometry;
class KFbxMesh;
class KFbxPatch;
class KFbxNurb;
class KFbxLink;
class KFbxSdkManager;
class KFBXSurfaceEvaluator;
class KFbxWeightedMapping;
class KFbxVector4;

//! Provides functions to triangulate and convert geometry node attributes.
class KFBX_DLL KFbxGeometryConverter
{

public:

	/** 
	  * \name Triangulation
	  */
	//@{

	/** Triangulate a mesh.
	  * \param pMesh Pointer to the mesh to triangulate.
	  * \return Pointer to the new triangulated mesh.
	  * \remarks This method creates a new mesh, leaving the source mesh unchanged.
	  */
	KFbxMesh* TriangulateMesh(KFbxMesh* pMesh);

	/** Triangulate a patch.
	  * \param pPatch Pointer to the patch to triangulate.
	  * \return Pointer to the new triangulated mesh.
	  * \remarks The links and shapes are also converted to fit the created mesh.
	  */
	KFbxMesh* TriangulatePatch(KFbxPatch* pPatch);

	/** Triangulate a nurb.
	  * \param pNurb Pointer to the nurb to triangulate.
	  * \return Pointer to the new triangulated mesh.
	  * \remarks The links and shapes are also converted to fit the created mesh.
	  */
	KFbxMesh* TriangulateNurb(KFbxNurb* pNurb);

	/** Triangulate a mesh, patch or nurb contained in a node in order to preserve 
	  * the related animation channels.
	  * \param pNode Pointer to the node containng the geometry to triangulate.
	  * \return \c true on success, \c false if the node attribute is not a mesh, 
	  * a patch or a nurb.
	  * \remarks The remarks relative to functions TriangulateMesh(), TriangulatePatch()
	  * and TriangulateNurb() are applicable.
	  */
	bool TriangulateInPlace(KFbxNode* pNode);

	//@}

	/** 
	  * \name Geometry Conversion
	  */
	//@{

	/** Convert from patch to nurb.
	  * \param pPatch Pointer to the patch to convert.
	  * \return Created nurb or \c NULL if conversion failed.
	  * \remarks The patch must be of type eBSPLINE, eBEZIER or eLINEAR.
	  */
	KFbxNurb* ConvertPatchToNurb(KFbxPatch *pPatch);

	/** Convert to nurb a patch contained in a node in order to preserve 
	  * the related animation channels.
	  * \param pNode Pointer to the node containing the patch.
	  * \return \c true on success, \c false if the node attribute is not a patch.
	  * \remarks The patch must be of type eBSPLINE, eBEZIER or eLINEAR.
	  */
	bool ConvertPatchToNurbInPlace(KFbxNode* pNode);

	//@}

	/** 
	  * \name Nurb UV and Links Swapping
	  */
	//@{

	/** Flip UV and/or links of a nurb.
	  * \param pNurb Pointer to the Source nurb.
	  * \param pSwapUV Set to \c true if the UVs have to be swapped.
	  * \param pSwapLink Set to \c true if the links control points indices have to be swapped.
	  * \return Created nurb or \c NULL if flipping failed.
	  */
	KFbxNurb* FlipNurb(KFbxNurb* pNurb, bool pSwapUV, bool pSwapLinks);

	//@}

	/** 
	  * \name Normals By Polygon Vertex Emulation
	  */
	//@{

	/** Emulate normals by polygon vertex mode for a mesh.
	  * \param pMesh Pointer to the mesh object.
	  * \return \c true on success, \c false if the number of normals in the 
	  * mesh and in its associated shapes don't match the number of polygon
	  * vertices.
	  * \remarks Since the FBX file format currently only supports normals by
	  * control points, this function duplicates control points to equal the 
	  * number of polygon vertices. The links and shapes are converted also.
	  * As preconditions, 1) polygons must have been created and 2) the number 
	  * of normals in the mesh and in its associated shapes must match the 
	  * number of polygon vertices.
	  */
	bool EmulateNormalsByPolygonVertex(KFbxMesh* pMesh);

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxGeometryConverter(KFbxSdkManager& pManager);
	~KFbxGeometryConverter();

	void InitializeWeightInControlPoints(KFbxGeometryBase* pGeometry);
	void InitializeWeightInNormals(KFbxLayerContainer* pLayerContainer);
	void TriangulateContinuousSurface
	( 
		KFbxMesh* pMesh, 
		KFBXSurfaceEvaluator* pSurface, 
		kUInt pPointCountX, 
		kUInt pPointCountY, 
		bool ClockWise = false
	);
	void CheckForZeroWeightInShape(KFbxGeometry *pGeometry);
	KFbxMesh* CreateMeshFromParametricSurface(KFbxGeometry* pGeometry);
	KFbxNurb* CreateNurbFromPatch(KFbxPatch* pPatch);
	void ConvertShapes(KFbxGeometry* pSource, 
					   KFbxGeometry* pDestination, 
					   KFBXSurfaceEvaluator* pEvaluator, 
					   int pUCount, 
					   int pVCount);
	void ConvertLinks(KFbxGeometry* pSource, 
					  KFbxGeometry* pDestination, 
					  KFbxWeightedMapping* pSourceToDestinationMapping);
	void ConvertLinks(KArrayTemplate<KFbxLink*>& pSourceLinks, 
					  int pSourceControlPointsCount,
					  KArrayTemplate<KFbxLink*>& pDestinationLinks, 
					  int pDestinationControlPointsCount,
					  KFbxWeightedMapping* pSourceToDestinationMapping);
	void BuildLinkToSourceMapping(KArrayTemplate<KFbxLink*>& pSourceLinks, 
								  KFbxWeightedMapping* pLinkToSourceMapping);
	void CheckLinkToSourceMapping(KFbxWeightedMapping* pLinkToSourceMapping);
	void ConvertLink(int pSourceLinkIndex, 
					 KFbxWeightedMapping* pLinkToSourceMapping,
					 KFbxWeightedMapping* pSourceToDestinationMapping,
					 KFbxLink* pDestinationLink);
	void DuplicateControlPoints(KArrayTemplate<KFbxVector4>& pControlPoints, 
		                        KArrayTemplate<int>& pPolygonVertices);
	void UpdatePolygonVertex(KFbxMesh *pNewMesh, KFbxMesh *pRefMesh,
							 int pCount, int pIndexCount, int* pNewIndex);
	void ClearPolygonVertex(KFbxMesh *pNewMesh);

	KFbxSdkManager* mManager;

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 

};

} // namespace
#endif // #ifndef _KFbxGeometryConverter_h


