#ifndef _KFbxLayerContainer_h
#define _KFbxLayerContainer_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxnodeattribute.h>
#include <kfbxplugins/kfbxlayer.h>

#include <klib/karrayul.h>

#include <kfbxmath/kfbxvector4.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;
class KFbxEntity;
class KFbxTexture;
class KFbxMaterial;


/** KFbxLayerContainer is the base class for managing Layers. 
  * The Layer Container manages the creation and destruction of
  * layers. A Layer contains Layer Element(s) of the following 
  * types: Normals, Materials, Polygon Groups, UVs, Vertex Color
  * and Textures. See KFbxLayerElement for more details.
  * \nosubgrouping
  */
class KFBX_DLL KFbxLayerContainer : public KFbxNodeAttribute
{

public:

	/** Return the type of node attribute.
	  * This class is pure virtual.
	  */
	virtual EAttributeType GetAttributeType() = 0;

	/**
	  * \name Layer Management 
	  */
	//@{

	/** Create a new layer on top of existing layers.
	  * \return Index of created layer or -1 if an error occured.
	  */
	int CreateLayer();

	//! Delete all layers.
    void ClearLayers();

	/** Get number of layers.
	 * \return Return the number of layers.
	 */
	int GetLayerCount();

	/** Get number of layers containing the specified layer element type.
	  * \param pType The requested Layer Element type.
	  * \return The number of layers containing a layer of type \e pType.
	  */
	int GetLayerCount(KFbxLayerElement::ELayerElementType pType);

	/** Get layer at given index.
	  *	\param pIndex Layer index.
	  * \return Pointer to layer or \c NULL if index is out of range.
	  */
	KFbxLayer* GetLayer(int pIndex);

	/** Get the n'th layer containing the specified layer element type.
	  *	\param pIndex Layer index.
	  * \param pType Layer element type.
	  * \return Pointer to the layer or \c NULL if \c pIndex is out of range for
	  *  the specified type.
	  */
	KFbxLayer* GetLayer(int pIndex, KFbxLayerElement::ELayerElementType pType);

	/**	Get the index of n'th layer containing the specified layer element type.
	  * \param pIndex Layer index of the specified type.
	  * \param pType Layer type.
	  * \return Index of the specified layer type or -1 if the layer is not found.
	  * \remarks The index returned is the position of the layer in the global array of layers.
	  * This index can then be used to call \c GetLayer(int pIndex).
	  */
	int GetLayerIndex(int pIndex, KFbxLayerElement::ELayerElementType pType);

	/** Convert the global index of the layer to a type-specific index.
	  * \param pGlobalIndex The index of the layer in the global array of layers.
	  * \param pType The type uppon which the typed index will be returned.
	  * \return Index of the requested layer element type or \c -1 if the layer
	  * element type is not found.
	  */
	int GetLayerTypedIndex(int pGlobalIndex, KFbxLayerElement::ELayerElementType pType);
	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

	int  GTC(kUInt i);
	void* GT (int  i,    kUInt l); 
	int  AT (void* t,    kUInt l);
	int  GTI(char* n, kUInt l);
	int  GMC(kUInt i);
	void* GM (int  i,    kUInt l);
	int  AM (void* m,    kUInt l);
	int  GMI(char* n, kUInt l);

protected:

	KFbxLayerContainer(KFbxSdkManager& pManager);
	virtual ~KFbxLayerContainer();

	void Copy(KFbxLayerContainer& pLayerContainer, KFbxSdkManager& pManager);
	void CopyLayers(KFbxLayerContainer* pLayerContainer);

	void SetEntity(KFbxEntity* pEntity);
	void UnsetEntity();

	KFbxEntity* mEntity;
	KArrayTemplate<KFbxLayer*> mLayerArray;

	friend class KFbxEntity;
	friend class KFbxGeometryConverter;
	friend class KFbxWriterFbx6;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

} // namespace
#endif // #ifndef _KFbxLayerContainer_h


