#ifndef _KFbxNodeAttribute_h
#define _KFbxNodeAttribute_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>
#include <klib/karrayul.h>
#include <klib/kstringlist.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxNode;
class KFbxTakeNode;
class KFbxSdkManager;
class KFbxUserProperty;

/**	This class is the base class to all types of node attributes.
  * \nosubgrouping
  *	A node attribute is the content of a node. A null node attribute is set 
  * by calling function KFbxNode::SetNodeAttribute() with a \c NULL pointer.
  */
class KFBX_DLL KFbxNodeAttribute 
{

public:

	// Node attribute types.
    typedef enum 
    {   
		eUNIDENTIFIED,
		eNULL,
		eMARKER,
	    eSKELETON, 
	    eMESH, 
	    eNURB, 
	    ePATCH, 
	    eCAMERA, 
		eCAMERA_SWITCHER,
	    eLIGHT,
		eOPTICAL_REFERENCE,
		eOPTICAL_MARKER,
		eCONSTRAINT
    } EAttributeType;

	/** Return the type of node attribute.
	  * \remarks This class is pure virtual.
	  */
	virtual EAttributeType GetAttributeType() = 0;

    /** Return the node this attribute is set to.
	  * \return Pointer to the node or \c NULL if the current attribute is not set to a node.
	  */
	KFbxNode* GetNode();

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxNodeAttribute(KFbxSdkManager& pManager);
	virtual ~KFbxNodeAttribute();

	//! Once it's set, the node should not change.
	virtual void SetNode(KFbxNode& pNode);
	virtual void UnsetNode();

	/** Add specialized channels to a newly created take node. 
	*   \param pTakeNode Take node to add specialized channels to.
    */
	virtual void AddChannels(KFbxTakeNode *pTakeNode);

	/** Remove specialized channels from a take node. 
	*   \param pTakeNode Take node to remove specialized channels from.
    */
	virtual void RemoveChannels(KFbxTakeNode *pTakeNode);

	/**
	  *	Used to retrive the KProperty list from an attribute.
	  */
	virtual void FillPropertyList(void* pPropertyList);

	virtual KString		GetTypeName();
	virtual KStringList	GetTypeFlags();

	//! Assignment operator.
    KFbxNodeAttribute& operator=(KFbxNodeAttribute& pAttribute);

	KFbxNode*                         mParentNode;
    KFbxSdkManager&                   mManager;

	friend class KFbxNode;
	friend class KFbxSdkManager;
	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
    
#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxNodeAttribute* HKFbxNodeAttribute;

} // namespace
#endif // #ifndef _KFbxNodeAttribute_h


