#ifndef _KFbxPatch_h
#define _KFbxPatch_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxgeometry.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxSdkManager;

/** A patch is a type of parametric geometry node attribute.
  * \nosubgrouping
  */
class KFBX_DLL KFbxPatch : public KFbxGeometry 
{

public:
    
	//! Return the type of node attribute which is EAttributeType::ePATCH.
	virtual EAttributeType GetAttributeType();

	//! Reset the patch to default values.
	void Reset();

	/**
	  * \name Patch Properties
	  */
	//@{

    /** Set surface mode.
	  * \param pMode Surface mode identifier (see class KfbxGeometry)
	  */
    void SetSurfaceMode(KFbxGeometry::ESurfaceMode pMode);

    /** Get surface mode.
	  * \return Currently set surface mode identifier.
	  */
    inline KFbxGeometry::ESurfaceMode GetSurfaceMode() {return mSurfaceMode;}
	
	/** \enum EPatchType Patch types.
	  * - \e eBEZIER
	  * - \e eBEZIER_QUADRIC
	  * - \e eCARDINAL
	  * - \e eBSPLINE
	  * - \e eLINEAR
	  */
    typedef enum 
    {
	    eBEZIER         = 0, 
	    eBEZIER_QUADRIC = 1, 
	    eCARDINAL       = 2, 
	    eBSPLINE        = 3, 
	    eLINEAR         = 4
    } EPatchType;

	/** Allocate memory space for the array of control points.
      * \param pUCount Number of control points in U direction.
      * \param pUType Patch type in U direction.
      * \param pVCount Number of control points in V direction.
      * \param pVType Patch type in V direction.
      */
	void InitControlPoints(int pUCount, EPatchType pUType, int pVCount, EPatchType pVType);
	
	/** Get number of control points in U direction.
	  * \return Number of control points in U.
	  */
	inline int GetUCount() {return mUCount;}
	
	/** Get number of control points in V direction.
	  * \return Number of control points in V.
	  */
	inline int GetVCount() {return mVCount;}
	
	/** Get patch type in U direction.
	  * \return Patch type identifier.
	  */
	inline EPatchType GetPatchUType() {return mUType;}
	
	/** Get patch type in V direction.
	  * \return Patch type identifier.
	  */
	inline EPatchType GetPatchVType () {return mVType;}

	/** Set step.
	  * The step is the number of divisions between adjacent control points.
      * \param pUStep Steps in U direction.
      * \param pVStep Steps in V direction.
      */
	void SetStep(int pUStep, int pVStep);

	/** Get the number of divisions between adjacent control points in U direction.
	  * \return Step value in U direction.
	  */
	inline int GetUStep() {return mUStep;}
	
	/** Get the number of divisions between adjacent control points in V direction.
	  * \return Step value in V direction.
	  */
	inline int GetVStep() {return mVStep;}

	/** Set closed flags.
      * \param pU \c true if patch is closed in U direction.
      * \param pV \c true if patch is closed in V direction.
      */
	void SetClosed(bool pU, bool pV);
	
	/** Get state of the U closed flag.
	  * \return \c true if patch is closed in U direction.
	  */
	inline bool GetUClosed() {return mUClosed;}
	
	/** Get state of the V closed flag.
	  * \return \c true if patch is closed in V direction.
	  */
	inline bool GetVClosed() {return mVClosed;}

	/** Set U capped flags.
      * \param pUBottom \c true if patch is capped at the bottom in the U direction.
      * \param pUTop \c true if patch is capped at the top in the U direction.
	  * \remarks Capping options are saved but not loaded by Motionbuilder because they
	  * are computed from the patch topography. 
      */
	void SetUCapped(bool pUBottom, bool pUTop);
	
	/** Get U capped bottom flag state.
	  * \return \c true if patch is capped at the bottom.
	  */
	inline bool GetUCappedBottom() {return mUCappedBottom;}
	
	/** Get U capped top flag state.
	  * \return \c true if patch is capped at the top.
	  */
	inline bool GetUCappedTop() {return mUCappedTop;}
	
	/** Set V capped flags.
      * \param pVBottom \c true if patch is capped at the bottom in the V direction.
      * \param pVTop \c true if patch is capped at the top in the V direction.
	  * \remarks Capping options are saved but not loaded by Motionbuilder because they
	  * are computed from the patch topography. 
      */
	void SetVCapped(bool pVBottom, bool pVTop);

	/** Get V capped bottom flag state.
	  * \return \c true if patch is capped at the bottom.
	  */
	inline bool GetVCappedBottom() {return mVCappedBottom;}
	
	/** Get V capped top flag state.
	  * \return \c true if patch is capped at the top.
	  */
	inline bool GetVCappedTop() {return mVCappedTop;}

	//@}

	//! Assignment operator.
    KFbxPatch& operator=(KFbxPatch& pPatch);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

	KFbxPatch(KFbxSdkManager& pManager);
	virtual ~KFbxPatch();

	KString		GetTypeName();
	KStringList	GetTypeFlags();

	EPatchType mUType, mVType;
	int mUCount, mVCount;
	int mUStep, mVStep;
	bool mUClosed, mVClosed;
	bool mUCappedBottom, mUCappedTop;
	bool mVCappedBottom, mVCappedTop;

    KFbxGeometry::ESurfaceMode mSurfaceMode;

	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxPatch* HKFbxPatch;

} // namespace
#endif // #ifndef _KFbxPatch_h


