#ifndef _KFbxSdkManager_h
#define _KFbxSdkManager_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <klib/karrayul.h>
#include <klib/kerror.h>

class KRenamingStrategy;
class KProperty;
class KObjectList;

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxGeometryConverter;
class KFbxImageConverter;
class KFbxScene;
class KFbxExporter;
class KFbxImporter;
class KFbxNode;
class KFbxMaterial;
class KFbxTexture;
class KFbxVideo;
class KFbxNodeAttribute;
class KFbxNull;
class KFbxMarker;
class KFbxCamera;
class KFbxCameraSwitcher;
class KFbxLight;
class KFbxMesh;
class KFbxPatch;
class KFbxNurb;
class KFbxSkeleton;
class KFbxOpticalReference;
class KFbxPose;
class KFbxThumbnail;
class KFbxLink;
class KFbxShape;
class KFbxUserProperty;
class KFbxSceneInfo;
class KFbxGenericNode;
class KFbxDeformer;
class KFbxConstraint;
class KFbxConstraintPosition;
class KFbxConstraintScale;
class KFbxConstraintRotation;
class KFbxConstraintParent;
class KFbxConstraintAim;
class KFbxConstraintSingleChainIK;

class KFbxLayer;
class KFbxTakeNode;

typedef  class KFBX_DLL KArrayTemplate<KFbxMaterial *>			KArrayKFbxMaterial;
typedef  class KFBX_DLL KArrayTemplate<KFbxTexture *>			KArrayKFbxTexture;
typedef  class KFBX_DLL KArrayTemplate<KFbxVideo *>				KArrayKFbxVideo;
typedef  class KFBX_DLL KArrayTemplate<KFbxLayer *>				KArrayKFbxLayer;
typedef  class KFBX_DLL KArrayTemplate<KFbxShape *>				KArrayKFbxShape;
typedef  class KFBX_DLL KArrayTemplate<KFbxLink *>				KArrayKFbxLink;
typedef  class KFBX_DLL KArrayTemplate<KFbxImporter *>			KArrayKFbxImporter;
typedef  class KFBX_DLL KArrayTemplate<KFbxExporter *>			KArrayKFbxExporter;
typedef  class KFBX_DLL KArrayTemplate<KFbxGeometryConverter *> KArrayKFbxGeometryConverter;
typedef  class KFBX_DLL KArrayTemplate<KFbxImageConverter *>    KArrayKFbxImageConverter;
typedef  class KFBX_DLL KArrayTemplate<KFbxNodeAttribute *>		KArrayKFbxNodeAttribute;
typedef  class KFBX_DLL KArrayTemplate<KFbxNode *>				KArrayKFbxNode;
typedef  class KFBX_DLL KArrayTemplate<KFbxScene *>				KArrayKFbxScene;
typedef  class KFBX_DLL KArrayTemplate<KFbxTakeNode *>			KArrayKFbxTakeNode;
typedef  class KFBX_DLL KArrayTemplate<KFbxThumbnail *>			KArrayKFbxThumbnail;





/** SDK object manager
  * The SDK manager is in charge of:
  *		- scene element allocation
  *		- scene element deallocation
  *		- scene element search and access
  * \par 
  * Upon destruction all objects allocated by the SDK manager and 
  * not explicitly destroyed are destroyed as well. A derived 
  * class can be defined to allocate and deallocate a set of 
  * specialized scene elements.
  * \nosubgrouping
  */
class KFBX_DLL KFbxSdkManager
{

public:
	/**
	  * \name FBX SDK Manager creation/destruction.
	  */
	//@{

	/** SDK manager allocation method.
	  *	\return A pointer to the SDK manager or \c NULL if this is an
	  * evaluation copy of the FBX SDK and it is expired.
	  */
	static KFbxSdkManager* CreateKFbxSdkManager();	

	/** Destructor.
	  *	Deallocates all object previously created by the SDK manager.
	  */
	virtual ~KFbxSdkManager();
	//@}

	/**
	  * \name Object Creation
	  * This section contains creation methods for the different objects
	  * of the FBX SDK. Objects of the FBX SDK can be derived. The derived
	  * object can be properly created by overwritting the corresponding
	  * virtual allocator routine. See the protected section below.
	  */
	//@{

	/**	Create an object of class KFbxScene.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxScene* CreateKFbxScene();

	/**	Create an object of class KFbxNode.
	  *	\param pName Name of the node.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxNode* CreateKFbxNode(char* pName);

	/**	Create an object of class KFbxNull.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxNull* CreateKFbxNull();

	/**	Create an object of class KFbxMarker.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxMarker* CreateKFbxMarker();

	/**	Create an object of class KFbxCamera.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxCamera* CreateKFbxCamera();

	/**	Create an object of class KFbxLight.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxLight* CreateKFbxLight();

	/**	Create an object of class KFbxSkeleton.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxSkeleton* CreateKFbxSkeleton();

	/**	Create an object of class KFbxMesh.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxMesh* CreateKFbxMesh();

	/**	Create an object of class KFbxPatch.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxPatch* CreateKFbxPatch();

	/**	Create an object of class KFbxNurb.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxNurb* CreateKFbxNurb();

	/**	Create an object of class KFbxOpticalReference.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxOpticalReference* CreateKFbxOpticalReference();

	/**	Create an object of class KFbxLink.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxLink* CreateKFbxLink();

	/**	Create an object of class KFbxMaterial.
	  * \param pName Name of the material.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxMaterial* CreateKFbxMaterial(char* pName);	

	/**	Create an object of class KFbxShape.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxShape* CreateKFbxShape();

	/**	Create an object of class KFbxTexture.
	  * \param pName Name of the texture.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxTexture* CreateKFbxTexture(char* pName);

	/**	Create an object of class KFbxVideo.
	  * \param pName Name of the video.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxVideo* CreateKFbxVideo(char* pName);

	/** Create an object of class KFbxPose.
	  * \param pName Name of the pose.
	  * \param pIsBindPose True to indicate that this pose is a
	  * bind pose. If false, the pose is a rest pose.
	  *	\return Pointer to object created or \c NULL if allocation failed.
	  */
	KFbxPose* CreateKFbxPose(char* pName, bool pIsBindPose);

	/**	Create an object of type KFbxUserProperty
	  * \param pName The name used by the FBX SDK to identify the
	  * property internally. 
	  * \param pTypeName This string identifies the data type for the property.
	  * Valid values are "Bool", "Real", "Color", "Integer" and "Vector".
	  * \param pLabel The name used in the user interface for this property.
	  * \return Pointer to object if the creation was successful or \c NULL otherwise.
	  * \remarks The pLabel parameter is defined for backward compatibility. Older FBX files
      * saved it along with the other attributes. Now, the label attribute exists only while
      * the property object is in memory and is lost when saving to the FBX file.
	  */
	KFbxUserProperty* CreateKFbxUserProperty(char* pName, char* pTypeName, char* pLabel);	

	/**	Create an object of class KFbxGenericNode.
	  * \param pName Name of the node.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxGenericNode* CreateKFbxGenericNode(char* pName);

	/**	Create an object of class KFbxThumbnail
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxThumbnail* CreateKFbxThumbnail();

	/**	Create an object of class KFbxSceneInfo.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxSceneInfo* CreateKFbxSceneInfo();

	/**	Create an object of class KFbxGeometryConverter.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxGeometryConverter* CreateKFbxGeometryConverter();

	/**	Create an object of class KFbxImageConverter.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxImageConverter* CreateKFbxImageConverter();

	/**	Create an object of class KFbxExporter.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxExporter* CreateKFbxExporter();

	/**	Create an object of class KFbxImporter.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxImporter* CreateKFbxImporter();

	/**	Create an object of class KFbxConstraint of a particular type.
	  * \param pName Name of the constraint.
	  * \param pType Type of the constraint.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	KFbxConstraint* CreateKFbxConstraint(char* pName, int pType );

	//@}

	/** Add a prefix to a name 
	  * \param pPrefix The prefix to be added to the \c pName. This
	  * string must contain the "::" characters in order to be considered
	  * as a prefix.
	  * \param pName The name to be prefix.
	  * \return The prefixed string
	  * \remarks If a prefix already exists, it is removed before
	  * adding \c pPrefix.
	  */
	static KString PrefixName(char* pPrefix, char* pName);

protected:

	/**
	  * \name Object Allocation
	  *	Each creation method declared above calls for an associated allocation 
	  * method declared in this section. The allocation methods can be 
	  * overridden by a child class. 
	  */

	/**	Allocate an object of class KFbxScene.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxScene* AllocateKFbxScene();

	/**	Allocate an object of class KFbxNode.
	  *	\param pName Name of the node.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxNode* AllocateKFbxNode(char* pName);

	/**	Allocate an object of class KFbxNull.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxNull* AllocateKFbxNull();

	/**	Allocate an object of class KFbxMarker.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxMarker* AllocateKFbxMarker();

	/**	Allocate an object of class KFbxCamera.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxCamera* AllocateKFbxCamera();

	/**	Allocate an object of class KFbxLight.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxLight* AllocateKFbxLight();

	/**	Allocate an object of class KFbxSkeleton.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxSkeleton* AllocateKFbxSkeleton();

	/**	Allocate an object of class KFbxMesh.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxMesh* AllocateKFbxMesh();

	/**	Allocate an object of class KFbxPatch.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxPatch* AllocateKFbxPatch();

	/**	Allocate an object of class KFbxNurb.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxNurb* AllocateKFbxNurb();

	/**	Allocate an object of class KFbxOpticalReference.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxOpticalReference* AllocateKFbxOpticalReference();

	/**	Allocate an object of class KFbxLink.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxLink* AllocateKFbxLink();

	/**	Allocate an object of class KFbxMaterial.
	  * \param pName Name of the material.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxMaterial* AllocateKFbxMaterial(char* pName);	

	/**	Allocate an object of class KFbxShape.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxShape* AllocateKFbxShape();

	/**	Allocate an object of class KFbxTexture.
	  * \param pName Name of the texture.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxTexture* AllocateKFbxTexture(char* pName);

	/**	Allocate an object of class KFbxVideo.
	  * \param pName Name of the video.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxVideo* AllocateKFbxVideo(char* pName);

	/** Allocate an object of class KFbxPose.
	  * \param pName Name of the pose.
	  * \param pIsBindPose True to indicate that this pose is a
	  * bind pose. If false, the pose is a rest pose.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxPose* AllocateKFbxPose(char* pName, bool pIsBindPose);

	/** Allocate an object of class KFbxUserProperty.
	  * \param pName The name used by the FBX SDK to identify the
	  * property internally. 
	  * \param pTypeName This string identifies the data type for the property.
	  * Valid values are "Bool", "Real", "Color", "Integer" and "Vector".
	  * \param pLabel The name used in the user interface for this property.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  * \remarks The pLabel parameter is defined for backward compatibility. Older FBX files
      * saved it along with the other attributes. Now, the label attribute exists only while
      * the property object is in memory and is lost when saving to the FBX file.
	  */
	virtual KFbxUserProperty* AllocateKFbxUserProperty(char* pName, char* pTypeName, char* pLabel);

	/** Allocate an object of class KFbxUserProperty.
	  * \remarks This method is reserved for internal use only.
	  */
	virtual KFbxUserProperty* AllocateKFbxUserProperty(KProperty* pProperty);

	/**	Allocate an object of class KFbxGenericNode.
	  * \param pName Name of the node.
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	virtual KFbxGenericNode*  AllocateKFbxGenericNode(char* pName);

	/**	Allocate an object of class AllocateKFbxDeformer.
	  * \param pName Name of the deformer.
	  * \param pTypeName Type of deformer. Valid values are "Cluster" and "Skin".
	  *	\return Pointer to object created or \c NULL if creation failed.
	  */
	virtual KFbxDeformer*     AllocateKFbxDeformer(char* pName, char* pTypeName);

	/** Allocate an object of class KFbxThumbnail
	  * \return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxThumbnail* AllocateKFbxThumbnail();

	/** Allocate an object of class KFbxSceneInfo
	  * \return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxSceneInfo* AllocateKFbxSceneInfo();

	/**	Allocate an object of class KFbxGeometryConverter.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxGeometryConverter* AllocateKFbxGeometryConverter();

	/**	Allocate an object of class KFbxImageConverter.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxImageConverter* AllocateKFbxImageConverter();

	/**	Allocate an object of class KFbxExporter.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxExporter* AllocateKFbxExporter();

	/**	Allocate an object of class KFbxImporter.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxImporter* AllocateKFbxImporter();

	/**	Allocate an object of class KFbxConstraintPosition.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintPosition* AllocateKFbxConstraintPosition(char* pName);

	/**	Allocate an object of class KFbxConstraintRotation.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintRotation* AllocateKFbxConstraintRotation(char* pName);

	/**	Allocate an object of class KFbxConstraintScale.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintScale* AllocateKFbxConstraintScale(char* pName);

	/**	Allocate an object of class KFbxConstraintParent.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintParent* AllocateKFbxConstraintParent(char* pName);

	/**	Allocate an object of class KFbxConstraintSingleChainIK.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintSingleChainIK* AllocateKFbxConstraintSingleChainIK(char* pName);

	/**	Allocate an object of class KFbxConstraintAim.
	  * \param pName Name of the constraint.
	  *	\return Pointer to object allocated or \c NULL if allocation failed.
	  */
	virtual KFbxConstraintAim* AllocateKFbxConstraintAim(char* pName);

	//@}

public:

	/**
	  * \name Object Destruction
	  */
	//@{

	/**	Destroy an object of class KFbxScene.
	  *	\param pScene Pointer to the scene to destroy.
	  *	\param pDestroyContent Flag if scene content is to be destroyed as well.
	  *	Note that object designed to be multiple assigned (such as materials, 
	  * textures and gobos) won't be destroyed in the process.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxScene(KFbxScene* pScene, bool pDestroyContent = true);

	/**	Destroy an object of class KFbxNode.
	  *	\param pNode Pointer to the node to destroy.
	  *	\param pDestroyContent Flag if node content is to be destroyed as well.
	  *	Note that objects that are designed to be multiple assigned (such as materials, 
	  * textures and gobos) won't be destroyed in the process.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxNode(KFbxNode* pNode, bool pDestroyContent = true);

	/**	Destroy an object of class KFbxNodeAttribute.
	  *	\param pNodeAttribute Pointer to the node attribute to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxNodeAttribute(KFbxNodeAttribute* pNodeAttribute);

	/**	Destroy an object of class KFbxLink.
	  *	\param pLink Pointer to the link to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxLink(KFbxLink* pLink);

	/**	Destroy an object of class KFbxMaterial.
	  *	A material is designed to be multipl assigned (sharing) to different nodes.
	  *	To get rid of a material, it must be explicitly destroyed.
	  *	Note that all remaining materials are deallocated when the SDK Manager is deleted.
	  *	\param pMaterial Pointer to the material to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxMaterial(KFbxMaterial* pMaterial);	

	/**	Destroy an object of class KFbxShape.
	  *	\param pShape Pointer to the shape to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxShape(KFbxShape* pShape);

	/**	Destroy an object of class KFbxTexture.
	  *	A texture is designed to be multiple assigned (sharing) to different nodes.
	  *	To get rid of a texture, it must be explicitly destroyed.
	  *	Note that all remaining textures are deallocated when the SDK Manager is deleted.
	  *	\param pTexture Pointer to the texture to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxTexture(KFbxTexture* pTexture);

	/**	Destroy an object of class KFbxVideo.
	  *	A video is designed to be multiple assigned (sharing) to different textures.
	  *	To get rid of a video, it must be explicitly destroyed.
	  *	Note that all remaining video are deallocated when the SDK Manager is deleted.
	  *	\param pVideo Pointer to the video to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxVideo(KFbxVideo* pVideo);

	/**	Destroy an object of class KFbxPose.
	  *	\param pPose Pointer to the pose to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxPose(KFbxPose* pPose);

	/**	Destroy an object of class KFbxUserProperty.
	  *	\param pUserProperty Pointer to the user property to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxUserProperty(KFbxUserProperty* pUserProperty);

	/**	Destroy an object of class KFbxGenericNode.
	  *	\param pGenericNode Pointer to the node to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxGenericNode(KFbxGenericNode* pGenericNode);

	/**	Destroy an object of class KFbxDeformer.
	  *	\param pDeformer Pointer to the deformer to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxDeformer(KFbxDeformer* pDeformer);

	/** Destroy an object of class KFbxThumbnail.
	  * \param pThumbnail Pointer to the thumbnail to destroy.
	  * \return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxThumbnail(KFbxThumbnail* pThumbnail);

	/** Destroy an object of class KFbxSceneInfo.
	  * \param pSceneInfo Pointer to the SceneInfo to destroy.
	  * \return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual bool DestroyKFbxSceneInfo(KFbxSceneInfo* pSceneInfo);

	/**	Destroy an object of class KFbxGeometryConverter.
	  *	\param pConverter Pointer to the converter to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	bool DestroyKFbxGeometryConverter(KFbxGeometryConverter* pConverter);

	/**	Destroy an object of class KFbxImageConverter.
	  *	\param pConverter Pointer to the converter to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	bool DestroyKFbxImageConverter(KFbxImageConverter* pConverter);

	/**	Destroy an object of class KFbxExporter.
	  *	\param pExporter Pointer to the exporter to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	bool DestroyKFbxExporter(KFbxExporter* pExporter);
	
	/**	Destroy an object of class KFbxImporter.
	  *	\param pImporter Pointer to the importer to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	bool DestroyKFbxImporter(KFbxImporter* pImporter);

	/**	Destroy an object of class KFbxConstraint.
	  *	\param pConstraint Pointer to the constraint to destroy.
	  *	\return \c true if the operation succeeded, \c false otherwise.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	bool DestroyKFbxConstraint(KFbxConstraint* pConstraint);

	//@}

	/**
	  * \name Object Search
	  */
	//@{
	
	/**	Find  an object of class KFbxNode.
	  *	Find the pIndex-th node named \c pName listed the manager's array.
	  *	\param pName Name of the node.
	  *	\param pIndex Zero based index of the requested name.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxNode* FindKFbxNode(char* pName, int pIndex = 0);

	/**	Find  an object of class KFbxMaterial.
	  *	Find the pIndex-th material named \c pName listed the manager's array.
	  *	\param pName Name of the material.
	  *	\param pIndex Zero based index of the requested name.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxMaterial* FindKFbxMaterial(char* pName, int pIndex = 0);

	/**	Find  an object of class KFbxTexture.
	  *	Find the pIndex-th texture named \c pName listed the manager's array.
	  *	\param pName Name of the texture.
	  *	\param pIndex Zero based index of the requested name.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxTexture* FindKFbxTexture(char* pName, int pIndex = 0);

	/**	Find  an object of class KFbxVideo.
	  *	Find the pIndex-th video named \c pName listed the manager's array.
	  *	\param pName Name of the video.
	  *	\param pIndex Zero based index of the requested name.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxVideo* FindKFbxVideo(char* pName, int pIndex = 0);

	/**	Find  an object of class KFbxPose.
	  *	Find the pose named \c pName listed the manager's array.
	  *	\param pName Name of the pose.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxPose* FindKFbxPose(char* pName);

	/**	Find  an object of class KFbxUserProperty.
	  *	Find the user property \c pName listed the manager's array.
	  *	\param pName Name of the user property.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxUserProperty* FindKFbxUserProperty(char* pName);

	/**	Find  an object of class KFbxGenericNode.
	  *	Find the node \c pName listed the manager's array.
	  *	\param pName Name of the node.
	  *	\return Pointer to object found or \c NULL if object does not belong to this manager.
 	  * In the last case \c KFbxSdkManager::GetLastErrorId() returns \e eOBJECT_NOT_FOUND.
	  */
	virtual KFbxGenericNode* FindKFbxGenericNode(char* pName);

	//@}

	/**
	  * \name Object direct access
	  */
	//@{

	/** Get the number of objects of class KFbxNode.
	  *\return Return the number of KFbxNode objects owned by the manager.
	  */
	int GetKFbxNodeCount();

	/** Indexed access to the KFbxNode array.
	  * \param pIndex Index of the requested KFbxNode.
	  *	\return Node found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxNode* GetKFbxNode(int pIndex);

	/** Get the number of objects of class KFbxMaterial.
	  * \return Return the number of KFbxMaterial objects owned by the manager.
	  */
	int GetKFbxMaterialCount();

	/** Indexed access to the KFbxMaterial array.
	  * \param pIndex Index of the requested KFbxMaterial.
	  *	\return Material found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxMaterial* GetKFbxMaterial(int pIndex);

	/** Get the number of objects of class KFbxTexture.
	  * \return Return the number of KFbxTexture objects owned by the manager.
	  */
	int GetKFbxTextureCount();

	/** Indexed access to the KFbxTexture array.
	  * \param pIndex Index of the requested KFbxTexture.
	  *	\return Texture found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxTexture* GetKFbxTexture(int pIndex);

	/** Get the number of objects of class KFbxVideo.
	  * \return Return the number of KFbxVideo objects owned by the manager.
	  */
	int GetKFbxVideoCount();

	/** Indexed access to the KFbxVideo array.
      * \param pIndex Index of the requested KFbxVideo.
	  *	\return Video found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxVideo* GetKFbxVideo(int pIndex);

	/** Get the number of objects of class KFbxPose.
	  * \return Return the number of KFbxPose objects owned by the manager.
	  */
	int GetKFbxPoseCount();

	/** Indexed access to the KFbxPose array.
	  * \param pIndex Index of the requested KFbxPose.
	  *	\return Pose found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxPose* GetKFbxPose(int pIndex);

	/** Get the number of objects of class KFbxLink.
	  * \return Return the number of KFbxLink objects owned by the manager.
	  */
	int GetKFbxLinkCount();

	/** Indexed access to the KFbxLink array.
	  * \param pIndex Index of the requested KFbxLink.
	  *	\return Link found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxLink* GetKFbxLink(int pIndex);

	/** Get the number of all the objects of class KFbxConstraint.
	  * \return Return the number of KFbxConstraint objects owned by the manager.
	  */
	int GetKFbxConstraintCount();

	/** Get the number of objects of a type of class KFbxConstraint.
	  * \return Return the number of a KFbxConstraint objects type owned by the manager.
	  */
	int GetKFbxConstraintCount(int pType);

	/** Indexed access to the KFbxConstraint array.
	  * \param pIndex Index of the requested KFbxConstraint.
	  *	\return Link found or \c NULL if \c pIndex is outside bounds.
	  */
	KFbxConstraint* GetKFbxConstraint(int pIndex, int pType);

	//@}


	/**
	  * \name Error Management
	  */
	//@{

	/** Retrieve error object.
	 *	\return Reference to the Manager's error object.
	 */
	KError& GetError();

	/** \enum EError Error codes
	  */
	typedef enum 
	{
		eOBJECT_NOT_FOUND,    /**< The requested object was not found in the Manager's database. */
		eNAME_ALREADY_IN_USE, /**< A name clash occurred.                                        */
		eERROR_COUNT          /**< Mark the end of the error enum.                               */
	} EError;
	
	/** Get last error code.
	 *	\return Last error code.
	 */
	EError GetLastErrorID();

	/** Get last error string.
	 *	\return Textual description of the last error.
	 */
	char* GetLastErrorString();

	//@}

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

public:
	KFbxUserProperty*    CreateKFbxUserProperty(KProperty* pProperty);
	KFbxDeformer*        CreateKFbxDeformer(char* pName, char* pTypeName);
	KFbxCameraSwitcher*  CreateKFbxCameraSwitcher();

	void CreateMissingBindPoses(KFbxScene *pScene);
	int  GetBindPoseCount(KFbxScene *pScene);

protected:

	KFbxSdkManager();

	//! Clear the manager of all objects it has created.
	virtual void Clear();

	KArrayTemplate<KFbxScene*>             mSceneArray;
	KArrayTemplate<KFbxNode*>              mNodeArray;
	KArrayTemplate<KFbxNodeAttribute*>     mNodeAttributeArray;
	KArrayTemplate<KFbxLink*>              mLinkArray;
	KArrayTemplate<KFbxMaterial*>          mMaterialArray;
	KArrayTemplate<KFbxShape*>             mShapeArray;
	KArrayTemplate<KFbxTexture*>           mTextureArray;
	KArrayTemplate<KFbxVideo*>             mVideoArray;
	KArrayTemplate<KFbxPose*>              mPoseArray;
    KArrayTemplate<KFbxUserProperty*>      mUserPropertiesArray;
	KArrayTemplate<KFbxThumbnail*>         mThumbnailArray;
	KArrayTemplate<KFbxSceneInfo*>         mSceneInfoArray;
	KArrayTemplate<KFbxGenericNode*>       mGenericNodeArray;

	KObjectList*						   mObjectArray;

	KArrayTemplate<KFbxGeometryConverter*> mConverterArray;
	KArrayTemplate<KFbxImageConverter*>    mImageConverterArray;

	KArrayTemplate<KFbxExporter*>          mExporterArray;
	KArrayTemplate<KFbxImporter*>          mImporterArray;

	// The KRenamingStrategy pointers may be set by derived class
	// By default, they are null and not used.
	KRenamingStrategy* mNodeRenamingStrategy;
	KRenamingStrategy* mMaterialRenamingStrategy;
	KRenamingStrategy* mTextureRenamingStrategy;
	KRenamingStrategy* mVideoRenamingStrategy;

	KError mError;

	friend class KFbxReaderFbx;
	friend class KFbxWriterFbx;
	friend class KFbxWriterFbx6;
	friend class KFbxPose;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

typedef KFbxSdkManager* HKFbxSdkManager;

} // namespace
#endif // #ifndef _KFbxSdkManager_h


