#ifndef _KFbxThumbnail_h
#define _KFbxThumbnail_h

/**************************************************************************************

 Copyright  2004 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KFBX_DLL 
	#define KFBX_DLL K_DLLIMPORT
#endif

#include <kaydara.h>

#include <kfbxplugins/kfbxnamespacedef.h>
namespace FBXSDK_NAMESPACE {

class KFbxThumbnailMembers;

/** Simple class to hold RGBA values of a thumbnail image.
  * \nosubgrouping
  */
class KFBX_DLL KFbxThumbnail
{
public:
	/**
	  * \name Thumbnail properties
	  */
	//@{

	/** \enum EDataFormat Data format.
	  * - \e eRGB_24
	  * - \e eRGBA_32
	  */
	typedef enum 
	{
		eRGB_24, // 3 components
		eRGBA_32 // 4 components
	} EDataFormat;

	/** Set the data format.
	  * \param pDataFormat Data format identifier.
	  */
	void SetDataFormat(EDataFormat pDataFormat);

	/** Get the data format.
	  * \return Data format identifier for the thumbnail.
	  */
	EDataFormat GetDataFormat();


	/** \enum eIMageSize Image size.
	  * - \e eNOT_SET
	  * - \e e64x64
	  * - \e e128x128
	  */
	typedef enum 
	{
		eNOT_SET = 0,
		e64x64   = 64,
		e128x128 = 128
	} EImageSize;

	/** Set the thumbnail dimensions.
	  * \param pImageSize Image size identifier.
	  */
	void SetSize(EImageSize pImageSize);	

	/** Get the thumbnail dimensions.
	  * \return Image size identifier.
	  */
	EImageSize GetSize();

	/** Get the thumbnail dimensions in bytes.
	  * \return Thumbnail size in bytes.
	  */
	unsigned long GetSizeInBytes();


	//@}

	/**
	  * \name Thumbnail data access
	  */
	//@{

	/** Fill the thumbnail image.
	  * \param pImage Pointer to the image data. A copy
	  * of the image data will be made.
	  *	\remarks This pointer must point to a buffer region
	  * that is at least Width * Height * Component count
	  * bytes long. This pointer points to the upper left
	  * corner of the image.
	  * \remarks You must set the data format and the dimensions
	  * before calling this function.
	  * \return \c true if the thumbnail properties were set
	  * before calling this funtion. \c false otherwise.
	  */
	bool SetThumbnailImage(const kUByte* pImage);

	/** Get the thumbnail image.
	  * \return Pointer to the image data, or \c NULL if the
	  * thumbnail is empty.
	  */
	kUByte* GetThumbnailImage();

	//@}


///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////
#ifndef DOXYGEN_SHOULD_SKIP_THIS

protected:

    //! Constructor.
    KFbxThumbnail();

    //! Destructor.
    ~KFbxThumbnail();

	// Members
	KFbxThumbnailMembers* mMembers;

	friend class KFbxSdkManager;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS 
};

} // namespace
#endif // _KFbxThumbnail_h
