#ifndef _KName_h
#define _KName_h

/**************************************************************************************

 Copyright  2001 - 2005 Systmes Alias Qubec Inc. and/or its licensors.  
 All rights reserved.

 The coded instructions, statements, computer programs, and/or related material 
 (collectively the "Data") in these files contain unpublished information proprietary
 to Systmes Alias Qubec Inc. and/or its licensors, which is protected by Canadian
 and US federal copyright law and by international treaties.

 The Data may not be disclosed or distributed to third parties, in whole or in part,
 without the prior written consent of Alias Systems Corp. (Alias).

 THE DATA IS PROVIDED AS IS AND WITHOUT WARRANTY. ALL WARRANTIES ARE EXPRESSLY
 EXCLUDED AND DISCLAIMED. ALIAS MAKES NO WARRANTY OF ANY KIND WITH RESPECT TO THE DATA,
 EXPRESS, IMPLIED OR ARISING BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED
 WARRANTIES OF TITLE, NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR 
 PURPOSE. IN NO EVENT SHALL ALIAS, ITS AFFILIATES, PARENT COMPANIES, LICENSORS OR 
 SUPPLIERS (ALIAS GROUP) BE LIABLE FOR ANY DAMAGES OR EXPENSES OF ANY KIND INCLUDING
 WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER SPECIAL, DIRECT, INDIRECT, 
 EXEMPLARY, INCIDENTAL, OR CONSEQUENTIAL DAMAGES OF ANY KIND, WHETHER OR NOT ALIAS HAS
 BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS, HOWSOEVER CAUSED AND ON ANY THEORY OF
 LIABILITY. 

**************************************************************************************/

#include <kaydaradef.h>
#ifndef KBASELIB_DLL 
	#define KBASELIB_DLL K_DLLIMPORT
#endif

#include <klib/kstring.h>

/** Name class.
  *	Provides two stings, current and an initial name, for reversible
  * renaming.  This is especially useful for name clashing, renaming
  *	strategies and merging back to a former 3D scene using the initial
  *	names.
  */
class KBASELIB_DLL KName
{
public:
	
	/** Constructor.
      * \param pInitialName Name string used to initialize both members (initialName and currentName)
      * of this class.
      */
    KName(char* pInitialName = "");

	//! Copy constructor.
    KName(KName& pName);

    //!Destructor
	~KName();

	/** Set initial name.
      * \param pInitialName New string for the initial name.
	  *	\remarks The current name will also be changed to this value.
	  */
	void SetInitialName(char* pInitialName);

	/** Get initial name.
      * \return Pointer to the InitialName string buffer.
      */
	char* GetInitialName();

	/** Set current name.
      * \param pNewName New string for the current name.
      * \remarks The initial name is not affected.
      */
	void SetCurrentName(char* pNewName);

	/** Get current name.
      * \return Pointert to the CurrentName string buffer.
      */
	char* GetCurrentName();


	/** Check if the current name and internal name match.
      * \return \c true if current name isn't identical to initial name.
	  */
	bool IsRenamed();

	
	//! Assignment operator
	KName& operator= (KName& pName);

///////////////////////////////////////////////////////////////////////////////
//
//  WARNING!
//
//	Anything beyond these lines may not be documented accurately and is 
// 	subject to change without notice.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef DOXYGEN_SHOULD_SKIP_THIS

private:

	KString mInitialName;
	KString mCurrentName;

#endif // #ifndef DOXYGEN_SHOULD_SKIP_THIS

};

#endif // #define _KName_h
