#ifndef NX_CHARACTER_CONTROLLER
#define NX_CHARACTER_CONTROLLER
/*----------------------------------------------------------------------------*\
|
|								NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

/* Exclude from documentation */
/** \cond */

enum TriangleCollisionFlag
	{
	// Must be the 3 first ones to be indexed by (flags & (1<<edge_index))
	TCF_ACTIVE_EDGE01	= (1<<0),
	TCF_ACTIVE_EDGE12	= (1<<1),
	TCF_ACTIVE_EDGE20	= (1<<2),
	TCF_DOUBLE_SIDED	= (1<<3),
//	TCF_WALKABLE		= (1<<4),
	};


/*
class TriangleMeshShape;

struct triangleData
	{
		NxVec3				verts[3];	// triangle vertices in world space
		TriangleMeshShape*	owner;		// mesh owner
		NxU32				edgeFlags;	// misc edge flags
		NxU32				index;		// triangle index in original database
	};
const NxU32 triangleDataSize = sizeof(triangleData)/sizeof(NxU32);

	// Customized container for triangle data
class triangleCollisionData : private Container
	{
		public:
		// Constructor / Destructor
									triangleCollisionData()		{}
									~triangleCollisionData()	{}

		NX_INLINE	void			reset()						{ Reset();												}
		NX_INLINE	udword			getNbTriangles()	const	{ return GetNbEntries()/triangleDataSize;				}
		NX_INLINE	triangleData*	getTriangleData()	const	{ return (triangleData*)GetEntries();					}
		NX_INLINE	triangleData*	reserve(NxU32 nb)			{ return (triangleData*)Reserve(nb*triangleDataSize);	}
	};
*/

	enum TouchedGeomType
	{
		TOUCHED_BOX_CONTROLLER,
		TOUCHED_CAPSULE_CONTROLLER,
		TOUCHED_MESH,
		TOUCHED_BOX,
		TOUCHED_SPHERE,
		TOUCHED_CAPSULE,

		TOUCHED_LAST,

		TOUCHED_FORCE_DWORD	= 0x7fffffff
	};

	class SweepTest;

// PT: apparently stupid .Net aligns some of them on 8-bytes boundaries for no good reason. This is bad.
#pragma pack(4)

	struct TouchedGeom
	{
		TouchedGeomType	mType;
		void*			mUserData;	// NxController or NxShape pointer
		BigPoint		mOffset;	// Local origin, typically the center of the world bounds around the character. We translate both
									// touched shapes & the character so that they are nearby this point, then add the offset back to
									// computed "world" impacts.
	};

	struct TouchedBoxController : public TouchedGeom
	{
		PruningAABB		mBox;
	};
	ICE_COMPILE_TIME_ASSERT(sizeof(TouchedBoxController)==sizeof(TouchedGeom)+sizeof(PruningAABB));

	struct TouchedCapsuleController : public TouchedGeom
	{
		PruningLSS		mCapsule;
	};

	struct TouchedMesh : public TouchedGeom
	{
		udword			mNbTris;
		udword			mIndexWorldTriangles;
		udword			mIndexWorldEdgeNormals;
		udword			mIndexEdgeFlags;
	};

	struct TouchedBox : public TouchedGeom
	{
		OBB				mBox;
	};
	ICE_COMPILE_TIME_ASSERT(sizeof(TouchedBox)==sizeof(TouchedGeom)+sizeof(OBB));

	struct TouchedSphere : public TouchedGeom
	{
		Sphere			mSphere;
	};
	ICE_COMPILE_TIME_ASSERT(sizeof(TouchedSphere)==sizeof(TouchedGeom)+sizeof(Sphere));

	struct TouchedCapsule : public TouchedGeom
	{
		LSS				mCapsule;
	};
	ICE_COMPILE_TIME_ASSERT(sizeof(TouchedCapsule)==sizeof(TouchedGeom)+sizeof(LSS));

#pragma pack()

	enum SweptContactType
	{
		SWEPT_CTC_SHAPE,		// We touched another shape
		SWEPT_CTC_CONTROLLER,	// We touched another controller
	};

	struct SweptContact
	{
		BigPoint			mWorldPos;		// Contact position in world space
		Point				mWorldNormal;	// Contact normal in world space
		float				mDistance;		// Contact distance
		udword				mIndex;			// Feature identifier (e.g. triangle index for meshes)
		TouchedGeom*		mGeom;
	};

	enum SweptVolumeType
	{
		SWEPT_BOX,
		SWEPT_CAPSULE,

		SWEPT_LAST
	};

	class SweptVolume
	{
		public:
			SweptVolume();
			~SweptVolume();

		virtual void ComputeTemporalBox(const SweepTest&, PruningAABB& box, const Point& direction)							const	= 0;
		virtual void ComputeTemporalBox(const SweepTest&, PruningAABB& box, const BigPoint& center, const Point& direction)	const	= 0;

		inline_	SweptVolumeType	GetType()	const	{ return mType;	}

		BigPoint		mCenter;
		protected:
		SweptVolumeType	mType;
	};

	class SweptBox : public SweptVolume
	{
		public:
			SweptBox();
			~SweptBox();

		virtual void ComputeTemporalBox(const SweepTest&, PruningAABB& box, const Point& direction)							const;
		virtual void ComputeTemporalBox(const SweepTest&, PruningAABB& box, const BigPoint& center, const Point& direction)	const;

		Point	mExtents;
	};

	class SweptCapsule : public SweptVolume
	{
		public:
						SweptCapsule();
						~SweptCapsule();

		virtual void	ComputeTemporalBox(const SweepTest&, PruningAABB& box, const Point& direction)							const;
		virtual void	ComputeTemporalBox(const SweepTest&, PruningAABB& box, const BigPoint& center, const Point& direction)	const;

		inline_	void	GetCapsule(const BigPoint& center, PruningLSS& capsule, udword axis) const
						{
							capsule.mRadius		= mRadius;
							capsule.mP0			= center;
							capsule.mP1			= center;
							capsule.mP0[axis]	+= mHeight*0.5f;
							capsule.mP1[axis]	-= mHeight*0.5f;
						}
		inline_	void	GetLocalCapsule(LSS& capsule, udword axis) const
						{
							capsule.mRadius		= mRadius;
							capsule.mP0.Zero();
							capsule.mP1.Zero();
							capsule.mP0[axis]	+= mHeight*0.5f;
							capsule.mP1[axis]	-= mHeight*0.5f;
						}

		float	mRadius;
		float	mHeight;
	};

	class SweepTest
	{
		public:
							SweepTest();
							~SweepTest();

				void		MoveCharacter(
							void* user_data,
							void* user_data2,
							SweptVolume& volume,
							const Point& direction,
							udword nb_boxes, const PruningAABB* boxes, const void** box_user_data,
							udword nb_capsules, const PruningLSS* capsules, const void** capsule_user_data,
							udword groups, float min_dist,
							udword& collision_flags
							);

				bool		DoSweepTest(
								void* user_data,
								void* user_data2,
								udword nb_boxes, const PruningAABB* boxes, const void** box_user_data,
								udword nb_capsules, const PruningLSS* capsules, const void** capsule_user_data,
								SweptVolume& swept_volume,
								const Point& direction, float bump, float friction, udword max_iter,
								udword* nb_collisions, udword group_flags, float min_dist, bool down_pass=false);

				void		FindTouchedCCTs(
									udword nb_boxes, const PruningAABB* boxes, const void** box_user_data,
									udword nb_capsules, const PruningLSS* capsules, const void** capsule_user_data,
									const PruningAABB& world_box
									);

//		private:
				TriList		mWorldTriangles;
				TriList		mWorldEdgeNormals;
				Container	mEdgeFlags;
				Container	mGeoms;
				PruningAABB	mCachedTBV;
				udword		mNbCachedStatic;
				udword		mNbCachedT;
				udword		mNbCachedEN;
				udword		mNbCachedF;
		public:
				Triangle	mTouched;

				float		mSlopeLimit;
				float		mSkinWidth;
				float		mStepOffset;
				float		mVolumeGrowth;	// Must be >1.0f and not too big
				udword		mUpDirection;
				udword		mMaxIter;
				bool		mHitNonWalkable;
				bool		mWalkExperiment;
				bool		mHandleSlope;
				bool		mValidTri;
				bool		mValidateCallback;
				bool		mNormalizeResponse;
		private:
				void		UpdateTouchedGeoms(void* user_data,
												udword nb_boxes, const PruningAABB* boxes, const void** box_user_data,
												udword nb_capsules, const PruningLSS* capsules, const void** capsule_user_data,
												udword group_flags, const PruningAABB& world_box, bool first_iter);
	};

bool FindTouchedGeometry(
	void* user_data,
	const PruningAABB& world_aabb,

	TriList& world_triangles,
	TriList& world_edge_normals,
	Container& edge_flags,
	Container& geoms,
	udword group_flags,
	bool static_shapes, bool dynamic_shapes);

	void ShapeHitCallback(void* user_data2, const SweptContact& contact, const Point& dir, float length);
	void ControllerHitCallback(void* user_data2, const SweptContact& contact, const Point& dir, float length);

	// On the NX side:

	// User-defined structure passed to callbacks
	class NxScene;
	class NxActor;
	class NxShape;
	struct CallbackData
	{
		NxScene*	scene;
		NxShape*	shape;
		NxActor*	actor;
	};

/** \endcond */
#endif
