#ifndef PHYSX_LOADER_H
#define PHYSX_LOADER_H
/*----------------------------------------------------------------------------*\
|
|								NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/

#include "NxPhysicsSDK.h"
#include "NxUtilLib.h"

#ifdef _USRDLL
	#define NXPHYSXLOADERDLL_API extern "C" __declspec(dllexport)
#elif defined NX_USE_SDK_STATICLIBS
	#define NXPHYSXLOADERDLL_API
#else
	#define NXPHYSXLOADERDLL_API extern "C" __declspec(dllimport)
#endif


/** \addtogroup PhysXLoader
  @{
*/

/**
\brief Creates an instance of the physics SDK.

Creates an instance of this class. May not be a class member to avoid name mangling.
Pass the constant NX_PHYSICS_SDK_VERSION as the argument.
Because the class is a singleton class, multiple calls return the same object.

\param sdkVersion Version number we are expecting(should be NX_PHYSICS_SDK_VERSION)
\param allocator User supplied interface for allocating memory(see #NxUserAllocator)
\param outputStream User supplied interface for reporting errors and displaying messages(see #NxUserOutputStream)
\param desc Optional descriptor used to define hardware allocation parameters
*/
NXPHYSXLOADERDLL_API NxPhysicsSDK    *NxCreatePhysicsSDK(NxU32 sdkVersion, NxUserAllocator* allocator = NULL, NxUserOutputStream* outputStream = NULL, const NxPhysicsSDKDesc &desc = NxPhysicsSDKDesc());

/**
\brief Release an instance of the PhysX SDK.
*/
NXPHYSXLOADERDLL_API void             NxReleasePhysicsSDK(NxPhysicsSDK *sdk);

/**
\brief Retrieves the Physics SDK allocator.

Used by NxAllocateable's inlines and other macros below.

Before using this function the user must call #NxCreatePhysicsSDK(). If #NxCreatePhysicsSDK()
has not been called then NULL will be returned.
*/
NXPHYSXLOADERDLL_API NxUserAllocator *NxGetPhysicsSDKAllocator();

/**
\brief Retrieves the Foundation SDK after it has been created.

Before using this function the user must call #NxCreatePhysicsSDK(). If #NxCreatePhysicsSDK()
has not been called then NULL will be returned..
*/
NXPHYSXLOADERDLL_API NxFoundationSDK* NxGetFoundationSDK();

/**
\brief Retrieves the Physics SDK after it has been created.

Before using this function the user must call #NxCreatePhysicsSDK(). If #NxCreatePhysicsSDK()
has not been called then NULL will be returned.
*/
NXPHYSXLOADERDLL_API NxPhysicsSDK*    NxGetPhysicsSDK();

/**
\brief Retrieves the Physics SDK Utility Library

Before using this function the user must call #NxCreatePhysicsSDK(). If #NxCreatePhysicsSDK()
has not been called then NULL will be returned.
*/

NXPHYSXLOADERDLL_API NxUtilLib *NxGetUtilLib();

/** @} */

#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

