#ifndef NX_PHYSICS_NXUSERNOTIFY
#define NX_PHYSICS_NXUSERNOTIFY
/*----------------------------------------------------------------------------*\
|
|						Public Interface to NovodeX Technology
|
|							     www.novodex.com
|
\*----------------------------------------------------------------------------*/
/** \addtogroup physics
  @{
*/

#include "Nxp.h"

class NxJoint;
/**
 \brief An interface class that the user can implement in order to receive simulation events.

  <b>Threading:</b> It is not necessary to make this class thread safe as it will only be called in the context of the
  user thread.

 <h3>Example</h3>

 \include NxUserNotify_Example.cpp

 @see NxScene.setUserNotify() NxScene.getUserNotify()
*/
class NxUserNotify
	{
	public:
	/**
	\brief This is called when a breakable joint breaks.
	
	The user should not release the joint inside this call! 
	Instead, if the user would like to have the joint
	released and no longer holds any referenced to it, he should return true.
	In this case the joint will be released by the system. Otherwise the user should return false.

	\param[in] breakingForce The force which caused the joint to break.
	\param[in] brokenJoint The joint which has been broken.
	\return \return True to have the system release the joint. False to keep the joint.

	<b>Platform:</b>
	\li PC SW: Yes
	\li PPU  : No
	\li PS3  : Yes
	\li XB360: Yes

	@see NxJoint.setBreakable() NxJointDesc.maxForce NxJointDesc.maxTorque
	*/
	virtual bool onJointBreak(NxReal breakingForce, NxJoint & brokenJoint) = 0;
	};

/** @} */
#endif


//AGCOPYRIGHTBEGIN
///////////////////////////////////////////////////////////////////////////
// Copyright  2005 AGEIA Technologies.
// All rights reserved. www.ageia.com
///////////////////////////////////////////////////////////////////////////
//AGCOPYRIGHTEND

