/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* CEngineFactory - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Loader/CFBXLoader.h>
#include <Loader/CTGALoader.h>
#include <Loader/CTerrainBuilder.h>
#include <Renderer/COGLRenderer.h>
#include <Renderer/COGLModelFactory.h>
#include <Input/CDirectInputMgr.h>
#include <Sound/CFModSoundMgr.h>
#include "CEngineFactory.h"
#include "Objects/Terrain/CTerrain.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
CEngineFactory::CEngineFactory() : m_pRenderer(NULL), m_pModelFactory(NULL), m_pInputMgr(NULL), m_pGuiMgr(NULL), 
        m_pSoundMgr(NULL), m_pPhysicsMgr(NULL)
{

	//openGL Renderer
	m_pRenderer = new COGLRenderer();
	
	//openGL ModelFactory
    TSmartPointer<ITextureLoader> pTextureLoader = new CTGALoader;
	m_pModelFactory = new COGLModelFactory(new CFBXLoader, pTextureLoader, new CTerrainBuilder(pTextureLoader));
}

const TSmartPointer<IInputMgr> &CEngineFactory::GetInputMgr()
{
    if(!m_pInputMgr)
    {
        m_pInputMgr = new CDirectInputMgr;
    }

    return m_pInputMgr;
}

const TSmartPointer<CGuiMgr> &CEngineFactory::GetGuiMgr()
{
    if(!m_pGuiMgr)
    {
        m_pGuiMgr = new CGuiMgr;
    }

    return m_pGuiMgr;
}

const TSmartPointer<ISoundMgr> &CEngineFactory::GetSoundMgr()
{
	if(!m_pSoundMgr)
	{
		m_pSoundMgr = new CFModSoundMgr;
	}

	return m_pSoundMgr;
}

const TSmartPointer<CPhysXMgr> &CEngineFactory::GetPhysicsMgr()
{
    if(!m_pPhysicsMgr)
    {
        m_pPhysicsMgr = new CPhysXMgr;
    }

    return m_pPhysicsMgr;
}

CEngineFactory::~CEngineFactory(){
    m_pRenderer = NULL;
    m_pModelFactory = NULL;

    m_pInputMgr = NULL;
    m_pGuiMgr = NULL;
	m_pSoundMgr = NULL;
    m_pPhysicsMgr = NULL;
}
/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


