/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* CEngineFactory - Header File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CENGINEFACTORY_H__
#define __CENGINEFACTORY_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Misc/TSingleton.h>

#include <Renderer/IRenderer.h>
#include <Renderer/IModelFactory.h>
#include <Input/IInputMgr.h>
#include <Sound/ISoundMgr.h>
#include <Display/Gui/CGuiMgr.h>
#include <Collision/CPhysXMgr.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CEngineFactory : public TSingleton<CEngineFactory>
{
	friend class TSingleton;

    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		inline TSmartPointer<IRenderer> GetRenderer() const {return m_pRenderer;}
		inline TSmartPointer<IModelFactory> GetModelFactory() const {return m_pModelFactory;}
        const TSmartPointer<IInputMgr> &GetInputMgr();
        const TSmartPointer<CGuiMgr> &GetGuiMgr();
		const TSmartPointer<ISoundMgr> &GetSoundMgr();
        const TSmartPointer<CPhysXMgr> &GetPhysicsMgr();

    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		CEngineFactory();
		virtual ~CEngineFactory();
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        CEngineFactory(const CEngineFactory &);
        CEngineFactory &operator =(const CEngineFactory &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		TSmartPointer<IRenderer>      m_pRenderer;
		TSmartPointer<IModelFactory>  m_pModelFactory;
        TSmartPointer<IInputMgr>      m_pInputMgr;
        TSmartPointer<CGuiMgr>        m_pGuiMgr;
		TSmartPointer<ISoundMgr>      m_pSoundMgr;
        TSmartPointer<CPhysXMgr>      m_pPhysicsMgr;

};

#endif
