/****************************************************************************\
*                                                                            *
* 11.05.2006                                                                 *
* CPhysXActor - Source File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <NxActor.h>
#include "CPhysXActor.h"
#include "CPhysXCharacter.h"
#include <Math/CMatrix33.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CPhysXActor::SetPos(const CVector3f &vPos)
{
    m_pActor->setGlobalPosition(NxVec3(vPos.m_X, vPos.m_Y, vPos.m_Z));
}

void CPhysXActor::SetRot(const CVector3f &vRot)
{
	CMatrix33 mat;
	mat.RotationZXY(vRot);
	NxMat33 nxm;
	nxm.setColumnMajor(&mat._11);
	m_pActor->setGlobalOrientation(nxm);
}

const CVector3f CPhysXActor::GetPos()
{
	NxVec3 vGPos(m_pActor->getGlobalPosition());
	return CVector3f(vGPos.x, vGPos.y, vGPos.z);
}

const CVector3f CPhysXActor::GetRot()
{
	NxMat33 rot(m_pActor->getGlobalOrientation());
	CMatrix33 mat;
	rot.getColumnMajor(&mat._11);
	return mat.GetZXYRotationVector();
}

void CPhysXActor::SetGroup(word wGroup)
{
    m_pActor->setGroup(wGroup);
}

void CPhysXActor::AddForce(const CVector3f &vForce, EForceMode mode)
{
    NxForceMode m;
    switch(mode)
    {
        case EFM_IMPULSE:
            m = NX_IMPULSE;
        case EFM_VELOCITY:
            m = NX_VELOCITY_CHANGE;
        default:
            m = NX_FORCE;
    }

    m_pActor->addForce(NxVec3(vForce.m_X, vForce.m_Y, vForce.m_Z), m);
}

void CPhysXActor::AddTorque(const CVector3f &vForce, EForceMode mode)
{
    NxForceMode m;
    switch(mode)
    {
    case EFM_IMPULSE:
        m = NX_IMPULSE;
    case EFM_VELOCITY:
        m = NX_VELOCITY_CHANGE;
    default:
        m = NX_FORCE;
    }

    m_pActor->addTorque(NxVec3(vForce.m_X, vForce.m_Y, vForce.m_Z), m);
}

CPhysXActor::~CPhysXActor()
{
    m_pActor = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
CPhysXActor::CPhysXActor(NxActor *pActor) : m_pActor(pActor)
{
    //Bind this class to the NxActor to allow the Manager to notify us
    m_pActor->userData = this;
}


