/****************************************************************************\
*                                                                            *
* 10.05.2006                                                                 *
* CPhysXMgr - Header File													 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CPHYSXMGR_H__
#define __CPHYSXMGR_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include "CPhysXActor.h"
#include "CPhysXCharacter.h"
#include "CPhysXMesh.h"
#include <Math/CMatrix44.h>
#include <Math/TVector3.h>
#include <Math/TAABox3.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class NxPhysicsSDK;
class NxScene;
class ControllerManager;
class NxActorDesc;
class NxBodyDesc;
class NxShapeDesc;
class MyContactReport;
class MyControllerHitReport;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CPhysXMgr : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CPhysXMgr();
        
        bool Create();

        void Update(float dt);

        void EnableDebugRendering(bool bEnable);
        void RenderDebugData();

        inline const CVector3f &GetGravity() const {return m_vGravity;}

        //Mesh cooking
        const TSmartPointer<CPhysXMesh> CookTerrain(dword dwNumVertices, dword dwNumIndices,
                                                    const CVector3f *pVertexBuffer, const dword *pIndexBuffer, float fTerrainDepth);

        //Actor creation functions
        void BeginActorCreation(const CMatrix44 &orientation, byte bCollisionGroup, bool bDynamicActor, float fDensity = 1.0f);
        CPhysXActor *FinishActorCreation();
        //These functions should only be called between BeginActorCreation and FinishActorCreation
        void AddMesh(const CMatrix44 &orientation, const TSmartPointer<CPhysXMesh> &pMesh);
        void AddSphere(const CMatrix44 &orientation, float fRadius);
        void AddCube(const CMatrix44 &orientation, const CVector3f &vSize);
        void AddCapsule(const CMatrix44 &orientation, float fRadius, float fHeight);
		void AddPlane(const CVector4f &plane);

        void DeleteActor(CPhysXActor *pActor);

        //Character creation
        CPhysXCharacter *CreateCapsuleCharacter(const CVector3f &vPos, float fRadius, float fHeight, byte bCollisionGroup);
        CPhysXCharacter *CreateAABoxCharacter(const CVector3f &vPos, const CVector3f &vSize, byte bCollisionGroup);
        void DeleteCharacter(CPhysXCharacter *pCharacter);

		bool CheckOverlapping(const CAABox3f &box, dword dwGroupMask);

        //Collision flags
        void SetGroupCollisionTesting(byte group1, byte group2, bool enable);
/*
	void SetGroupCollisionNotification(word wGroup1, word wGroup2, bool bEnable);
*/

        virtual ~CPhysXMgr();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void SetDebugColor(dword dwColor) const;

        //Private copy constructor and assignment operator to prevent mistakes:
        CPhysXMgr(const CPhysXMgr &);
        const CPhysXMgr &operator =(const CPhysXMgr &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		byte m_bActorGroup;
        NxActorDesc  *m_pActorDesc;
        NxBodyDesc  *m_pBodyDesc;
        typedef std::vector<NxShapeDesc *> ShapeList;
        ShapeList m_ShapeDescs;

        NxPhysicsSDK *m_pPhysicsSDK;
        NxScene      *m_pScene;
        ControllerManager *m_pControllerMgr;
        typedef std::vector< TSmartPointer<CPhysXMesh> > MeshList;
        MeshList m_Meshes;

        //Gravity
        CVector3f m_vGravity;

        //Debug: Ground plane
        NxActor *m_pGroundPlane;
};

#endif
