/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* CQTSceneGraph - Source File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CQTSceneGraph.h"
#include "CEngineFactory.h"
#include "TQTNode.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CQTSceneGraph::Create(float fSizeX, float fSizeZ, float fMinY, float fMaxY, word wGraphDepth)
{
    assert(fSizeX && fSizeZ && wGraphDepth && "Error: One or more Parameters are ZERO in CQTSceneGraph::Create");
    assert(m_pRootNode == NULL && "Error: CQTSceneGraph already created!");
    
    //Lets get to work:
    m_pRootNode = new TQTNode<IRepresentation>;
    m_pRootNode->Create(0, 0, fSizeX, fSizeZ, fMinY, fMaxY, wGraphDepth);

	m_fSizeX      = fSizeX;
	m_fSizeZ      = fSizeZ;
	m_fMinY       = fMinY;
	m_fMaxY       = fMaxY;
	m_wGraphDepth = wGraphDepth;
    
    return true;
}

TQTNode<IRepresentation> * const CQTSceneGraph::CreateSpecialRootNode(TSmartPointer<IStateNode> pStateNode){

	assert( (bool)pStateNode && "pStartRep is NULL in CQTSceneGraph::CQTCreateSpecialRootNode()");
	
	TQTNode<IRepresentation> * pSpecialRoot = new TQTNode<IRepresentation>;
	pSpecialRoot->Create(0, 0, m_fSizeX, m_fSizeZ, m_fMinY, m_fMaxY, m_wGraphDepth);
	pSpecialRoot->SetStateNode(pStateNode);
		
	m_vSpecialRootNodes.push_back(pSpecialRoot);
	return pSpecialRoot;
}

#include <gl/gl.h>

void CQTSceneGraph::Render(bool bVerify)
{
    assert((bool) m_pCamera && "Error: Camera not set in CQTSceneGraph::Render");
    assert(m_pRootNode && "Error: CQTSceneGraph::Render called before creation");

    bool bCamChanged = m_pCamera->CameraChanged();
    m_pCamera->UseForRendering();

	if(m_pEnvironment)
    {
        if(bCamChanged)
        {
            m_CameraRotM.RotationZYX(-m_pCamera->GetRot());
        }
        CEngineFactory::InstancePtr()->GetRenderer()->SetMatrixMode(EMM_MODELVIEW);
		CEngineFactory::InstancePtr()->GetRenderer()->SaveMatrix();
		CEngineFactory::InstancePtr()->GetRenderer()->SetMatrix(m_CameraRotM);
		CEngineFactory::InstancePtr()->GetRenderer()->EnableZWrite(false);
		m_pEnvironment->Render(m_pCamera->GetFrustum(), false);
		CEngineFactory::InstancePtr()->GetRenderer()->EnableZWrite(true);
		CEngineFactory::InstancePtr()->GetRenderer()->RestoreMatrix();

		//"Sonne" an den Renderer weitergeben
		CEngineFactory::InstancePtr()->GetRenderer()->SetLightDir(m_vSunDir);
    }

	
    //Debug:
    CEngineFactory::InstancePtr()->GetPhysicsMgr()->RenderDebugData();
  
	//RootNode
	m_pRootNode->ms_dwCount = 0;
    m_pRootNode->Render(m_pCamera->GetFrustum(), bVerify);

	//SpecialRootNodes
	for(int i = 0 ; i < (int)m_vSpecialRootNodes.size() ; i++){
		m_vSpecialRootNodes[i]->getStateNode()->ExecutePreState();
		m_vSpecialRootNodes[i]->Render(m_pCamera->GetFrustum(), bVerify);
		m_vSpecialRootNodes[i]->getStateNode()->ExecutePostState();
	}
}

void CQTSceneGraph::AddObject(const TSmartPointer<IRepresentation> &pRepresentation)
{
    assert(m_pRootNode && "Error: CQTSceneGraph::AddObject called before creation");
    m_pRootNode->AddObject(pRepresentation);
}

void CQTSceneGraph::RemoveObject(const TSmartPointer<IRepresentation> &pRepresentation)
{
    assert(m_pRootNode && "Error: CQTSceneGraph::AddObject called before creation");
	if(!m_pRootNode->RemoveObject(pRepresentation)){
		for(int i = 0; i < (int)m_vSpecialRootNodes.size() ;i++){
			if(m_vSpecialRootNodes[i]->RemoveObject(pRepresentation)){
				break;
			}
		}
	}
}

void CQTSceneGraph::PickObject(const CVector3f &vRayOrigin, const CVector3f &vRayDir,
                               TSmartPointer<IRepresentation> &pRepresentation, CVector3f &vHitPos)
{
    assert(m_pRootNode && "Error: CQTSceneGraph::PickObject called before creation");
    assert((bool) m_pCamera && "Error: CQTSceneGraph::PickObject called with NULL camera");
    float fDistance = cg_fMaxIntersectionDist;

    m_pRootNode->PickObject(m_pCamera->GetFrustum(), vRayOrigin, vRayDir, pRepresentation, fDistance, vHitPos, true);
}

void CQTSceneGraph::Destroy()
{
    SAFE_DELETE(m_pRootNode);

	for(int i = 0 ; i < (int)m_vSpecialRootNodes.size() ; i++){
		SAFE_DELETE(m_vSpecialRootNodes[i]);
	}

	Clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CQTSceneGraph::Clear()
{
    m_pRootNode = NULL;
    m_pCamera = NULL;
}