/****************************************************************************\
*                                                                            *
* 28.03.2006                                                                 *
* CCamera - Header File                                                      *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CCAMERA_H__
#define __CCAMERA_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include "CFrustum.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CCamera : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CCamera() {Clear();}

        //State setting
        void SetPerspective(const float &fFov, const float &fAspectRatio, const float &fNearZ, const float &fFarZ);
        void SetPos(const CVector3f &vPos);
        void SetRot(const CVector3f &vRot);
        void LookAt(const CVector3f &vPoint, const CVector3f &vUp);

        void MoveIn(float length);
        void MoveUp(float length);
        void MoveRight(float length);
        void MoveX(float length);
        void MoveY(float length);
        void MoveZ(float length);
        void Move(const CVector3f &vDistance);

        //Rotations (require recalculation afterwards)
        //void Rotate(float fX, float fY, float fZ);
        void RotateX(float angle);
        void RotateY(float angle);
        void RotateZ(float angle);

        //State request:
        inline const CVector3f GetPos() const {return -m_vPos;}
        inline const CVector3f GetRot() const {return -m_vRot;}
        const CVector3f &GetIn();
        const CVector3f &GetUp();
        const CVector3f &GetRight();

        //Debugging:
        void Log(word wTargetLog);

        void UseForRendering();

        inline bool CameraChanged() const {return !m_bViewValid;}

        const CFrustum &GetFrustum();       //Get current frustum. Only valid until next movement/state function is called

        const CVector3f GetRay(float fX, float fY);  //X and Y are between 0 and 1 (relative screen coordinates on near plane)
        const CMatrix44 &GetTransformation();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        void Update();

        //Private copy constructor and assignment operator to prevent mistakes:
        CCamera(const CCamera &);
        CCamera &operator =(const CCamera &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CMatrix44 m_Projection;
        CMatrix44 m_View;
        CVector3f m_vPos, m_vRot;
        CFrustum m_Frustum;

        //Helpers
        CVector3f m_vUp, m_vIn, m_vRight;
        bool m_bVectorsValid;
        bool m_bViewValid;
        bool m_bFrustumValid;

};

#endif
