/****************************************************************************\
*                                                                            *
* 16.04.2006                                                                 *
* CFlyCamCtrl - Source File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CFlyCamCtrl.h"
#include "CCamera.h"
#include "CEngineFactory.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CFlyCamCtrl::CFlyCamCtrl(const TSmartPointer<CTimer> &pTimer, word wInputPriority)
{
    assert((bool) pTimer && "Error: Timer is NULL in CFlyCamCtrl::CFlyCamCtrl");

    m_pTimer = pTimer;
    m_pCamera = NULL;
    m_bForward = false;
    m_bBackward = false;
    m_bLeft = false;
    m_bRight = false;

    SetPriority(wInputPriority);
}

void CFlyCamCtrl::Enable(const TSmartPointer<CCamera> &pCamera)
{
    if(!IsEnabled())
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->RegisterHandler(this);
        m_bForward = false;
        m_bBackward = false;
        m_bLeft = false;
        m_bRight = false;
    }
    ICamCtrl::Enable(pCamera);
}

void CFlyCamCtrl::Update()
{
    if(IsEnabled())
    {
		float dt = (float) m_pTimer->GetFrameDuration();
        if(m_bForward)
        {
            m_pCamera->MoveIn(dt * 50.0f);
        }
        if(m_bBackward)
        {
            m_pCamera->MoveIn(dt * -50.0f);
        }
        if(m_bLeft)
        {
            m_pCamera->MoveRight(dt * -50.0f);
        }
        if(m_bRight)
        {
            m_pCamera->MoveRight(dt * 50.0f);
        }
    }
}

void CFlyCamCtrl::Disable()
{
    if(IsEnabled())
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->UnregisterHandler(this);
    }
    ICamCtrl::Disable();
    m_bForward = false;
    m_bBackward = false;
    m_bLeft = false;
    m_bRight = false;
}

CFlyCamCtrl::~CFlyCamCtrl()
{
    m_pTimer = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
bool CFlyCamCtrl::OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ)
{
    if(!IsEnabled())
        return false;

    m_pCamera->RotateX((float)sdwY / 150.0f);
    m_pCamera->RotateY((float)sdwX / 150.0f);
    m_pCamera->RotateZ((float) sdwZ / 1000.0f);

    return true;
}

bool CFlyCamCtrl::OnKeyDown(dword dwKey)
{
    if(!IsEnabled())
        return false;

    switch(dwKey)
    {
        case cg_dwCursorUp:
            m_bForward = true;
            return true;
        case cg_dwCursorDown:
            m_bBackward = true;
            return true;
        case cg_dwCursorLeft:
            m_bLeft = true;
            return true;
        case cg_dwCursorRight:
            m_bRight = true;
            return true;
    }

    return false;

}

bool CFlyCamCtrl::OnKeyUp(dword dwKey)
{
    if(!IsEnabled())
        return false;

    switch(dwKey)
    {
        case cg_dwCursorUp:
            m_bForward = false;
            return true;
        case cg_dwCursorDown:
            m_bBackward = false;
            return true;
        case cg_dwCursorLeft:
            m_bLeft = false;
            return true;
        case cg_dwCursorRight:
            m_bRight = false;
            return true;
    }

    return false;
}
