/****************************************************************************\
*                                                                            *
* 29.03.2006                                                                 *
* CFrustum - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CFrustum.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CFrustum::CalculateFrustum(const CMatrix44 &projection, const CMatrix44 &view)
{
    CMatrix44 temp;
    CVector3f v;
    float n;

    temp.Multiply(projection, view);

    //Calculating frustum planes
    //Left
    v.Set(temp._41 + temp._11, temp._42 + temp._12, temp._43 + temp._13);
    n = 1.0f / v.Length();
    m_Planes[0].m_A = v.m_X * n;
    m_Planes[0].m_B = v.m_Y * n;
    m_Planes[0].m_C = v.m_Z * n;
    m_Planes[0].m_D = (temp._44 + temp._14) * n;

    //Right
    v.Set(temp._41 - temp._11, temp._42 - temp._12, temp._43 - temp._13);
    n = 1.0f / v.Length();
    m_Planes[1].m_A = v.m_X * n;
    m_Planes[1].m_B = v.m_Y * n;
    m_Planes[1].m_C = v.m_Z * n;
    m_Planes[1].m_D = (temp._44 - temp._14) * n;

    //Bottom
    v.Set(temp._41 + temp._21, temp._42 + temp._22, temp._43 + temp._23);
    n = 1.0f / v.Length();
    m_Planes[2].m_A = v.m_X * n;
    m_Planes[2].m_B = v.m_Y * n;
    m_Planes[2].m_C = v.m_Z * n;
    m_Planes[2].m_D = (temp._44 + temp._24) * n;

    //Top
    v.Set(temp._41 - temp._21, temp._42 - temp._22, temp._43 - temp._23);
    n = 1.0f / v.Length();
    m_Planes[3].m_A = v.m_X * n;
    m_Planes[3].m_B = v.m_Y * n;
    m_Planes[3].m_C = v.m_Z * n;
    m_Planes[3].m_D = (temp._44 - temp._24) * n;

    //Near
    v.Set(temp._41 + temp._31, temp._42 + temp._32, temp._43 + temp._33);
    n = 1.0f / v.Length();
    m_Planes[4].m_A = v.m_X * n;
    m_Planes[4].m_B = v.m_Y * n;
    m_Planes[4].m_C = v.m_Z * n;
    m_Planes[4].m_D = (temp._44 + temp._34) * n;

    //Far
    v.Set(temp._41 - temp._31, temp._42 - temp._32, temp._43 - temp._33);
    n = 1.0f / v.Length();
    m_Planes[5].m_A = v.m_X * n;
    m_Planes[5].m_B = v.m_Y * n;
    m_Planes[5].m_C = v.m_Z * n;
    m_Planes[5].m_D = (temp._44 - temp._34) * n;
}

sbyte CFrustum::SphereInFrustum(const CSpheref &sphere) const
{
    float fDistance;
    for(byte p = 0; p < 6; ++p)  //Knnte man entfalten... schneller?
    {
        fDistance = m_Planes[p].m_A * sphere.m_vCenter.m_X + m_Planes[p].m_B * sphere.m_vCenter.m_Y + m_Planes[p].m_C * sphere.m_vCenter.m_Z + m_Planes[p].m_D;
        if(fDistance < -sphere.m_Radius)
            return FRUSTUM_OUTSIDE;
        else if(fDistance > -sphere.m_Radius && fDistance < sphere.m_Radius)
            return FRUSTUM_INTERSECTS;
    }
    return FRUSTUM_INSIDE;
}

bool CFrustum::BoxInFrustum(const CAABox3f &box) const
{
    byte p;
    for(p = 0; p < 6; ++p)  //Knnte man entfalten... schneller?
    {
        if(m_Planes[p].m_A * box.m_MinX + m_Planes[p].m_B * box.m_MinY + m_Planes[p].m_C * box.m_MinZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MaxX + m_Planes[p].m_B * box.m_MinY + m_Planes[p].m_C * box.m_MinZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MinX + m_Planes[p].m_B * box.m_MaxY + m_Planes[p].m_C * box.m_MinZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MaxX + m_Planes[p].m_B * box.m_MaxY + m_Planes[p].m_C * box.m_MinZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MinX + m_Planes[p].m_B * box.m_MinY + m_Planes[p].m_C * box.m_MaxZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MaxX + m_Planes[p].m_B * box.m_MinY + m_Planes[p].m_C * box.m_MaxZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MinX + m_Planes[p].m_B * box.m_MaxY + m_Planes[p].m_C * box.m_MaxZ + m_Planes[p].m_D > 0)
            continue;
        if(m_Planes[p].m_A * box.m_MaxX + m_Planes[p].m_B * box.m_MaxY + m_Planes[p].m_C * box.m_MaxZ + m_Planes[p].m_D > 0)
            continue;
        return false;
    }
    return true;
}

bool CFrustum::BoxInFrustum(const CBox3f &box) const
{
    byte p;
    for(p = 0; p < 6; ++p)  //Knnte man entfalten... schneller?
    {
        if(m_Planes[p].Distance(box.m_vPoint) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vA) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vB) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vC) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vA + box.m_vB) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vA + box.m_vC) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vB + box.m_vC) > 0.0f)
            continue;
        if(m_Planes[p].Distance(box.m_vPoint + box.m_vA + box.m_vB + box.m_vC) > 0.0f)
            continue;

        return false;
    }
    return true;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


