/****************************************************************************\
*                                                                            *
* 22.03.2006                                                                 *
* CGuiButton - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CGuiButton.h"
#include "CGuiMgr.h"
#include "CEngineFactory.h"

//Debug
#include <gl/GL.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CGuiButton::SetText(const std::string &sText, dword dwColor)
{
    if(!m_pText)
    {
        m_pText = CEngineFactory::InstancePtr()->GetGuiMgr()->CreateText();
    }
    m_pText->SetText(sText);
    m_pText->SetColor(dwColor);
    //ToDo: Zentrieren und so..
    m_pText->SetPos(CVector3f(10.0f, GetScale().m_Y / 2.0f - 10.0f, 1.0f));
}

void CGuiButton::SetToggleButton(bool bIsToggleButton)
{
    m_bIsToggleButton = bIsToggleButton;
}

void CGuiButton::SetToggleState(bool bDown)
{
    assert(m_bIsToggleButton && "Error: SetToggleState called for non-toggle button!");

    if(bDown)
    {
        m_State |= GBS_DOWN;
    }
    else
    {
        m_State &= ~GBS_DOWN;
    }
}

bool CGuiButton::OnMouseKeyDown(byte bKey)
{
    if(IsVisible())
    {
        CGuiMgr *pMgr(CEngineFactory::InstancePtr()->GetGuiMgr());
        CVector2f vCursor(pMgr->GetCursorPos());
        CVector3f vAbsPos(GetAbsPos());
        if(vCursor.m_X >= vAbsPos.m_X && vCursor.m_X <= vAbsPos.m_X + GetScale().m_X &&
           vCursor.m_Y >= vAbsPos.m_Y && vCursor.m_Y <= vAbsPos.m_Y + GetScale().m_Y)
        {
            if(IsEnabled() && bKey == 0 && !(m_State & GBS_BEING_CLICKED))
            {
                m_State |= GBS_BEING_CLICKED;
                pMgr->GrabInput(this);
            }

            return true;
        }
    }

    return false;
}

bool CGuiButton::OnMouseKeyUp(byte bKey)
{
    if(IsVisible())
    {
        CGuiMgr *pMgr(CEngineFactory::InstancePtr()->GetGuiMgr());
        CVector2f vCursor(pMgr->GetCursorPos());
        CVector3f vAbsPos(GetAbsPos());
        if(vCursor.m_X >= vAbsPos.m_X && vCursor.m_X <= vAbsPos.m_X + GetScale().m_X &&
            vCursor.m_Y >= vAbsPos.m_Y && vCursor.m_Y <= vAbsPos.m_Y + GetScale().m_Y)
        {
            if(IsEnabled() && bKey == 0 && (m_State & GBS_BEING_CLICKED))
            {
                if(m_bIsToggleButton && !(m_State & GBS_DOWN))
                {
                    m_State |= GBS_DOWN;
                }
                else
                {
                    m_State &= ~GBS_DOWN;
                }

                if(m_pOnClickHandler)
                {
                    //DEBUG
                    if(m_pClickSound){
                        m_pClickSound->Play();
                    }
                    //END DEBUG

                    (*m_pOnClickHandler)(m_dwButtonId);     //Use callback functor
                }
                CEngineFactory::InstancePtr()->GetGuiMgr()->ReleaseInput(this);
                m_State &= ~GBS_BEING_CLICKED;
            }

            return true;
        }
    }

    if((m_State & GBS_BEING_CLICKED) && bKey == 0)  //Doesn't matter if the cursor is over me
    {
        CEngineFactory::InstancePtr()->GetGuiMgr()->ReleaseInput(this);
        m_State &= ~GBS_BEING_CLICKED;
    }

    return false;
}

bool CGuiButton::OnKeyDown(dword dwKey)
{
    //ToDo: Falls Enter => Wie MouseKeyUp
    //ToDo: Falls Tab => Fokus weitergeben
    return false;
}

bool CGuiButton::OnGetFocus()
{
    if(IGuiObject::OnGetFocus())
    {
        m_State |= GBS_HIGHLIGHTED;

		//DEBUG
		if(m_pRolloverSound){
			m_pRolloverSound->Play();
		}
		//END DEBUG
		
        return true;
    }

    return false;
}

void CGuiButton::OnLoseFocus()
{
	//DEBUG
	if(m_pRolloverSound){
		//m_pRolloverSound->Stop();
	}
	//END DEBUG
	
    m_State &= ~GBS_HIGHLIGHTED;
}

bool CGuiButton::OnMouseMove(float fX, float fY, sdword sdwZ)
{
    if(IsVisible())
    {
        CGuiMgr *pMgr(CEngineFactory::InstancePtr()->GetGuiMgr());
        CVector2f vCursor(pMgr->GetCursorPos());
        CVector3f vAbsPos(GetAbsPos());
        if(vCursor.m_X >= vAbsPos.m_X && vCursor.m_X <= vAbsPos.m_X + GetScale().m_X &&
           vCursor.m_Y >= vAbsPos.m_Y && vCursor.m_Y <= vAbsPos.m_Y + GetScale().m_Y)
        {
            if(IsEnabled())
            {
				pMgr->SetFocus(this);
            }
            return true;
        }
        else
        {
            if(m_State & GBS_HIGHLIGHTED)
            {
                pMgr->SetFocus(NULL);
            }
        }
    }
    return false;
}

void CGuiButton::Destroy()
{
    IGuiObject::Destroy();
    Clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
CGuiButton::CGuiButton(const TSmartPointer<ISound> &pRolloverSound,
                       const TSmartPointer<ISound> &pClickSound) : IGuiObject()
{
    Clear();
    m_pRolloverSound = pRolloverSound;
    m_pClickSound = pClickSound;
}

void CGuiButton::RenderSub()
{
    CMatrix44 scale;
    scale.Scaling(GetScale());
    CEngineFactory::InstancePtr()->GetRenderer()->SaveMatrix();
    CEngineFactory::InstancePtr()->GetRenderer()->MultMatrix(scale);

	if(((m_State & GBS_BEING_CLICKED) && (m_State & GBS_HIGHLIGHTED)) || (m_State & GBS_DOWN))
    {
        m_pDownRep->Render();
    }
    else if(m_State & GBS_HIGHLIGHTED)
    {
        m_pRollOverRep->Render();
    }
    else if(!IsEnabled())
	{
	    m_pDisabledRep->Render();
	}
	else
	{
		m_pNormalRep->Render();
	}

    CEngineFactory::InstancePtr()->GetRenderer()->RestoreMatrix();
	if(m_pText)
		m_pText->Render();

}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CGuiButton::Clear()
{
    m_State = GBS_NORMAL;
    m_pText = NULL;
    m_dwButtonId = 0;
    m_bIsToggleButton = false;
}
