/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* CGuiButton - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CGUIBUTTON_H__
#define __CGUIBUTTON_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Math/TVector2.h>
#include <Misc/Functors.h>
#include "IGuiObject.h"
#include "CGuiText.h"
#include <Renderer/IStockGeometry.h>

#include <Sound/ISound.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
enum EGuiButtonState
{
    GBS_NORMAL = 0,
    GBS_DOWN = 1,
    GBS_HIGHLIGHTED = 2,    //Flag!
    GBS_BEING_CLICKED = 4,  //Flag!
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CGuiButton : public IGuiObject
{
    friend class CGuiMgr;
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        //Status change
        virtual void SetText(const std::string &sText, dword dwColor);
        void SetToggleButton(bool bIsToggleButton); //Sets whether a Button is a toggle button
        void SetToggleState(bool bDown); //Sets the current state of a toggle button

        //Input-Events         
        virtual bool OnMouseKeyDown(byte bKey);
        virtual bool OnMouseKeyUp(byte bKey);
        virtual bool OnKeyDown(dword dwKey);

        //Events (additional to IInputHandler functions; called by GUI manager)
        virtual bool OnGetFocus();
        virtual void OnLoseFocus();
        virtual bool OnMouseMove(float fX, float fY, sdword sdwZ);

        //Handler
        inline void SetButtonId(dword dwId) {m_dwButtonId = dwId;}
		inline dword GetButtonId() const {return m_dwButtonId;}
        inline void SetOnClickHandler(TSmartPointer<IParamFunctor> pHandler) {m_pOnClickHandler = pHandler;}

        void Destroy();

        virtual ~CGuiButton() {Destroy();}

		//DEBUG
		void setNormalRep(IStockGeometry *obj){m_pNormalRep = obj;}
        void setOverRep(IStockGeometry *obj){m_pRollOverRep = obj;}
        void setDownRep(IStockGeometry *obj){m_pDownRep = obj;}
		void setDisabledRep(IStockGeometry *obj){m_pDisabledRep = obj;}
		//END DEBUG


    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        CGuiButton(const TSmartPointer<ISound> &pRolloverSound, const TSmartPointer<ISound> &pClickSound);

        virtual bool CanGetFocus() {return true;}
        virtual void RenderSub();


    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();

        //Private copy constructor and assignment operator to prevent mistakes:
        CGuiButton(const CGuiButton &);
        CGuiButton &operator =(const CGuiButton &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        byte m_State;
        dword m_dwButtonId;
        bool m_bIsToggleButton;

        TSmartPointer<IParamFunctor>       m_pOnClickHandler;
        TSmartPointer<CGuiText>       m_pText;
		TSmartPointer<IStockGeometry> m_pNormalRep;
		TSmartPointer<IStockGeometry> m_pRollOverRep;
        TSmartPointer<IStockGeometry> m_pDownRep;
		TSmartPointer<IStockGeometry> m_pDisabledRep;

        TSmartPointer<ISound> m_pRolloverSound;
        TSmartPointer<ISound> m_pClickSound;
};

#endif
