/****************************************************************************\
*                                                                            *
* 24.03.2006                                                                 *
* CGuiContainer - Source File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CGuiContainer.h"
#include <algorithm>
#include "CEngineFactory.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void CGuiContainer::Update(float dt)
{
    if(m_bResortChildren)
        ResortChildren();
    ObjectList::iterator it(m_Children.begin());
    while(it != m_Children.end())
    {
        (*it)->Update(dt);
        ++it;
    }
}

bool CGuiContainer::OnGetFocus()
{
    if(IsVisible() && IsEnabled())
    {
        if(m_bResortChildren)
            ResortChildren();
        ObjectList::iterator it(m_Children.begin());
        while(it != m_Children.end())
        {
            if((*it)->OnGetFocus())
            {
                CEngineFactory::InstancePtr()->GetGuiMgr()->SetFocus((*it));
                return false;
            }
            ++it;
        }
    }
    return false;
}

bool CGuiContainer::OnMouseMove(float fX, float fY, sdword sdwZ)
{
    if(IsVisible())
    {
        if(m_bResortChildren)
            ResortChildren();
        ObjectList::iterator it(m_Children.begin());
        while(it != m_Children.end())
        {
            if((*it)->OnMouseMove(fX, fY, sdwZ))
            {
                return true;
            }
            ++it;
        }
    }
    return false;
}

bool CGuiContainer::OnMouseKeyDown(byte bKey)
{
    if(IsVisible())
    {
        if(m_bResortChildren)
            ResortChildren();
        ObjectList::iterator it(m_Children.begin());
        while(it != m_Children.end())
        {
            if((*it)->OnMouseKeyDown(bKey))
            {
                return true;
            }
            ++it;
        }
    }
    return false;
}

bool CGuiContainer::OnMouseKeyUp(byte bKey)
{
    if(IsVisible())
    {
        if(m_bResortChildren)
            ResortChildren();
        ObjectList::iterator it(m_Children.begin());
        while(it != m_Children.end())
        {
            if((*it)->OnMouseKeyUp(bKey))
            {
                return true;
            }
            ++it;
        }
    }
    return false;
}

bool CGuiContainer::OnChar(char c)
{
    if(m_bResortChildren)
        ResortChildren();
    ObjectList::iterator it(m_Children.begin());
    while(it != m_Children.end())
    {
        if((*it)->OnChar(c))
        {
            return true;
        }
        ++it;
    }
    return false;
}

bool CGuiContainer::OnKeyDown(dword dwKey)
{
    if(m_bResortChildren)
        ResortChildren();
    ObjectList::iterator it(m_Children.begin());
    while(it != m_Children.end())
    {
        if((*it)->OnKeyDown(dwKey))
        {
            return true;
        }
        ++it;
    }
    return false;
}

bool CGuiContainer::OnKeyUp(dword dwKey)
{
    if(m_bResortChildren)
        ResortChildren();
    ObjectList::iterator it(m_Children.begin());
    while(it != m_Children.end())
    {
        if((*it)->OnKeyUp(dwKey))
        {
            return true;
        }
        ++it;
    }
    return false;
}

void CGuiContainer::AddChild(const TSmartPointer<IGuiObject> &pObject)
{
    pObject->SetParent(this);
    m_Children.push_back(pObject);
    m_bResortChildren = true;
}

void CGuiContainer::RemoveChild(const TSmartPointer<IGuiObject> &pObject)
{
    ObjectList::iterator it(find(m_Children.begin(), m_Children.end(), pObject));
    if(it != m_Children.end())
    {
        (*it)->SetParent(NULL);
        m_Children.erase(it);
    }
}

bool CGuiContainer::Contains(const TSmartPointer<IGuiObject> &pObject) const
{
    if(IGuiObject::Contains(pObject))
        return true;

    ObjectList::const_iterator it(find(m_Children.begin(), m_Children.end(), pObject));
    if(it != m_Children.end())
        return true;

    return false;
}

void CGuiContainer::Destroy()
{
    IGuiObject::Destroy();
    m_Children.clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
struct SGuiObjSort
{
    bool operator()(const TSmartPointer<IGuiObject> &pFirst, const TSmartPointer<IGuiObject> &pSecond)
    {
        return pFirst->GetPos().m_Z > pSecond->GetPos().m_Z;
    }
};

void CGuiContainer::ResortChildren()
{
    sort(m_Children.begin(), m_Children.end(), SGuiObjSort());
    m_bResortChildren = false;
}

void CGuiContainer::RenderSub()
{
    if(m_bResortChildren)
        ResortChildren();
    ObjectList::reverse_iterator it(m_Children.rbegin());
    while(it != m_Children.rend())
    {
        (*it)->Render();
        ++it;
    }
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CGuiContainer::Clear()
{
    m_bResortChildren = false;
}
