/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* IGuiObject - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "IGuiObject.h"
#include "CEngineFactory.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
void IGuiObject::Render()
{
    if(IsVisible())
    {
        CEngineFactory::InstancePtr()->GetRenderer()->SetMatrixMode(EMM_MODELVIEW);
        CEngineFactory::InstancePtr()->GetRenderer()->SaveMatrix();
         //Debug
        CMatrix44 trans, rot;
        trans.Translation(GetPos());
        rot.RotationZXY(GetRot());
        CEngineFactory::InstancePtr()->GetRenderer()->MultMatrix(trans * rot);
        //End debug
        
        RenderSub();

        CEngineFactory::InstancePtr()->GetRenderer()->RestoreMatrix();
    }
}

void IGuiObject::Show(bool bShow)
{
    m_bVisible = bShow;
}

void IGuiObject::Enable(bool bEnable)
{
    m_bEnabled = bEnable;
}

void IGuiObject::SetParent(IGuiObject *pParent)
{
    m_pParent = pParent;
}

bool IGuiObject::OnGetFocus()
{
    return IsVisible() && IsEnabled() && CanGetFocus();
}

void IGuiObject::Destroy()
{
    Clear();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void IGuiObject::Clear()
{
    m_pParent = NULL;
    m_bVisible = true;
    m_bEnabled = true;
    SetPos(CVector3f(0.0f, 0.0f, 1.0f));
}