/****************************************************************************\
*                                                                            *
* 24.02.2006                                                                 *
* IGuiObject - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __IGUIOBJECT_H__
#define __IGUIOBJECT_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Math/TVector3.h>
#include <Input/IInputHandler.h>
#include <Memory/TSmartPointer.h>
#include <Memory/IManagedObject.h>
#include <Display/CTransformableObj.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IGuiObject : public CTransformableObj, public IManagedObject, public IInputHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        virtual void Update(float dt) {}
        void Render();

        //Status change
        virtual void SetPos(const CVector3f &vPos) {CTransformableObj::SetPos(vPos); if(m_pParent){m_pParent->OnChildPosUpdate();}}
        virtual void Show(bool bShow);
        virtual void Enable(bool bEnable);
        virtual void SetParent(IGuiObject *pParent);

        //Status request
        inline bool IsVisible() const {return m_bVisible;}
        inline bool IsEnabled() const {return m_bEnabled;}
        inline CVector3f GetAbsPos() const {if(m_pParent) {return GetPos() + m_pParent->GetAbsPos();} return GetPos();}

        //Events (additional to IInputHandler functions; called by GUI manager)
        virtual bool OnGetFocus();
        virtual void OnLoseFocus() {}
        virtual bool OnMouseMove(float fX, float fY, sdword sdwZ) {return false;} //Erstetzt die IInputHandler-Funktion, da X und Y durch das virtuelle Koordinatensystem zu floats werden
        
        //Graph management
        virtual void AddChild(const TSmartPointer<IGuiObject> &pObject) {}
        virtual void RemoveChild(const TSmartPointer<IGuiObject> &pObject) {}
        virtual bool Contains(const TSmartPointer<IGuiObject> &pObject) const {return this == pObject;} //Standard-Objekt beinhaltet nur sich selbst
        virtual void OnChildPosUpdate() {};

        virtual void Destroy();
                                                          
        virtual ~IGuiObject() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        IGuiObject() {Clear();}

        virtual bool CanGetFocus() {return false;}
        virtual void RenderSub() = 0;
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();

        //Private copy constructor and assignment operator to prevent mistakes:
        IGuiObject(const IGuiObject &);
        IGuiObject &operator =(const IGuiObject &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        IGuiObject *m_pParent;
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        bool m_bVisible;
        bool m_bEnabled;
};

#endif
