/****************************************************************************\
*                                                                            *
* 19.03.2006                                                                 *
* IRepresentation - Header File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __IREPRESENTATION_H__
#define __IREPRESENTATION_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>
#include <Math/TVector3.h>
#include <Math/TAABox3.h>
#include <Math/TSphere.h>
#include "CTransformableObj.h"
#include "Camera/CFrustum.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class ISceneGraph;
class IGameObj;

template<class T>
class TSceneNode;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IRepresentation : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		IRepresentation() : m_pParentNode(NULL), m_pSceneGraph(NULL), m_bSphereValid(false), m_bBoxValid(false), m_bIsPickable(true) {}

        virtual void Create(ISceneGraph *pSceneGraph, const CSpheref &boundingSphere, const CAABox3f &boundingBox)
                            {m_pSceneGraph = pSceneGraph; m_BoundingSphere = boundingSphere; m_AABoundingBox = boundingBox;}

        virtual const CVector3f &GetPos() const = 0;
        virtual const CVector3f &GetRot() const = 0;
        virtual const CVector3f &GetScale() const = 0;
        virtual const CMatrix44 &GetTransformation() = 0;

        inline bool IsPickable() const {return m_bIsPickable;}
        inline void SetPickable(bool bIsPickable) {m_bIsPickable = bIsPickable;}

        virtual IGameObj * const GetGameObj() const = 0;

        virtual void ObjTransformationChanged();

        const CSpheref &GetBoundingSphere();
        const CBox3f   &GetBoundingBox();
        inline const CAABox3f &GetOriginalBoundingBox() const {return m_AABoundingBox;}

        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const = 0;

        void SetParentNode(TSceneNode<IRepresentation> *pNode) {m_pParentNode = pNode;}

        virtual void Render(const CFrustum &frustum, bool bVerify);

        virtual void Destroy();

        virtual ~IRepresentation() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void RenderSub(const CFrustum &frustum, bool bVerify) = 0;
        //Private copy constructor and assignment operator to prevent mistakes:
        IRepresentation(const IRepresentation &);
        IRepresentation &operator =(const IRepresentation &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        ISceneGraph *m_pSceneGraph;
        bool m_bIsPickable;

        TSceneNode<IRepresentation> *m_pParentNode;
        CSpheref m_BoundingSphere;
        CAABox3f m_AABoundingBox;

        CSpheref m_CurrBoundingSphere;
        bool m_bSphereValid;
        CBox3f m_CurrBoundingBox;
        bool m_bBoxValid;

		/**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:

    //Debug
    public:
        static dword ms_dwRendered;
        static dword ms_dwVerified;
        static dword ms_dwBothTests;
        
};

#endif
