
/****************************************************************************\
*                                                                            *
* 15.03.2006                                                                 *
* CIniFile - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <boost/spirit.hpp>
#include "CIniFile.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
using namespace std;
using namespace boost;
using namespace spirit;

int ParseIniFile(char * buffer, CategoryList & file_data);

/**************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                   *
\**************************************************************************/
CIniFile::CIniFile(){
	m_status = 0;
}

void CIniFile::ClearAll(){
	m_file.clear();
	m_status = 0;
}

void CIniFile::LoadFile(string filename){
	m_file.clear();

	if (filename.empty()){
		m_status = 1;
		return;
	}

	FILE * fin = fopen(filename.c_str(),"rt");

	if (fin){
		fseek(fin,0, SEEK_END);
		int filesize = ftell(fin);
		fseek(fin,0, SEEK_SET);
		char * buffer = new char [filesize + 1];
        ZeroMemory(buffer, sizeof(char) * (filesize + 1));
		fread(buffer, filesize, 1, fin);
		fclose(fin);
		int ec = ParseIniFile(buffer, m_file);
		if (!ec)
			m_status = 2;

		delete [] buffer;
	}else{
		m_status = 1;
		printf("Could not open input file %s\n", filename.c_str());
	}
}

void CIniFile::SaveToFile(string filename){
	
	FILE * fout = fopen(filename.c_str(),"wt");

	if (fout){
		unsigned int c;
		for (c=0;c<m_file.size();c++){
			fprintf(fout,"[%s]\n",m_file[c].name.c_str());
			unsigned int i;
			for (i=0;i<m_file[c].entries.size();i++){
				fprintf(fout,"%s=%s\n", m_file[c].entries[i].name.c_str(), m_file[c].entries[i].value.c_str());
			}
			fprintf(fout,"\n");
		}
		fclose(fout);
	}else{
		printf("Could not open output file %s\n", filename.c_str());
	}
}

int CIniFile::GetNumCategories(){
	return (int)m_file.size();
}

void CIniFile::SetCategory(int cat){
	m_crt_category = cat;
}

int CIniFile::SetCategory(string name){
	int i;
	for (i=0;i< (int)m_file.size();i++){
		if (m_file[i].name == name){
			m_crt_category = i;
			return 1;
		}
	}
	return 0;
}

void CIniFile::AddCategoryUnique(string name){
	if ( SetCategory(name) )
		return;

	Category cat;
	cat.name = name;
	m_file.push_back(cat);
	m_crt_category = (int)m_file.size() - 1;
}

void CIniFile::AddCategory(string name){
	Category cat;
	cat.name = name;
	m_file.push_back(cat);
	m_crt_category = (int)m_file.size() - 1;
}

int CIniFile::GetNumEntries(){
	if ( ! b_CategoryIsValid() ){
		return 0;
	}
	
	return (int)m_file[m_crt_category].entries.size();
}

void CIniFile::SetEntry(int entry){
	m_crt_entry = entry;
}

int CIniFile::SetEntry(string name){
	if ( ! b_CategoryIsValid() ){
		return 0;
	};

	int i;
	for (i=0;i< (int)m_file[m_crt_category].entries.size();i++){
		if (m_file[m_crt_category].entries[i].name == name){
			m_crt_entry = i;
			return 1;
		};
	};
	return 0;
}

int CIniFile::SetEntryByValue(string value){
	if ( ! b_CategoryIsValid() ){
		return 0;
	};

	int i;
	for (i=0;i< (int)m_file[m_crt_category].entries.size();i++){
		string cval = m_file[m_crt_category].entries[i].value;
		if (cval == value){
			m_crt_entry = i;
			return 1;
		};
	};
	return 0;
}

void CIniFile::ChangeEntry(string name, string value){
	if ( !SetEntry(name) )
		return;
	
	m_file[m_crt_category].entries[m_crt_entry].value = value;
}

void CIniFile::SetValue(string value){
	if ( ! b_EntryIsValid() ){
		return ;
	};
	
	m_file[m_crt_category].entries[m_crt_entry].value = value;
}

void CIniFile::AddEntryUnique(string name, string value){
	
	if ( SetEntry(name) )
		return;
	
	Entry entry;
	entry.name = name;
	entry.value = value;
	m_file[m_crt_category].entries.push_back(entry);
	m_crt_entry = (int)m_file[m_crt_category].entries.size() - 1;
}

void CIniFile::AddEntry(string name, string value){
	Entry entry;
	entry.name = name;
	entry.value = value;
	m_file[m_crt_category].entries.push_back(entry);
	m_crt_entry = (int)m_file[m_crt_category].entries.size() - 1;
}

string CIniFile::GetCategoryName(){
	if ( ! b_CategoryIsValid() ){
		return "";
	};

	return m_file[m_crt_category].name;
}

string CIniFile::GetEntryName(){
	if ( ! b_EntryIsValid() ){
		return "";
	};

	return m_file[m_crt_category].entries[m_crt_entry].name ;
}

string CIniFile::GetEntryValue(){
	if ( ! b_EntryIsValid() ){
		return "";
	};

	return m_file[m_crt_category].entries[m_crt_entry].value ;
}

string CIniFile::GetEntryValueByName(string name){
	if(SetEntry(name))
	{
		return GetEntryValue();
	}
	return "";
}

void CIniFile::DeleteEntry(string name){
	if ( SetEntry(name) ){
		DeleteEntry(m_crt_entry);
	}
}

void CIniFile::DeleteEntry(int entry){
	if ( ! b_EntryIsValid() ){
		return ;
	};
	vector<Entry> & entries = m_file[m_crt_category].entries;
	entries.erase( entries.begin() + m_crt_entry );
}

void CIniFile::DeleteByValue(string value){
	if ( SetEntryByValue(value) ){
		DeleteEntry(m_crt_entry);
	}
}

void CIniFile::Print(){
	int i;
	for (i=0;i<GetNumCategories();i++){
		SetCategory(i);
		printf("Cat %d = [%s]\n", i,GetCategoryName().c_str());
		int j;
		for (j=0;j<GetNumEntries();j++){
			SetEntry(j);
			printf("Entry %d [%s]=[%s]\n", j,GetEntryName().c_str(),GetEntryValue().c_str());
		}
		printf("\n");
	}
}

/**************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                 *
\**************************************************************************/
int CIniFile::b_CategoryIsValid(){
	if ( ( m_crt_category >= 0 ) && ( m_crt_category < (int)m_file.size() ) ){
		return 1;
	}
	else{
		return 0;
	}

}

int CIniFile::b_EntryIsValid(){
	if (b_CategoryIsValid()){
		if ( ( m_crt_entry >= 0) && ( m_crt_entry < (int)m_file[m_crt_category].entries.size() ) ){
			return 1;
		}else{
			return 0;
		}
	}else{
		return 0;
	}
}

/**************************************************************************\
* GLOBAL FUNCTION  - PARSER                                                *
\**************************************************************************/
int ParseIniFile(char * buffer, CategoryList & file_data)
{
	string name, value;

	AddCategory addCategory;
	addCategory.m_file = &file_data;

	AddName addName;
	addName.m_file = &file_data;

	AddValue addValue;
	addValue.m_file = &file_data;

	rule<> char_ident_start = alpha_p | ch_p('_') ;
	rule<> char_ident_middle = alnum_p | ch_p('_') ;
	rule<> ident = char_ident_start >> * char_ident_middle ;
	rule<> char_start_comment = ch_p('#') | ch_p(';') | str_p("//");
	rule<> blanks_p = * blank_p;
	rule<> value_p = * ( alnum_p | blank_p | punct_p );

	rule<> l_category = 
		blanks_p >> 
		ch_p('[') >> 
		blanks_p >> 
		ident [ addCategory ] >>
		blanks_p >> 
		ch_p(']') >> 
		blanks_p >> 
		eol_p
		;

	
	rule<> l_comment = blanks_p >> char_start_comment >> * print_p >> eol_p; 
	rule<> l_empty = blanks_p >> eol_p; 
		
	//statt dem Auskommentierten
	rule<> l_blockCommentStart = ch_p('/') >> ch_p('*');
	rule<> l_blockCommentEnd  = ch_p('*') >> ch_p('/');
	rule<> l_blockComment = l_blockCommentStart >> * anychar_p >> l_blockCommentEnd >> eol_p;
	rule<> c_comment_rule = l_blockComment;
	//
	//rule<> c_comment_rule = blanks_p >>  confix_p("/*", *anychar_p, "*/");
	//

	rule<> b_comment = 
		blanks_p >> 
		c_comment_rule >>
		blanks_p >> 
		eol_p
		; 

	rule<> l_entry =  
		blanks_p >>
		ident [ addName ] >> 
		blanks_p >>
		ch_p('=') >> 
		blanks_p >>
		value_p [ addValue ] >> 
		blanks_p >>
		eol_p
		; 

	rule<> lines    = l_comment | b_comment | l_category | l_entry | l_empty;
	rule<> ini_file =  lexeme_d [ * lines ] ;

	file_data.clear();

	int errcode = parse(buffer,ini_file).full;

	return errcode;
}