/****************************************************************************\
*                                                                            *
* 02.07.2003                                                                 *
* CStdFile - Header File                                                     *
*                                                                            *
* CStdFile is just a simple wrapper for file access                          *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __CSTDFILE_H__
#define __CSTDFILE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>

/****************************************************************************\
* DEFINITIONS	                                                               *
\****************************************************************************/
enum EAccess
{
	ACCESS_READ_ONLY  = GENERIC_READ,
	ACCESS_WRITE_ONLY = GENERIC_WRITE,
	ACCESS_READ_WRITE = GENERIC_READ | GENERIC_WRITE,
};

enum ESeek
{
	SEEK_FROM_BEGIN   = FILE_BEGIN,
	SEEK_FROM_CURRENT = FILE_CURRENT,
	SEEK_FROM_END     = FILE_END,
};

enum ESharing
{
	SHARE_NONE       = 0,
	SHARE_READ_ONLY  = FILE_SHARE_READ,
	SHARE_WRITE_ONLY = FILE_SHARE_WRITE,
	SHARE_READ_WRITE = FILE_SHARE_READ | FILE_SHARE_WRITE,
};

enum ECreation
{
	CREATION_NEW               = CREATE_NEW,
	CREATION_ALWAYS            = CREATE_ALWAYS,
	CREATION_OPEN_EXISTING     = OPEN_EXISTING,
	CREATION_OPEN_ALWAYS       = OPEN_ALWAYS,
	CREATION_TRUNCATE_EXISTING = TRUNCATE_EXISTING,
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CStdFile
{
  /**************************************************************************\
  * PRIVATE CLASS MEMBERS                                                    *
  \**************************************************************************/
  private:
    
  /**************************************************************************\
  * PROTECTED CLASS MEMBERS                                                  *
  \**************************************************************************/
  protected:
  	HANDLE m_hFileHandle;
    
  /**************************************************************************\
  * PUBLIC CLASS MEMBERS                                                     *
  \**************************************************************************/
  public:
    
  /**************************************************************************\
  * PRIVATE CLASS FUNCTIONS                                                  *
  \**************************************************************************/
  private:
      void Clear() {m_hFileHandle = INVALID_HANDLE_VALUE;}
   
  /**************************************************************************\
  * PROTECTED CLASS FUNCTIONS                                                *
  \**************************************************************************/
  protected:
    
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    CStdFile() {Clear();}

    bool Create(const std::string &sFileName, EAccess access = ACCESS_READ_ONLY, ECreation creation = CREATION_OPEN_EXISTING, ESharing sharing = SHARE_READ_ONLY,
                dword dwFlags = FILE_ATTRIBUTE_NORMAL);

    inline bool IsOpen() const {return m_hFileHandle != INVALID_HANDLE_VALUE;}
    dword GetSize() const;

    bool Seek(sdword sdwOffset, ESeek seek = SEEK_FROM_CURRENT);
    dword GetSeekPos();

    dword Read(void *pBuffer, dword dwSize) const;

    dword Write(const void *pBuffer, dword dwSize);
    dword Print(const char* sText, ...);

    virtual void Destroy();
    virtual ~CStdFile() {Destroy();}
};

#endif
