/****************************************************************************\
*                                                                            *
* 21.02.2006                                                                 *
* IInputMgr - Header File                                                    *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __IINPUTMGR_H__
#define __IINPUTMGR_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <list>
#include <algorithm>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Memory/IManagedObject.h>
#include <Display/Windows/IWindow.h>
#include "IInputHandler.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IInputMgr : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        IInputMgr() {};
        
        virtual bool Create(HINSTANCE hInstance, const TSmartPointer<IWindow> &pWindow) = 0;
        virtual void Update() = 0;
        virtual void GetCursorPos(sdword &sdwPosX, sdword &sdwPosY) const = 0;  //Get mouse cursor position (relative to given window)
        virtual void GetNormalizedCursorPos(float &fPosX, float & fPosY) const = 0; //Get mouse cursor position normalized by the window size (==> between 0 and 1)
        virtual void LockCursor(bool bLock) = 0;
        virtual void ShowCursor(bool bShow) = 0;
        virtual void Destroy() {m_InputHandlers.clear();}

        virtual void RegisterHandler(IInputHandler *pHandler)
        {
            m_InputHandlers.push_back(pHandler);
            SortHandlers();
        }

        virtual void UnregisterHandler(IInputHandler *pHandler)
        {
            HandlerListIterator i(m_InputHandlers.begin());

            while(i != m_InputHandlers.end() && !((*i) == pHandler))
            {
                ++i;
            }
            if(i != m_InputHandlers.end())
                m_InputHandlers.erase(i);
        };

        virtual void SortHandlers() //Needs only to be calles if the priority of a handler has changed since insertion
        {
            m_InputHandlers.sort(SInputHandlerSort());
        }

        virtual ~IInputMgr() {Destroy();};
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        IInputMgr(const IInputMgr &);
        IInputMgr &operator =(const IInputMgr &);
        
        struct SInputHandlerSort
        {
            bool operator()(IInputHandler *&pFirst, IInputHandler *&pSecond)
            {
                return pFirst->GetPriority() < pSecond->GetPriority();
            }
        };

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        typedef std::list<IInputHandler *> HandlerList;
        typedef HandlerList::iterator HandlerListIterator;

        HandlerList m_InputHandlers;
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
};

#endif
