/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* CFBXLoader - Source File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CFBXLoader.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
using namespace std;
using namespace FBXSDK_NAMESPACE;

#define prefix_shader "s_"

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

//Loading a file - repeated Load will destroy all former content
bool CFBXLoader::Load(const std::string & fileName){
	if(m_bInitialized){
		if(m_pScene != NULL){
			m_pSdkManager->DestroyKFbxScene(m_pScene,true);
			m_pScene      = NULL;
			m_pRootNode   = NULL;
			m_pMeshNode   = NULL;
		}
		if(m_pScene = m_pSdkManager->CreateKFbxScene()){
			if(LoadScene(m_pScene,fileName)){
				if(m_pRootNode  = m_pScene->GetRootNode()){
					return true;
				}
			}else{
				m_pSdkManager->DestroyKFbxScene(m_pScene,true);
				m_pScene = NULL;
				return false;
			}
		}
	}
	return false;
}

//Loading a file - repeated Load will destroy all former content
bool CFBXLoader::LoadAndTriangulate(const std::string & fileName){
	m_bTriangulate = true;
	return Load(fileName);
}

//GeometryInfo
bool CFBXLoader::GetGeometryInfo(TSmartPointer<CGeometryInfo> const gPtr){
	if(m_pScene != NULL){
		
		m_pGeoInfo = gPtr;

		KFbxNode *tmp = traverse(&CFBXLoader::getMeshNode,"");
		
		if(tmp){
			m_pMeshNode = tmp->GetMesh();
			if(buildGeometry()){

				/*KArrayTemplate<KFbxPose *> poseObjects;
				KArrayTemplate<int> indicesInPoseObjects;
				if(KFbxPose::GetBindPoseContaining(*m_pSdkManager,m_pMeshNode->GetNode()->GetName(),poseObjects,indicesInPoseObjects)){
					
					for(int i = 0; i < poseObjects.GetCount() ; i++){

						LOG(TL_ERROR,poseObjects.GetAt(i)->GetName());
						printf("%s%d%s\n","MeshNode is the ",indicesInPoseObjects.GetAt(i)," Member in this PoseNode");
						printf("%s\n",poseObjects.GetAt(i)->IsBindPose() == true ? "bindPose" : "restPose" );
						for(int j = 0 ; j < poseObjects.GetAt(i)->GetCount() ; j++){
							printf("%s%s\n","Member: ",poseObjects.GetAt(i)->GetNodeName(j));
						}
						LOG(TL_ERROR,"----------------------------");


					}
				};*/

				/*printf("%d",m_pMeshNode->GetLinkCount());
				for(int i = 0 ; i < m_pMeshNode->GetLinkCount() ; i++){
					printf("%s\n",m_pMeshNode->GetLink(i)->GetLink()->GetName());
					KFbxNodeAttribute * nAttr = m_pMeshNode->GetLink(i)->GetLink()->GetNodeAttribute();
					KFbxNodeAttribute::EAttributeType eAttrType = nAttr->GetAttributeType();
					if(eAttrType == KFbxNodeAttribute::eSKELETON){
						KFbxSkeleton * sk = m_pMeshNode->GetLink(i)->GetLink()->GetSkeleton();
						KFbxSkeleton::ESkeletonType eType = sk->GetSkeletonType();
						
						int i = 5;
					}
						

					KFbxVector4 vec;
					m_pMeshNode->GetLink(i)->GetTransformAssociateModelT(vec);
					if(m_pMeshNode->GetLink(i)->GetAssociateModel())
						printf("%s\n",m_pMeshNode->GetLink(i)->GetAssociateModel()->GetName());
					int * ind = m_pMeshNode->GetLink(i)->GetControlPointIndices();
					double * weights = m_pMeshNode->GetLink(i)->GetControlPointWeights();
					KFbxLink::ELinkMode mode = m_pMeshNode->GetLink(i)->GetLinkMode();
					for(int j = 0 ; j < m_pMeshNode->GetLink(i)->GetControlPointIndicesCount() ; j++){
						printf("%d%s%f%s",ind[j],"\\",weights[j],"||");
					}
					printf("\n");
				}*/

				//KArrayTemplate<KString *> takeNames;
				//m_pScene->FillTakeNameArray(takeNames);

				//for(int i = 0 ; i < takeNames.GetCount() ; i++){
				//	LOG(TL_ERROR,takeNames[i]->Buffer());
				//}
			
				//if(tmp->SetCurrentTakeNode(takeNames[1]->Buffer())){
				//	char * name = tmp->GetCurrentTakeNodeName();
				//	if(tmp->GetCurrentTakeNode()->GetTranslationX){
               //         KFbxTakeInfo *info = m_pScene->GetTakeInfo(name);
				//	    int nmbOfKeys = tmp->GetCurrentTakeNode()->GetTranslationX()->KeyGetCount();
				//	    int h = 5;h+=6;
				//	}
				//}
				

				//val=tmp->GetTakeNode(0)->GetEulerRotationY()->KeyGet(0).GetValue();
				//val=tmp->GetTakeNode(0)->GetEulerRotationY()->KeyGet(1).GetValue();
				//val=tmp->GetTakeNode(0)->GetEulerRotationY()->KeyGet(2).GetValue();

				

				return true;
			}else{
				return false;
			}
		}
	}

	return false;
}

//AnimationInfo
bool CFBXLoader::GetAnimationInfo(TSmartPointer<CAnimationInfo> const aPtr){
	if(m_pScene != NULL){
		m_pAniInfo = aPtr;
		return true;
	}
	return false;
}

//SkeletonInfo
bool CFBXLoader::GetSkeletonInfo(TSmartPointer<CSkeletonInfo> const sPtr){
	if(m_pScene != NULL){
		m_pSkelInfo = sPtr;
		return true;
	}
	return false;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

//Constructor - initializes fbx
CFBXLoader::CFBXLoader(void){
	m_pImporter      = NULL;
	m_pGeomConverter = NULL;
	m_pScene         = NULL;
	m_pRootNode      = NULL;
	m_pMeshNode      = NULL;
	m_bTriangulate   = false;

	m_pSdkManager = KFbxSdkManager::CreateKFbxSdkManager();
	
	if (!m_pSdkManager){
		LOG(TL_ERROR,"FBX-SDK INIT Failed in CFBXLoader::CFBXLoader(void)");
		m_bInitialized = false;
	}else{
		m_pImporter      = m_pSdkManager->CreateKFbxImporter();
		m_pGeomConverter = m_pSdkManager->CreateKFbxGeometryConverter();
		m_bInitialized   = true;
	}
}

//Destructor - frees fbx
CFBXLoader::~CFBXLoader(void){
	m_pSdkManager->DestroyKFbxScene(m_pScene,true);
	m_pSdkManager->DestroyKFbxImporter(m_pImporter);
	m_pSdkManager->DestroyKFbxGeometryConverter(m_pGeomConverter);
	SAFE_DELETE(m_pSdkManager);
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

//INFO
//contrlpoints - must be defined on layer 0
//normals      - must be defined on layer 0
//vertexColors - must be defined on layer 0
//materials    - must be defined on layer 0 

//GetGeometryInfo Implementation
bool CFBXLoader::buildGeometry(){

	bool bValid = false;

	if(m_bTriangulate){
		m_pMeshNode = m_pGeomConverter->TriangulateMesh(m_pMeshNode);
		m_bTriangulate = false;
	}

	//*********************************************************************************
	// Basic check - if no controlpoints and normals, return false
	//*********************************************************************************
	KFbxVector4 *fbx_pControlPoints = m_pMeshNode->GetControlPoints();
	KFbxVector4 *fbx_pNormals = m_pMeshNode->GetNormals();

	if(fbx_pControlPoints){
		if(fbx_pNormals){
			//check if normals are mapped as being implemented
			KFbxLayerElementNormal *fbx_le_pAllNormals = m_pMeshNode->GetLayer(0)->GetNormals();
			if(fbx_le_pAllNormals->GetMappingMode() == KFbxLayerElement::eBY_CONTROL_POINT) 
				if(fbx_le_pAllNormals->GetReferenceMode() == KFbxLayerElement::eDIRECT)
					bValid = true;
		}
	}

	if(!bValid){
		return false;
	}

	//*********************************************************************************
	// Basic check - if no material assigned return false, else copy materials
	//*********************************************************************************
	std::vector<int> materialLookup;
	KFbxLayerElement::EMappingMode mat_referenceMode;
	

	KFbxLayerElementMaterial *fbx_le_pAllMat = m_pMeshNode->GetLayer(0)->GetMaterials();
	if(fbx_le_pAllMat){

        //polygons are assigned one or more materials
		if(fbx_le_pAllMat->GetMappingMode() == KFbxLayerElement::eBY_POLYGON){
			if(fbx_le_pAllMat->GetReferenceMode()== KFbxLayerElement::eINDEX_TO_DIRECT){

				mat_referenceMode = KFbxLayerElement::eBY_POLYGON;

				//in case there are more than one materials
				SMaterial insertMaterial;
				int fbx_nmbOfMaterials = fbx_le_pAllMat->GetDirectArray().GetCount();
				for (int m = 0 ; m < fbx_nmbOfMaterials ; m++){
					convertKFbxMaterial(&insertMaterial,fbx_le_pAllMat->GetDirectArray().GetAt(m));
					materialLookup.push_back(m_pGeoInfo->Insert(insertMaterial));
				}

			}
		}
		
		//all polygons share the same material - maya exports it like this
		if(fbx_le_pAllMat->GetMappingMode() == KFbxLayerElement::eALL_SAME){
			if(fbx_le_pAllMat->GetReferenceMode()== KFbxLayerElement::eINDEX_TO_DIRECT){

				mat_referenceMode = KFbxLayerElement::eALL_SAME;
				
				//in case there is only one material - maya saves it like this
				SMaterial insertMaterial;
				convertKFbxMaterial(&insertMaterial,fbx_le_pAllMat->GetDirectArray().GetAt(0));
				materialLookup.push_back(m_pGeoInfo->Insert(insertMaterial));

			}
		}

        //all polygons share the same material - motionbuilder exports it like this
		if(fbx_le_pAllMat->GetMappingMode() == KFbxLayerElement::eALL_SAME){
			if(fbx_le_pAllMat->GetReferenceMode()== KFbxLayerElement::eINDEX){

				mat_referenceMode = KFbxLayerElement::eALL_SAME;
				
				KArrayTemplate<KFbxMaterial *> fbx_materials;
				m_pScene->FillMaterialArray(fbx_materials);
				int fbx_nmbOfMaterials = fbx_materials.GetCount();

				if(!fbx_nmbOfMaterials){
					//model has no materials defined
					return false;
				}

				SMaterial insertMaterial;
				convertKFbxMaterial(&insertMaterial,fbx_materials.GetFirst());
				materialLookup.push_back(m_pGeoInfo->Insert(insertMaterial));

			}
		}

	}else{
		//model has no materials defined
		return false;
	}

	//********************************************************************************
	//check the actual number of GeoGroups
	//********************************************************************************
	int nmbOfGeoGroups = (int)m_pGeoInfo->m_Materials.size(); 

	//********************************************************************************
	//check if the mesh has VertexColors
	//********************************************************************************
	bool bHasVertexColors = false;

	KFbxLayerElementVertexColor *fbx_le_allColors = m_pMeshNode->GetLayer(0)->GetVertexColors();
	if(fbx_le_allColors){
		if(fbx_le_allColors->GetMappingMode() == KFbxLayerElement::eBY_POLYGON_VERTEX){
			if(fbx_le_allColors->GetReferenceMode() == KFbxLayerElement::eINDEX_TO_DIRECT){
				bHasVertexColors = true;
			}
		}
	}
	
	//********************************************************************************
	//check if the mesh has valid TextureInfo 
	//********************************************************************************
	bool bHasTextureInfo  = false;
	int  nmbOfTexLayers = m_pMeshNode->GetLayerCount(KFbxLayerElement::eTEXTURE);
	int  nmbOfUvLayers  = m_pMeshNode->GetLayerCount(KFbxLayerElement::eUV);

	if(nmbOfUvLayers > 0 && nmbOfTexLayers > 0 && nmbOfTexLayers == nmbOfUvLayers){
		bHasTextureInfo = true;
	}

	//********************************************************************************
	//loop over all polygons
	//********************************************************************************	
    int polygonCount  = m_pMeshNode->GetPolygonCount();

	int materialId;
	int geoGroupId;

	int *geoGroupLookup = new int[nmbOfGeoGroups];
	int *nmbOfActTexLayersInGeoGroup = new int[nmbOfGeoGroups];
	for(int i = 0 ; i < nmbOfGeoGroups ; i++){
		geoGroupLookup[i] = -1;
		nmbOfActTexLayersInGeoGroup[i] = 0;
	} 
	
	int vertexID = 0;
	int controlPointIndex;
	int index;
	int p,v;

	KFbxVector4 *fbxCoord;
	KFbxVector4 *fbxNormal;
	KFbxColor   *fbxColor;
	KFbxVector2 *fbxUv;
	CVector3f    Vcoord;
	CVector3f    Vnormal;
	CVector4f    Vcolor;
	CVector2f   *VUvs = new CVector2f[nmbOfUvLayers];

	KFbxLayerElementUV *fbx_le_UvPtr;
    bool bGroupCreated;
	
	//for every polygon	
	for(p = 0 ; p < polygonCount ; p++)
    {
		//check to which geoGroup this Triangle belongs
		if(mat_referenceMode == KFbxLayerElement::eBY_POLYGON){
			materialId = materialLookup[fbx_le_pAllMat->GetIndexArray().GetAt(p)];
		}else{
			materialId = materialLookup[fbx_le_pAllMat->GetIndexArray().GetAt(0)];
		}
		geoGroupId = geoGroupLookup[materialId];
		//********************************************************************************
		//If this Polygon has a MaterialIndex assigned not yet seen before - we create a new geoGroup 
		//all Polygons with the same MaterialIndex will be pushed into this Group
		//further - Material and TextureInfo found on this very first Polygon will  
		//be extracted and is representative for the whole Group
		//********************************************************************************
		if(geoGroupId < 0)
        {
            bGroupCreated = false;
			//dont do the stuff again for this group
			geoGroupLookup[materialId] = geoGroupId = (int)(m_pGeoInfo->m_pGeoGroups.size());

			//if there is valid TextureInfo
			if(bHasTextureInfo)
            {
				KFbxLayerElementTexture *fbx_le_TexPtr;
				std::vector<int> layerTexIndices;
			
				//check if this group should be textured
				//the texturInfo found on the first polygon will be copied and used for the whole Group  
				for(int i = 0 ; i < nmbOfTexLayers ; i++){

					fbx_le_TexPtr = m_pMeshNode->GetLayer(i)->GetTextures();
					
					//check if the layer has a mapping and referenceMode we implemented yet
					if(fbx_le_TexPtr->GetMappingMode() == KFbxLayerElement::eBY_POLYGON){
						if(fbx_le_TexPtr->GetReferenceMode() == KFbxLayerElement::eINDEX_TO_DIRECT){

							int texIndex = fbx_le_TexPtr->GetIndexArray().GetAt(p);
							//check if this polygon has a valid textureIndex - hence is textured on this layer
							if(texIndex != -1){
								//insert the Texture and save the index it can be referenced 
								STexture insertTexture;
								convertKFbxTexture(&insertTexture,fbx_le_TexPtr->GetDirectArray().GetAt(texIndex));
								layerTexIndices.push_back(m_pGeoInfo->Insert(insertTexture));
							}

						}else{
							LOG(TL_ERROR,"ReferenceMode for Texture on Layer %d not yet implemented!",i);
							return false;
						}
					}else{
						LOG(TL_ERROR,"MappingMode for Texture on Layer %d not yet implemented!",i);
						return false;
					}

				}

				//store the actual number of TexLayers of the GeoGroup
				int nmbOfActTexLayers = (int)layerTexIndices.size();
				nmbOfActTexLayersInGeoGroup[geoGroupId] = nmbOfActTexLayers;
				
				//check if some problem occured - the group will be assigned no textureInfo
				if(nmbOfActTexLayers)
                {
					//create a geoGroup with textureInfo

					SGeoGroup *pGroup = m_pGeoInfo->CreateGroup(nmbOfActTexLayers);
					//assign the material index for this group
					pGroup->m_matIndex = materialId;
					//assign the texture Indices
					for(int i = 0 ; i < nmbOfActTexLayers ; i++){
						pGroup->m_pTexIndices[i] = layerTexIndices[i];

						//TODO - need to extract this information from the model
						pGroup->m_eApplyModes.push_back(eMODULATE);
					}
                    bGroupCreated = true;
				
				}
            }
            if(!bGroupCreated)
            {
				//create a geoGroup with no textureInfo
                SGeoGroup *pGroup = m_pGeoInfo->CreateGroup(0);
				//assign the material index for this group
				pGroup->m_matIndex = materialId;
			}
		}
		
       	//********************************************************************************
		//loop over all vertices in this polygon
		//********************************************************************************

		if(bHasVertexColors)
        {
			if(nmbOfActTexLayersInGeoGroup[geoGroupId]) //colors and tex
            {
				for(v = 0 ; v < 3 ; ++v, ++vertexID)
                {
					controlPointIndex = m_pMeshNode->GetPolygonVertex(p,v);

					//coord
					fbxCoord  = &fbx_pControlPoints[controlPointIndex];
					Vcoord.Set((float)fbxCoord->GetAt(0),(float)fbxCoord->GetAt(1),(float)fbxCoord->GetAt(2));
					//normal
					fbxNormal = &fbx_pNormals[controlPointIndex];
					Vnormal.Set((float)fbxNormal->GetAt(0),(float)fbxNormal->GetAt(1),(float)fbxNormal->GetAt(2));
					//vertexColor
					index = fbx_le_allColors->GetIndexArray().GetAt(vertexID);
					fbxColor = &fbx_le_allColors->GetDirectArray().GetAt(index);
					Vcolor.Set((float)fbxColor->mRed,(float)fbxColor->mGreen,(float)fbxColor->mBlue,(float)fbxColor->mAlpha);
					//uvs
					for(int i = 0 ; i < nmbOfActTexLayersInGeoGroup[geoGroupId] ; i++){
						fbx_le_UvPtr = m_pMeshNode->GetLayer(i)->GetUVs();
						index = fbx_le_UvPtr->GetIndexArray().GetAt(vertexID);
						fbxUv = &fbx_le_UvPtr->GetDirectArray().GetAt(index);
						VUvs[i].Set((float)fbxUv->GetAt(0),(float)fbxUv->GetAt(1)); 
					}
					
					m_pGeoInfo->m_pGeoGroups[geoGroupId]->Insert(Vcoord,Vnormal,Vcolor,VUvs);
				}			
			}
            else    //Colors but no tex
            {
				for(v = 0 ; v < 3 ; ++v, ++vertexID)
                {
					controlPointIndex = m_pMeshNode->GetPolygonVertex(p,v);

					//coord
					fbxCoord  = &fbx_pControlPoints[controlPointIndex];
					Vcoord.Set((float)fbxCoord->GetAt(0),(float)fbxCoord->GetAt(1),(float)fbxCoord->GetAt(2));
					//normal
					fbxNormal = &fbx_pNormals[controlPointIndex];
					Vnormal.Set((float)fbxNormal->GetAt(0),(float)fbxNormal->GetAt(1),(float)fbxNormal->GetAt(2));
					//color
					index = fbx_le_allColors->GetIndexArray().GetAt(vertexID);
					fbxColor = &fbx_le_allColors->GetDirectArray().GetAt(index);
					Vcolor.Set((float)fbxColor->mRed,(float)fbxColor->mGreen,(float)fbxColor->mBlue,(float)fbxColor->mAlpha);

					m_pGeoInfo->m_pGeoGroups[geoGroupId]->Insert(Vcoord,Vnormal,Vcolor);
				}
			}
		}
        else //No colors
        {
			if(nmbOfActTexLayersInGeoGroup[geoGroupId]) //no colors but tex
            {
				for(v = 0 ; v < 3 ; ++v, ++vertexID)
                {
					controlPointIndex = m_pMeshNode->GetPolygonVertex(p,v);

					//coord
					fbxCoord  = &fbx_pControlPoints[controlPointIndex];
					Vcoord.Set((float)fbxCoord->GetAt(0),(float)fbxCoord->GetAt(1),(float)fbxCoord->GetAt(2));
					//normal
					fbxNormal = &fbx_pNormals[controlPointIndex];
					Vnormal.Set((float)fbxNormal->GetAt(0),(float)fbxNormal->GetAt(1),(float)fbxNormal->GetAt(2));
					//uvs
					for(int i = 0 ; i < nmbOfActTexLayersInGeoGroup[geoGroupId] ; i++){
						fbx_le_UvPtr = m_pMeshNode->GetLayer(i)->GetUVs();
						index = fbx_le_UvPtr->GetIndexArray().GetAt(vertexID);
						fbxUv = &fbx_le_UvPtr->GetDirectArray().GetAt(index);
						VUvs[i].Set((float)fbxUv->GetAt(0),(float)fbxUv->GetAt(1)); 
					}
					m_pGeoInfo->m_pGeoGroups[geoGroupId]->Insert(Vcoord,Vnormal,VUvs);

				}
			}
            else //no Colors, no tex
            {
				for(v = 0 ; v < 3 ; ++v, ++vertexID)
                {
					controlPointIndex = m_pMeshNode->GetPolygonVertex(p,v);

					//coord
					Vcoord.Set((float) fbx_pControlPoints[controlPointIndex].GetAt(0),
                               (float) fbx_pControlPoints[controlPointIndex].GetAt(1),
                               (float) fbx_pControlPoints[controlPointIndex].GetAt(2));
					//normal
                    Vnormal.Set((float) fbx_pNormals[controlPointIndex].GetAt(0),
                                (float) fbx_pNormals[controlPointIndex].GetAt(1),
                                (float) fbx_pNormals[controlPointIndex].GetAt(2));
					m_pGeoInfo->m_pGeoGroups[geoGroupId]->Insert(Vcoord,Vnormal);
                }
			}
		}
	}

	//cleanup
	SAFE_DELETE_ARRAY(geoGroupLookup);
	SAFE_DELETE_ARRAY(nmbOfActTexLayersInGeoGroup);
	SAFE_DELETE_ARRAY(VUvs);
	
    return true;

}

//*********************************************************************
//Helper
//*********************************************************************

//Private helper for Loading
bool CFBXLoader::LoadScene(KFbxScene* pScene,const string &fileName)
{
	int lMajor,lMinor,lRevision;
	int i,lTakeCount;
	KString lCurrentTakeName;
	bool lStatus;
	char lPassword[1024];

	// Set File Format
	m_pImporter->SetFileFormat(GetFileFormat(fileName));

	// Initialize the importer by providing a filename.
	if(m_pImporter->Initialize(fileName.c_str()) == false){

		LOG(TL_ERROR,"Call to KFbxImporter::Initialize() failed.\n");
		LOG(TL_ERROR,"Error returned: %s\n\n", m_pImporter->GetLastErrorString());

		if(m_pImporter->GetLastErrorID() == KFbxIO::eFILE_VERSION_NOT_SUPPORTED_YET ||
		m_pImporter->GetLastErrorID() == KFbxIO::eFILE_VERSION_NOT_SUPPORTED_ANYMORE){

			KFbxIO::GetCurrentVersion(lMajor,lMinor,lRevision);
			LOG(TL_ERROR,"FBX version number for this version of the FBX SDK is %d.%d.%d\n",lMajor,lMinor,lRevision);
			m_pImporter->GetFileVersion(lMajor, lMinor, lRevision);
			LOG(TL_ERROR,"FBX version number for file %s is %d.%d.%d\n\n",fileName,lMajor,lMinor,lRevision);

		}
		return false;
	}

	KFbxIO::GetCurrentVersion(lMajor,lMinor,lRevision);
	//LOG(TL_DEBUG,"FBX version number for this version of the FBX SDK is %d.%d.%d\n", lMajor, lMinor, lRevision);

	bool isFbx = m_pImporter->GetFileFormat() == KFbxImporter::eFBX_ENCRYPTED ||
	m_pImporter->GetFileFormat() == KFbxImporter::eFBX_ASCII ||
	m_pImporter->GetFileFormat() == KFbxImporter::eFBX_BINARY;

	if (isFbx){
		m_pImporter->GetFileVersion(lMajor, lMinor, lRevision);
		//LOG(TL_DEBUG,"FBX version number for file %s is %d.%d.%d\n\n",fileName.c_str(),lMajor,lMinor,lRevision);

		// From this point, it is possible to access take information without
		// the expense of loading the entire file.
		//LOG(TL_DEBUG,"Take Information\n");

		lTakeCount = m_pImporter->GetTakeCount();
		//LOG(TL_DEBUG,"    Number of takes: %d\n", lTakeCount);
		//LOG(TL_DEBUG,"    Current take: \"%s\"\n", m_pImporter->GetCurrentTakeName());
		//LOG(TL_DEBUG,"\n");

		for(i = 0; i < lTakeCount; i++){
			KFbxTakeInfo* lTakeInfo = m_pImporter->GetTakeInfo(i);
			LOG(TL_DEBUG,"    Take %d\n", i);
			LOG(TL_DEBUG,"         Name: \"%s\"\n", lTakeInfo->mName.Buffer());
			LOG(TL_DEBUG,"         Description: \"%s\"\n", lTakeInfo->mDescription.Buffer());

			// Change the value of the import name if the take should be imported 
			// under a different name.
			LOG(TL_DEBUG,"         Import Name: \"%s\"\n", lTakeInfo->mImportName.Buffer());

			// Set the value of the import state to false if the take should be not
			// be imported. 
			LOG(TL_DEBUG,"         Import State: %s\n", lTakeInfo->mSelect ? "true" : "false");
			LOG(TL_DEBUG,"\n");
		}

		// Set the import states
		m_pImporter->SetState(KFbxImporter::eIMPORT_MATERIAL, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_TEXTURE, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_LINK, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_SHAPE, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_GOBO, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_ANIMATION, true);
		m_pImporter->SetState(KFbxImporter::eIMPORT_GLOBAL_SETTINGS, true);
	}

	// Import the scene.
	lStatus = m_pImporter->Import(*pScene);

	if(lStatus == false && m_pImporter->GetLastErrorID() == KFbxIO::ePASSWORD_ERROR){
		
		LOG(TL_USER,"Please enter password: ");

		lPassword[0] = '\0';
		scanf("%s", lPassword);
		m_pImporter->SetPassword(lPassword);
		lStatus = m_pImporter->Import(*pScene);

		if(lStatus == false && m_pImporter->GetLastErrorID() == KFbxIO::ePASSWORD_ERROR){
			LOG(TL_ERROR,"\nPassword is wrong, import aborted.\n");
		}
	}

	return lStatus;
}

//Private Helper for FileFormat
KFbxImporter::EFileFormat CFBXLoader::GetFileFormat(const string & fileName)
{
	KFbxImporter::EFileFormat lFileFormat = KFbxImporter::eFBX_BINARY;
	KString lFileName(fileName.c_str());
	int lIndex;

	// Find the '.' separating the file extension
	if((lIndex = lFileName.ReverseFind('.')) > -1){
		// extension found
		KString ext = lFileName.Right(lFileName.GetLen() - lIndex - 1);
		if (ext.CompareNoCase("obj") == 0){
			lFileFormat = KFbxImporter::eALIAS_OBJ;
		}
		else 
		if (ext.CompareNoCase("3ds") == 0){
			lFileFormat = KFbxImporter::e3D_STUDIO_3DS;
		}
		else if (ext.CompareNoCase("dxf") == 0){
			lFileFormat = KFbxImporter::eAUTOCAD_DXF;
		}
#ifdef ARUBA
		else
		if (ext.CompareNoCase("apf") == 0){
			lFileFormat = KFbxImporter::eALIAS_APF;
		}
#endif
	}
	return lFileFormat;
}


//Traverse from root using a function pointer to a function that takes a rootNode and a Name
KFbxNode * CFBXLoader::traverse( KFbxNode * (CFBXLoader::*memberPtr) (KFbxNode *, const std::string &) , const std::string & nodeName){

	for(int i = 0; i < m_pRootNode->GetChildCount() ; i++){
		KFbxNode *foundNode = (*this.*memberPtr)(m_pRootNode->GetChild(i),nodeName);
		if(foundNode != NULL)
			return foundNode;
	}
	return NULL;
}


//MeshNode Traverse Function
KFbxNode * CFBXLoader::getMeshNode(KFbxNode *parent, const std::string & nodeName){
	
	KFbxNode *tmpNode = NULL;
		
	if(parent->GetNodeAttribute()){
		KFbxNodeAttribute::EAttributeType lAttributeType = parent->GetNodeAttribute()->GetAttributeType();
		if(lAttributeType == KFbxNodeAttribute::eMESH){
			return tmpNode = parent;
		}
	}
	for(int i = 0; i < parent->GetChildCount(); i++){
		if( (tmpNode = getMeshNode(parent->GetChild(i),nodeName)) != NULL ) {
			return tmpNode;
		}
	}
	return tmpNode;
}


//GroupNode Traverse Function - specify MaterialGroup characteristics 
KFbxNode * CFBXLoader::getMaterialGroupInfo(KFbxNode *parent, const std::string & nodeName){
	
	KFbxNode *tmpNode = NULL;
		
	if(parent->GetNodeAttribute()){
		KFbxNodeAttribute::EAttributeType lAttributeType = parent->GetNodeAttribute()->GetAttributeType();
		if(lAttributeType == KFbxNodeAttribute::eNULL){
			if(nodeName.compare(parent->GetNameWithoutNameSpacePrefix())==0){
				return tmpNode = parent;
			}
		}
	}
	for(int i = 0; i < parent->GetChildCount(); i++){
		if( (tmpNode = getMaterialGroupInfo(parent->GetChild(i),nodeName)) != NULL ){
			return tmpNode;
		}
	}
	return tmpNode;
}


void CFBXLoader::convertKFbxUv(TVector2<double> *uv,KFbxVector2 *fbx_Uv){
	uv->Set(fbx_Uv->GetAt(0),fbx_Uv->GetAt(1));
}


void CFBXLoader::convertKFbxColor(TVector4<double> *color,KFbxColor *fbx_C){
	color->Set(fbx_C->mRed,fbx_C->mGreen,fbx_C->mBlue,fbx_C->mAlpha);
}


void getValue(void * parameter, KFbxUserProperty &prop){
	switch(prop.GetPropertyType()){
		case KFbxUserProperty::eREAL :
			prop.GetDefaultValue(*(double*)parameter);
			break;
		case KFbxUserProperty::eBOOL :
			prop.GetDefaultValue(*(bool*)parameter);
			break;
	}
}

bool extractParameter(KString nodeName,SParameter & p){
	
	p.name = nodeName.Left(9).Buffer();

	char pType = *(nodeName.Buffer()+10);
	
	switch( pType ){
		case 's':
            p.paramType = eParam_String;
			p.value     = new std::string(nodeName.Right(nodeName.GetLen()-12).Buffer());
			return true;
		default:
			p.paramType = eParam_Unknown;
			p.value = NULL;
			return false;
	}
}


void CFBXLoader::convertKFbxMaterial(SMaterial *matInfo,KFbxMaterial *fbx_Mat){
	
	convertKFbxColor(&matInfo->ambientC,&fbx_Mat->GetAmbient());
	convertKFbxColor(&matInfo->diffuseC,&fbx_Mat->GetDiffuse());
	convertKFbxColor(&matInfo->specularC,&fbx_Mat->GetSpecular());
	convertKFbxColor(&matInfo->selfIllumC,&fbx_Mat->GetEmissive());
	
	//try to find a NullNode for the corresponding material
	//the name we look for is the Materialname prefixed with "mat_"
	KString prefix       = "mat_";
	KString materialName = (prefix += fbx_Mat->GetNameWithoutNameSpacePrefix());
	
	//traverse with getMaterialGroupInfo() - returns NULL or the nullNode with the specified name 
	KFbxNode *tmp = traverse(&CFBXLoader::getMaterialGroupInfo, materialName.ToChars() );

	if(tmp){

		//extract aditional material Info 
		KFbxUserProperty *prop;
		if( (prop = tmp->FindPropertyByName("reflectivity")) != NULL )
			getValue(&matInfo->reflectivity,*prop);
		
		if( (prop = tmp->FindPropertyByName("opacity")) != NULL )
			getValue(&matInfo->opacity,*prop);
		
		if( (prop = tmp->FindPropertyByName("exponent")) != NULL ){
            getValue(&matInfo->exp,*prop);
			matInfo->exp = matInfo->exp*128.0;
		}
		
		if( (prop = tmp->FindPropertyByName("useLighting")) != NULL )
			getValue(&matInfo->useLighting,*prop);

		//extract cg shaderInfo
		int childCount = tmp->GetChildCount();

		if(childCount){
				
			KFbxNode *shaderNode;
			KFbxNode *paramNode;
			KString   nodeName;

			for(int i = 0; i < childCount ; i++){
				
				shaderNode = tmp->GetChild(i);
				nodeName   = shaderNode->GetNameWithoutNameSpacePrefix();
				
				if(nodeName.Left(2).Compare(prefix_shader) == 0){
					matInfo->shader.shaderFileName = nodeName.Right(nodeName.GetLen()-2).Append(".cg");
					
					//get the Parameters
					childCount = shaderNode->GetChildCount();
					if(childCount){

						for(int i = 0; i < childCount ; i++){
							paramNode = shaderNode->GetChild(i);
							nodeName   = paramNode->GetNameWithoutNameSpacePrefix();

							SParameter p;
							if(extractParameter(nodeName,p)){
								matInfo->shader.params.push_back(p);
							}
						}
					}
				}
			}
		}

	}else{
		//setDefault Values
		matInfo->reflectivity = 0.0;
		matInfo->opacity      = 1.0;
		matInfo->exp          = 90.0;
		matInfo->useLighting  = false;
	}
		
}


void CFBXLoader::convertKFbxTexture(STexture *texture,KFbxTexture *fbx_Texture){
	//flename
	texture->fileName = fbx_Texture->GetFileName();
	//applyMode
	KFbxTexture::EBlendMode mode = fbx_Texture->GetBlendMode();
	
	switch(mode){
		case KFbxTexture::EBlendMode::eTRANSLUCENT:
			texture->e_applyMode = eREPLACE;
			break;
		case KFbxTexture::EBlendMode::eADDITIVE:
			texture->e_applyMode = eADDITIVE;
			break;
		case KFbxTexture::EBlendMode::eMODULATE: 
			texture->e_applyMode = eMODULATE;
			break;
		default:
			texture->e_applyMode = eREPLACE;
	}
}
