/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* CFBXLoader - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CFBXLOADER_H__
#define __CFBXLOADER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "fbxsdk.h"
#include <Definitions.h>
#include <Memory/mmgr.h>
#include "IModelloader.h"

#pragma comment(lib, "fbxsdk_mt.lib")
#pragma comment(lib, "zlib_mt.lib")

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CFBXLoader : public IModelLoader 
{
	/**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:
	CFBXLoader(void);

	virtual bool Load(const std::string &);
	virtual bool LoadAndTriangulate(const std::string &);
	virtual bool GetGeometryInfo(TSmartPointer<CGeometryInfo> const);
	virtual bool GetAnimationInfo(TSmartPointer<CAnimationInfo> const);
	virtual bool GetSkeletonInfo(TSmartPointer<CSkeletonInfo> const);

	virtual ~CFBXLoader(void);

	/**************************************************************************\
	* PROTECTED CLASS FUNCTIONS                                                *
	\**************************************************************************/
protected:
	
	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	//Private copy constructor and assignment operator to prevent mistakes:
	CFBXLoader(const CFBXLoader &);
	CFBXLoader &operator =(const CFBXLoader &);

	//Scene Mamagement
	bool LoadScene(FBXSDK_NAMESPACE::KFbxScene* pScene,const std::string &fileName);
	FBXSDK_NAMESPACE::KFbxImporter::EFileFormat GetFileFormat(const std::string &fileName);

	//Traversing the Graph
	FBXSDK_NAMESPACE::KFbxNode * traverse(FBXSDK_NAMESPACE::KFbxNode * (CFBXLoader::*)(FBXSDK_NAMESPACE::KFbxNode *, const std::string &) , const std::string &);
	FBXSDK_NAMESPACE::KFbxNode * getMeshNode(FBXSDK_NAMESPACE::KFbxNode *root, const std::string & nodeName);
	FBXSDK_NAMESPACE::KFbxNode * getMaterialGroupInfo(FBXSDK_NAMESPACE::KFbxNode *root, const std::string & nodeName);

	//Build Geometry
	bool buildGeometry();
	void convertKFbxColor(TVector4<double> *color,FBXSDK_NAMESPACE::KFbxColor *fbxC);
	void convertKFbxMaterial(SMaterial *material,FBXSDK_NAMESPACE::KFbxMaterial *fbxMat);
	void convertKFbxTexture(STexture *texture,FBXSDK_NAMESPACE::KFbxTexture *fbxTexture);
	void convertKFbxUv(TVector2<double> *uv,FBXSDK_NAMESPACE::KFbxVector2 *fbxUv);

	/**************************************************************************\
	* PROTECTED CLASS MEMBERS                                                  *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:
	bool m_bInitialized; 
	FBXSDK_NAMESPACE::KFbxSdkManager        *m_pSdkManager;
	FBXSDK_NAMESPACE::KFbxImporter          *m_pImporter;
	FBXSDK_NAMESPACE::KFbxGeometryConverter *m_pGeomConverter;
	FBXSDK_NAMESPACE::KFbxScene             *m_pScene;
	FBXSDK_NAMESPACE::KFbxNode              *m_pRootNode;
	FBXSDK_NAMESPACE::KFbxMesh              *m_pMeshNode;

	bool m_bTriangulate;

};

#endif