/****************************************************************************\
*                                                                            *
* 25.04.2006                                                                 *
* CTerrainInfo - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTerrainInfo.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTerrainInfo::CTerrainInfo() : m_pHeightmap(NULL)
{
}

bool CTerrainInfo::Create(word wWidth, word wHeight, byte *pColorData, byte bytesPerPixel,
                          float fCellX, float fCellZ, float fScaleY, const TSmartPointer<CPhysXMesh> &pMesh)
{
    m_wWidth = wWidth;
    m_wHeight = wHeight;
    m_pMesh = pMesh;
    m_pHeightmap = new byte[wWidth * wHeight];
    dword dwDestOffset = 0;
    dword dwSrcOffset = 0;
    word x, y;
    for(y = 0; y < wHeight; ++y)
    {
        for(x = 0; x < wWidth; ++x)
        {
            m_pHeightmap[dwDestOffset] = pColorData[dwSrcOffset];
            ++dwDestOffset;
            dwSrcOffset += bytesPerPixel;
        }
    }
    m_fCellX = fCellX;
    m_fCellZ = fCellZ;
    m_fScaleY = fScaleY;

    return true;
}

float CTerrainInfo::HeightAt(float fX, float fZ, bool bVerbose) const
{
    if(fX < 0.0f || fZ < 0.0f || fX > GetSizeX() || fZ > GetSizeZ())
    {
        if(bVerbose)
        {
            LOG(TL_DEBUG, "Outside of terrain!");
        }
        return 0.0f;    //Outside of terrain!
    }

    word wCellX = word(fX / m_fCellX);
    word wCellZ = word(fZ / m_fCellZ);
    float fSubX = (fX / m_fCellX) - wCellX;
    float fSubZ = (fZ / m_fCellZ) - wCellZ;
    dword dwCellOffset = wCellX + wCellZ * m_wWidth;
    float fCenterHeight = (m_pHeightmap[dwCellOffset] + m_pHeightmap[dwCellOffset + 1] +
                          m_pHeightmap[dwCellOffset + m_wWidth] + m_pHeightmap[dwCellOffset + m_wWidth + 1]) / 4.0f;
    //Helpers for readability
    float fBaseHeight, fDiffX, fDiffZ;
    if(bVerbose)
        LOG(TL_DEBUG, "Cell: %u, %u; Sub: %.2f, %.2f", wCellX, wCellZ, fSubX, fSubZ);

    //Which triangle of the cell?
    if(fSubX + fSubZ > 1.0f)    //Are we beyond the "slash" diagonal?
    {
        //B or C
        if(fSubX >= fSubZ)
        {
            //B
            fBaseHeight = (float(m_pHeightmap[dwCellOffset + 1]) + float(m_pHeightmap[dwCellOffset + m_wWidth + 1])) / 2.0f;
            fSubX = (1.0f - fSubX) * 2.0f;
            fDiffX = (float(fCenterHeight - fBaseHeight));
            if(fSubZ <= 0.5f)
            {
                //B1
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: B1");
                fSubZ = 1.0f - fSubZ * 2.0f;
                fDiffZ = (float(m_pHeightmap[dwCellOffset + 1]) - fBaseHeight);
            }
            else
            {
                //B2
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: B2");
                fSubZ = (fSubZ - 0.5f) * 2.0f;
                fDiffZ = (float(m_pHeightmap[dwCellOffset + m_wWidth + 1]) - fBaseHeight);
            }
        }
        else
        {
            //C
            fBaseHeight = (float(m_pHeightmap[dwCellOffset + m_wWidth]) + float(m_pHeightmap[dwCellOffset + m_wWidth + 1])) / 2.0f;
            fSubZ = (1.0f - fSubZ) * 2.0f;
            fDiffZ = (float(fCenterHeight - fBaseHeight));
            if(fSubX <= 0.5f)
            {
                //C1
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: C1");
                fSubX = 1.0f - fSubX * 2.0f;
                fDiffX = (float(m_pHeightmap[dwCellOffset + m_wWidth]) - fBaseHeight);
            }
            else
            {
                //C2
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: C2");
                fSubX = (fSubX - 0.5f) * 2.0f;
                fDiffX = (float(m_pHeightmap[dwCellOffset + m_wWidth + 1]) - fBaseHeight);
            }
        }
    }
    else
    {
        //A or D
        if(fSubX >= fSubZ)
        {
            //A
            fBaseHeight = (float(m_pHeightmap[dwCellOffset]) + float(m_pHeightmap[dwCellOffset + 1])) / 2.0f;
            fSubZ *= 2.0f;
            fDiffZ = (float(fCenterHeight - fBaseHeight));
            if(fSubX <= 0.5f)
            {
                //A1
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: A1");
                fSubX = 1.0f - fSubX * 2.0f;
                fDiffX = (float(m_pHeightmap[dwCellOffset]) - fBaseHeight);
            }
            else
            {
                //A2
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: A2");
                fSubX = (fSubX - 0.5f) * 2.0f;
                fDiffX = (float(m_pHeightmap[dwCellOffset + 1]) - fBaseHeight);
            }
        }
        else
        {
            //D
            fBaseHeight = (float(m_pHeightmap[dwCellOffset]) + float(m_pHeightmap[dwCellOffset + m_wWidth])) / 2.0f;
            fSubX *= 2.0f;
            fDiffX = (float(fCenterHeight - fBaseHeight));
            if(fSubZ <= 0.5f)
            {
                //D1
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: D1");
                fSubZ = 1.0f - fSubZ * 2.0f;
                fDiffZ = (float(m_pHeightmap[dwCellOffset]) - fBaseHeight);
            }
            else
            {
                //D2
                if(bVerbose)
                    LOG(TL_DEBUG, "Quadrant: D2");
                fSubZ = (fSubZ - 0.5f) * 2.0f;
                fDiffZ = (float(m_pHeightmap[dwCellOffset + m_wWidth]) - fBaseHeight);
            }
        }
    }
    if(bVerbose)
        LOG(TL_DEBUG, "Base: %.2f; X: %.2f*%.2f; Z: %.2f*%.2f; Height = %.2f",  m_fScaleY * fBaseHeight,  m_fScaleY * fDiffX, fSubX,  m_fScaleY * fDiffZ, fSubZ,
                    m_fScaleY * (fBaseHeight + fDiffX * fSubX + fDiffZ * fSubZ));
    return m_fScaleY * (fBaseHeight + fDiffX * fSubX + fDiffZ * fSubZ);
}

float CTerrainInfo::GetSizeX() const
{
	return m_fCellX * (m_wWidth - 1);
}

float CTerrainInfo::GetSizeZ() const
{
	return m_fCellZ * (m_wHeight - 1);
}

CTerrainInfo::~CTerrainInfo()
{
    SAFE_DELETE_ARRAY(m_pHeightmap);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


