/****************************************************************************\
*                                                                            *
* 25.04.2006                                                                 *
* CTerrainInfo - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTERRAININFO_H__
#define __CTERRAININFO_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Collision/CPhysXMesh.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTerrainInfo
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTerrainInfo();

        bool Create(word wWidth, word wHeight, byte *pColorData, byte bytesPerPixel,
                    float fCellX, float fCellZ, float fScaleY, const TSmartPointer<CPhysXMesh> &pMesh); //Copies only the FIRST color value of each pixel

        float HeightAt(float fX, float fZ, bool bVerbose) const;

		float GetSizeX() const;
		float GetSizeZ() const;

        inline const TSmartPointer<CPhysXMesh> &GetMesh() {return m_pMesh;}
        ~CTerrainInfo();

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        CTerrainInfo(const CTerrainInfo &);
        const CTerrainInfo &operator =(const CTerrainInfo &);
        
    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    private:
        word m_wWidth, m_wHeight;
        byte *m_pHeightmap;
        float m_fCellX, m_fCellZ;
        float m_fScaleY;
        TSmartPointer<CPhysXMesh> m_pMesh;
};

#endif
