/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* IModelLoader - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "IModelLoader.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/


SParameter::SParameter(const SParameter &other){
	name = other.name;
	paramType = other.paramType;

	switch(paramType)
	{
		case eParam_String:
			value = new std::string(*((std::string *)other.value));
			break;
		default:
			LOG(TL_ERROR, "Warning: unknown parameter type in SParameter::SParameter");
			paramType = eParam_Unknown;
			value = NULL;
	}	
}

SParameter &SParameter::operator = (const SParameter &other){
	if(value)
	{
		SAFE_DELETE(value);
	}
	name = other.name;
	paramType = other.paramType;

	switch(paramType)
	{
		case eParam_String:
			value = new std::string(*((std::string *)other.value));
			break;
		default:
			LOG(TL_ERROR, "Warning: unknown parameter type in SParameter::SParameter");
			paramType = eParam_Unknown;
			value = NULL;
	}	

	return *this;
}

SGeoGroup::SGeoGroup(){
	m_matIndex = -1;
	m_bNumTexLayers = 0; 
	m_pTexIndices = NULL; 
	m_pUvs = NULL;
}

SGeoGroup::SGeoGroup(byte bNumTexLayers){
	m_matIndex = -1;
	m_bNumTexLayers  = bNumTexLayers; 
    if(bNumTexLayers)
    {
	    m_pTexIndices = new int[bNumTexLayers]; 
	    m_pUvs = new std::vector<CVector2f>[bNumTexLayers];
    }
    else
    {
        m_pTexIndices = NULL;
        m_pUvs = NULL;
    }
}

SGeoGroup::SGeoGroup(const SGeoGroup &other)
{
	m_bNumTexLayers = other.m_bNumTexLayers;
	m_matIndex = other.m_matIndex;
    m_Vertices = other.m_Vertices;
    m_Normals = other.m_Normals;
    m_Colors = other.m_Colors;
    m_Indices = other.m_Indices;

    if(m_bNumTexLayers)
    {
        m_pTexIndices = new int[m_bNumTexLayers];
        m_pUvs = new std::vector<CVector2f>[m_bNumTexLayers];

        for(byte b = 0; b < m_bNumTexLayers; ++b)
        {
            m_pTexIndices[b] = other.m_pTexIndices[b];
            m_pUvs[b] = other.m_pUvs[b];
        }
    }
    else
    {
        m_pTexIndices = NULL;
        m_pUvs = NULL;
    }
}

SGeoGroup &SGeoGroup::operator =(const SGeoGroup &other)
{
    SAFE_DELETE_ARRAY(m_pTexIndices);
    SAFE_DELETE_ARRAY(m_pUvs);

	m_bNumTexLayers = other.m_bNumTexLayers;
	m_matIndex = other.m_matIndex;
    m_Vertices = other.m_Vertices;
    m_Normals = other.m_Normals;
    m_Colors = other.m_Colors;
    m_Indices = other.m_Indices;

    if(m_bNumTexLayers)
    {
        m_pTexIndices = new int[m_bNumTexLayers];
        m_pUvs = new std::vector<CVector2f>[m_bNumTexLayers];

        for(byte b = 0; b < m_bNumTexLayers; ++b)
        {
            m_pTexIndices[b] = other.m_pTexIndices[b];
            m_pUvs[b] = other.m_pUvs[b];
        }
    }
    else
    {
        m_pTexIndices = NULL;
        m_pUvs = NULL;
    }

    return *this;
}

//schaut, ob genau diese Daten schon existieren. Wenn ja --> nur Index adden; wenn nein --> Daten adden und neuen index adden
void SGeoGroup::Insert(const CVector3f &vertex, const CVector3f &normal, const CVector4f &color, const CVector2f *pUvs)
{
	assert( pUvs != NULL && "SGeoGroup::Insert() called with UV Null-Pointer !");

	dword index = 0;
	byte tex;
	bool bSame;

	for(std::vector<CVector3f>::iterator it = m_Vertices.begin() ; it != m_Vertices.end(); ++it, ++index)
	{
		if((*it) == vertex)
		{
			bSame = true;
			//Schau, ob an dieser Position in den anderen Arrays die passenden Daten stehen
			if( (m_Normals[index] != normal) || (m_Colors[index] != color) )
			{
				bSame = false;
			}
			else
			{
				//Alle Textur-UVs berprfen
				for(tex = 0; tex < m_bNumTexLayers; ++tex)
				{
					if(m_pUvs[tex][index] != pUvs[tex])
					{
						bSame = false;
						break;
					}
				}
			}
			if(bSame)
			{
				//Wenn ja: Treffer! Einfach diese Position an die Indices anfgen und returnen
				m_Indices.push_back(index);
				return;
			}
		}
	}
	//Nicht gefunden: Daten anfgen und neuen Index anfgen
	m_Vertices.push_back(vertex);
	m_Normals.push_back(normal);
	m_Colors.push_back(color);
	for(byte i = 0; i < m_bNumTexLayers; ++i)
	{
		m_pUvs[i].push_back(pUvs[i]);
	}

	m_Indices.push_back((int)(m_Vertices.size() - 1));

}

//schaut, ob genau diese Daten schon existieren. Wenn ja --> nur Index adden; wenn nein --> Daten adden und neuen index adden
void SGeoGroup::Insert(const CVector3f &vertex, const CVector3f &normal, const CVector2f *pUvs)
{ 
	assert( pUvs != NULL && "SGeoGroup::Insert() called with UV Null-Pointer !");

	dword index = 0;
	byte tex;
	bool bSame;

	for(std::vector<CVector3f>::iterator it = m_Vertices.begin() ; it != m_Vertices.end(); ++it, ++index)
	{
		if((*it) == vertex)
		{
			bSame = true;
			//Schau, ob an dieser Position in den anderen Arrays die passenden Daten stehen
			if( (m_Normals[index] != normal) )
			{
				bSame = false;
			}
			else
			{
				//Alle Textur-UVs berprfen
				for(tex = 0; tex < m_bNumTexLayers; ++tex)
				{
					if(m_pUvs[tex][index] != pUvs[tex])
					{
						bSame = false;
						break;
					}
				}
			}
			if(bSame)
			{
				//Wenn ja: Treffer! Einfach diese Position an die Indices anfgen und returnen
				m_Indices.push_back(index);
				return;
			}
		}
	}
	//Nicht gefunden: Daten anfgen und neuen Index anfgen
	m_Vertices.push_back(vertex);
	m_Normals.push_back(normal);
	for(byte i = 0; i < m_bNumTexLayers; ++i)
	{
		m_pUvs[i].push_back(pUvs[i]);
	}

	m_Indices.push_back((int)(m_Vertices.size() - 1));
}


//schaut, ob genau diese Daten schon existieren. Wenn ja --> nur Index adden; wenn nein --> Daten adden und neuen index adden
void SGeoGroup::Insert(const CVector3f &vertex, const CVector3f &normal, const CVector4f &color)
{   
	dword index = 0;
	
	for(std::vector<CVector3f>::iterator it = m_Vertices.begin() ; it != m_Vertices.end(); ++it, ++index)
	{
		if((*it) == vertex)
		{
			//Schau, ob an dieser Position in den anderen Arrays die passenden Daten stehen
			if(m_Normals[index] == normal && m_Colors[index] == color)
			{
				//Wenn ja: Treffer! Einfach diese Position an die Indices anfgen und returnen
				m_Indices.push_back(index);
				return;
			}
		}
	}
	//Nicht gefunden: Daten anfgen und neuen Index anfgen
	m_Vertices.push_back(vertex);
	m_Normals.push_back(normal);
	m_Colors.push_back(color);
	
	m_Indices.push_back((int)(m_Vertices.size() - 1));
}

//schaut, ob genau diese Daten schon existieren. Wenn ja --> nur Index adden; wenn nein --> Daten adden und neuen index adden
void SGeoGroup::Insert(const CVector3f &vertex, const CVector3f &normal)
{   
	dword index = 0;
	for(std::vector<CVector3f>::iterator it = m_Vertices.begin() ; it != m_Vertices.end(); ++it, ++index)
	{
		if((*it) == vertex)
		{
			//Schau, ob an dieser Position in den anderen Arrays die passenden Daten stehen
			if(m_Normals[index] == normal)
			{
				//Wenn ja: Treffer! Einfach diese Position an die Indices anfgen und returnen
				m_Indices.push_back(index);
				return;
			}
		}
	}
	
	//Nicht gefunden: Daten anfgen und neuen Index anfgen
	m_Vertices.push_back(vertex);
	m_Normals.push_back(normal);

	m_Indices.push_back((int)(m_Vertices.size() - 1));
}

CGeometryInfo::CGeometryInfo()
{
}

CGeometryInfo::~CGeometryInfo()
{
    for(std::vector<SGeoGroup *>::iterator it = m_pGeoGroups.begin(); it != m_pGeoGroups.end(); ++it)
    {
        SAFE_DELETE((*it));
    }
}

SGeoGroup *CGeometryInfo::CreateGroup(byte bNumTexLayers)
{
    SGeoGroup *pGroup = new SGeoGroup(bNumTexLayers);
    m_pGeoGroups.push_back(pGroup);

    return pGroup;
}

int CGeometryInfo::Insert(const SMaterial &mat){
	dword index = 0;
	for(std::vector<SMaterial>::iterator it = m_Materials.begin() ; it != m_Materials.end(); ++it, ++index)
    {
		if((*it).ambientC == mat.ambientC && (*it).diffuseC == mat.diffuseC && (*it).specularC == mat.specularC &&
		   (*it).exp == mat.exp  && (*it).opacity == mat.opacity  && (*it).reflectivity == mat.reflectivity)
		    return index;
	}
	m_Materials.push_back(mat);
	return (int)(m_Materials.size() - 1);
}
	
int CGeometryInfo::Insert(const STexture &tex){
	dword index = 0;
	for(std::vector<STexture>::iterator it = m_Textures.begin() ; it != m_Textures.end(); ++it, ++index)
    {
		if((*it).e_applyMode == tex.e_applyMode && (*it).fileName == tex.fileName)
            return index;
	}
	m_Textures.push_back(tex);
	return (int)(m_Textures.size() - 1);
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/