/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* ModelStructs - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __MODELSTRUCTS_H__
#define __MODELSTRUCTS_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Math/TVector2.h>
#include <Math/TVector3.h>
#include <Math/TVector4.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

enum EApplyMode{
    eMODULATE,
    eREPLACE,
    eADDITIVE
};

enum EParamType{
	eParam_Double,
	eParam_Integer,
	eParam_String,
	eParam_Bool,
	eParam_Unknown
};

struct SParameter{

	SParameter() : value(NULL), paramType(eParam_Unknown) {}

	std::string name;
	EParamType  paramType;
	void *      value;

	SParameter(const SParameter &other);
	SParameter &operator = (const SParameter &other);

	~SParameter(){SAFE_DELETE(value);}
};

struct SShader{

	std::string shaderFileName;
	std::vector<SParameter> params;

	~SShader(){
		params.clear();	
	}

};

struct SMaterial{
    TVector4<double> ambientC;
    TVector4<double> diffuseC;
    TVector4<double> selfIllumC;
    TVector4<double> specularC;
    double           exp;
    double           opacity;
    double           reflectivity;
	bool             useLighting;

	SShader          shader;
};

struct STexture{
    std::string fileName;
    EApplyMode  e_applyMode;
};

struct SGeoGroup{

    byte m_bNumTexLayers;
    int *m_pTexIndices;
    int  m_matIndex;

	std::vector<CVector3f> m_Vertices;
    std::vector<CVector3f> m_Normals;
    std::vector<CVector4f> m_Colors;

	//TexLayer
    std::vector<CVector2f> *m_pUvs;
    std::vector<dword>      m_Indices;
	std::vector<EApplyMode> m_eApplyModes;

    SGeoGroup(); 
    SGeoGroup(byte bNumTexLayers);
    SGeoGroup(const SGeoGroup &other);
    SGeoGroup &operator =(const SGeoGroup &other);
    ~SGeoGroup() {SAFE_DELETE_ARRAY(m_pTexIndices); SAFE_DELETE_ARRAY(m_pUvs);}

    void Insert(const CVector3f &vertex, const CVector3f &normal, const CVector4f &color, const CVector2f *pUvs);
    void Insert(const CVector3f &vertex, const CVector3f &normal, const CVector2f *pUvs);
    void Insert(const CVector3f &vertex, const CVector3f &normal, const CVector4f &color);
    void Insert(const CVector3f &vertex, const CVector3f &normal);

};

//geometry
class CGeometryInfo : public IManagedObject{

public: 
    CGeometryInfo();
    ~CGeometryInfo();

    SGeoGroup *CreateGroup(byte bNumTexLayers);
    int Insert(const SMaterial &mat);
    int Insert(const STexture  &tex);
	
    std::vector<SGeoGroup *> m_pGeoGroups;
    std::vector<SMaterial>   m_Materials;
    std::vector<STexture>    m_Textures;	
};

class CAnimationInfo : public IManagedObject{

};

class CSkeletonInfo : public IManagedObject{

};

#endif
