/****************************************************************************\
*                                                                            *
* 18.07.2003                                                                 *
* CMatrix33 - Source File                                                    *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CMatrix33.h"
#include <math.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CMatrix33::CMatrix33(const float &f)
{
    _11 = _12 = _13 = 
    _21 = _22 = _23 = 
    _31 = _31 = _33 = f;
}

CMatrix33::CMatrix33(const float &f11, const float &f12, const float &f13,
                     const float &f21, const float &f22, const float &f23,
                     const float &f31, const float &f32, const float &f33)
{
    _11 = f11; _12 = f12; _13 = f13;
    _21 = f21; _22 = f22; _23 = f23;
    _31 = f31; _32 = f32; _33 = f33;
}

CMatrix33::CMatrix33(const CMatrix33 &other)
{
    _11 = other._11; _12 = other._12; _13 = other._13;
    _21 = other._21; _22 = other._22; _23 = other._23;
    _31 = other._31; _32 = other._32; _33 = other._33;
}

const CMatrix33 &CMatrix33::operator=(const CMatrix33 &other)
{
    _11 = other._11; _12 = other._12; _13 = other._13;
    _21 = other._21; _22 = other._22; _23 = other._23;
    _31 = other._31; _32 = other._32; _33 = other._33;

    return *this;
}

// Slower than target.Multiply(this, other)
const CMatrix33 CMatrix33::operator*(const CMatrix33 &other) const
{
    return CMatrix33(_11 * other._11 + _12 * other._21 + _13 * other._31,
                     _11 * other._12 + _12 * other._22 + _13 * other._32,
                     _11 * other._13 + _12 * other._23 + _13 * other._33,

                     _21 * other._11 + _22 * other._21 + _23 * other._31,
                     _21 * other._12 + _22 * other._22 + _23 * other._32,
                     _21 * other._13 + _22 * other._23 + _23 * other._33,

                     _31 * other._11 + _32 * other._21 + _33 * other._31,
                     _31 * other._12 + _32 * other._22 + _33 * other._32,
                     _31 * other._13 + _32 * other._23 + _33 * other._33);
}

const CMatrix33 &CMatrix33::operator*=(const CMatrix33 &other)
{
  Set(_11 * other._11 + _12 * other._21 + _13 * other._31,
      _11 * other._12 + _12 * other._22 + _13 * other._32,
      _11 * other._13 + _12 * other._23 + _13 * other._33,

      _21 * other._11 + _22 * other._21 + _23 * other._31,
      _21 * other._12 + _22 * other._22 + _23 * other._32,
      _21 * other._13 + _22 * other._23 + _23 * other._33,

      _31 * other._11 + _32 * other._21 + _33 * other._31,
      _31 * other._12 + _32 * other._22 + _33 * other._32,
      _31 * other._13 + _32 * other._23 + _33 * other._33);

  return *this;
}

const CVector3f CMatrix33::operator*(const CVector3f &v) const
{
    return CVector3f(_11 * v.m_X + _12 * v.m_Y + _13 * v.m_Z,
                     _21 * v.m_X + _22 * v.m_Y + _23 * v.m_Z,
                     _31 * v.m_X + _32 * v.m_Y + _33 * v.m_Z);
}

bool CMatrix33::operator==(const CMatrix33 &other) const
{
    return _11 == other._11 && _12 == other._12 && _13 == other._13
        && _21 == other._21 && _22 == other._22 && _23 == other._23
        && _31 == other._31 && _32 == other._32 && _33 == other._33;
}

bool CMatrix33::operator!=(const CMatrix33 &other) const
{
    return _11 != other._11 || _12 != other._12 || _13 != other._13
        || _21 != other._21 || _22 != other._22 || _23 != other._23
        || _31 != other._31 || _32 != other._32 || _33 != other._33;
}

void CMatrix33::Set(const float &f)
{
    _11 = _12 = _13 =
    _21 = _22 = _23 =
    _31 = _31 = _33 = f;
}

void CMatrix33::Set(const float &f11, const float &f12, const float &f13,
                    const float &f21, const float &f22, const float &f23,
                    const float &f31, const float &f32, const float &f33)
{
    _11 = f11; _12 = f12; _13 = f13;
    _21 = f21; _22 = f22; _23 = f23;
    _31 = f31; _32 = f32; _33 = f33;
}

void CMatrix33::Identity()
{
    _11 = 1.0f; _12 = 0.0f; _13 = 0.0f;
    _21 = 0.0f; _22 = 1.0f; _23 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = 1.0f;
}

void CMatrix33::Transpose()
{
  float f;

  f = _12;
  _12 = _21;
  _21 = f;

  f = _13;
  _13 = _31;
  _31 = f;

  f = _23;
  _23 = _32;
  _32 = f;
}

void CMatrix33::TransposeFrom(const CMatrix33 &m)
{
    _11 = m._11; _12 = m._21; _13 = m._31;
    _21 = m._12; _22 = m._22; _23 = m._32;
    _31 = m._13; _32 = m._23; _33 = m._33;
}


void CMatrix33::Multiply(const CMatrix33 &first, const CMatrix33 &second)
{
    _11 = first._11 * second._11 + first._12 * second._21 + first._13 * second._31;
    _12 = first._11 * second._12 + first._12 * second._22 + first._13 * second._32;
    _13 = first._11 * second._13 + first._12 * second._23 + first._13 * second._33;

    _21 = first._21 * second._11 + first._22 * second._21 + first._23 * second._31;
    _22 = first._21 * second._12 + first._22 * second._22 + first._23 * second._32;
    _23 = first._21 * second._13 + first._22 * second._23 + first._23 * second._33;

    _31 = first._31 * second._11 + first._32 * second._21 + first._33 * second._31;
    _32 = first._31 * second._12 + first._32 * second._22 + first._33 * second._32;
    _33 = first._31 * second._13 + first._32 * second._23 + first._33 * second._33;
}

void CMatrix33::Scaling(const float &fX, const float &fY, const float &fZ)
{
    _11 = fX;   _12 = 0.0f; _13 = 0.0f;
    _21 = 0.0f; _22 = fY;   _23 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = fZ;
}

void CMatrix33::Scaling(const CVector3f &vScale)
{
    _11 = vScale.m_X;   _12 = 0.0f; _13 = 0.0f;
    _21 = 0.0f; _22 = vScale.m_Y;   _23 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = vScale.m_Z;
}

void CMatrix33::RotationX(const float &fRad)
{
    _11 = 1.0f; _12 = 0.0f;      _13 = 0.0f;
    _21 = 0.0f; _22 = cos(fRad); _23 = sin(fRad);
    _31 = 0.0f; _32 = -_23;      _33 = _22;
}

void CMatrix33::RotationY(const float &fRad)
{
    _11 = cos(fRad); _12 = 0.0f; _13 = -sin(fRad);
    _21 = 0.0f;      _22 = 1.0f; _23 = 0.0f;
    _31 = -_13;      _32 = 0.0f; _33 = _11;
}

void CMatrix33::RotationZ(const float &fRad)
{
    _11 = cos(fRad); _12 = sin(fRad); _13 = 0.0f;
    _21 = -_12;      _22 = _11;       _23 = 0.0f;
    _31 = 0.0f;      _32 = 0.0f;      _33 = 1.0f;
}

void CMatrix33::RotationZXY(const float &fX, const float &fY, const float &fZ)
{
    float sinx = sin(fX);
    float siny = sin(fY);
    float sinz = sin(fZ);
    float cosx = cos(fX);
    float cosy = cos(fY);
    float cosz = cos(fZ);

	_11 = cosy*cosz - sinx*siny*sinz; _12 = cosy*sinz + sinx*siny*cosz; _13 = -cosx*siny;
	_21 =-cosx*sinz; _22 = cosx*cosz; _23 = sinx;
	_31 = siny*cosz + sinx*cosy*sinz; _32 = siny*sinz - sinx*cosy*cosz;_33 = cosx*cosy;
}

void CMatrix33::RotationZXY(const CVector3f &vRotation)
{
    float sinx = sin(vRotation.m_X);
    float siny = sin(vRotation.m_Y);
    float sinz = sin(vRotation.m_Z);
    float cosx = cos(vRotation.m_X);
    float cosy = cos(vRotation.m_Y);
    float cosz = cos(vRotation.m_Z);

	_11 = cosy*cosz - sinx*siny*sinz; _12 = cosy*sinz + sinx*siny*cosz; _13 = -cosx*siny;
	_21 =-cosx*sinz; _22 = cosx*cosz; _23 = sinx;
	_31 = siny*cosz + sinx*cosy*sinz; _32 = siny*sinz - sinx*cosy*cosz;_33 = cosx*cosy;
}

const CVector3f CMatrix33::GetZXYRotationVector()
{
	CVector3f rot;
	rot.m_Z = atan2(-_21, _22);
/*
	CMatrix33 invZ;
	invZ.RotationZ(-rot.m_Z);
	CMatrix33 noZ((*this) * invZ);
*/
	rot.m_X = asin(_23);
	rot.m_Y = atan2(-_13, _33);

	return rot;
}

void CMatrix33::Log(word wTargetLog) const
{
  LOG(wTargetLog, "(%.2f, %.2f, %.2f)\n"
                  "(%.2f, %.2f, %.2f)\n"
                  "(%.2f, %.2f, %.2f)\n",    _11, _12, _13,
                                             _21, _22, _23,
                                             _31, _32, _33);
}