/****************************************************************************\
*                                                                            *
* 19.02.2006                                                                 *
* CMatrix33 - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
* If you are wondering about all the consts and references, read             *
* "Effective C++" and "More Effective C++" by Scott Meyers                   *
\****************************************************************************/

#ifndef __CMATRIX33_H__
#define __CMATRIX33_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include "TVector3.h"

/****************************************************************************\
* DEFINITIONS	                                                               *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
#pragma pack(push, 1)    //Ensures that the floats are tightly packed
class CMatrix33
{
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    CMatrix33() {};
    CMatrix33(const float &f);
    CMatrix33(const float &f11, const float &f12, const float &f13,
              const float &f21, const float &f22, const float &f23,
              const float &f31, const float &f32, const float &f33);
    CMatrix33(const CMatrix33 &other);

    const CMatrix33 &operator=(const CMatrix33 &other);
    const CMatrix33 operator*(const CMatrix33 &other) const; //target.Multiply(this, other) is most likely faster
    const CMatrix33 &operator*=(const CMatrix33 &other); //Slower than target.Multiply(this, other), so use the faster one, if possible!
                                                         //Edit: Still true? Benchmark necessary

	const CVector3f CMatrix33::operator*(const CVector3f &v) const;

	bool operator==(const CMatrix33 &other) const;
    bool operator!=(const CMatrix33 &other) const;


    void Set(const float &f);
    void Set(const float &f11, const float &f12, const float &f13,
             const float &f21, const float &f22, const float &f23,
             const float &f31, const float &f32, const float &f33);

    void Identity();
    bool Inverse();
    bool InverseFrom(const CMatrix33 &m);
    void Transpose();
    void TransposeFrom(const CMatrix33 &m);

    //Don't use "first.Multiply(first, second)" or "second.Multiply(first, second)" This gives wrong results!
    //Use "first *= second" instead
    void Multiply(const CMatrix33 &first, const CMatrix33 &second); //Matrix contains "first" x "second" --> "second" is applied before "first"

	void Scaling(const float &fX, const float &fY, const float &fZ);
	void Scaling(const CVector3f &vScale);
    void RotationX(const float &fRad);
    void RotationY(const float &fRad);
    void RotationZ(const float &fRad);
	void RotationZXY(const float &fX, const float &fY, const float &fZ);
	void RotationZXY(const CVector3f &vRotation);
	const CVector3f GetZXYRotationVector();

    void Log(word wTargetLog) const;

    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
    
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:

    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    public:
        //Column major fr OpenGL (grrrr), dies betrifft aber _nur_ die Ausrichtung im SPEICHER
        //Logisch liegen die Elemente trotzdem row major (wie in der Mathematik blich)
        struct
        {
            float _11, _21, _31;
            float _12, _22, _32;
            float _13, _23, _33;
        };
};
#pragma pack(pop)

#endif