/****************************************************************************\
*                                                                            *
* 18.07.2003                                                                 *
* CMatrix44 - Source File                                                    *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CMatrix44.h"
#include <math.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CMatrix44::CMatrix44(const float &f)
{
    _11 = _12 = _13 = _14 =
    _21 = _22 = _23 = _24 =
    _31 = _31 = _33 = _34 =
    _41 = _42 = _43 = _44 = f;
}

CMatrix44::CMatrix44(const float &f11, const float &f12, const float &f13, const float &f14,
                     const float &f21, const float &f22, const float &f23, const float &f24,
                     const float &f31, const float &f32, const float &f33, const float &f34,
                     const float &f41, const float &f42, const float &f43, const float &f44)
{
    _11 = f11; _12 = f12; _13 = f13; _14 = f14;
    _21 = f21; _22 = f22; _23 = f23; _24 = f24;
    _31 = f31; _32 = f32; _33 = f33; _34 = f34;
    _41 = f41; _42 = f42; _43 = f43; _44 = f44;
}

CMatrix44::CMatrix44(const CMatrix44 &other)
{
    _11 = other._11; _12 = other._12; _13 = other._13; _14 = other._14;
    _21 = other._21; _22 = other._22; _23 = other._23; _24 = other._24;
    _31 = other._31; _32 = other._32; _33 = other._33; _34 = other._34;
    _41 = other._41; _42 = other._42; _43 = other._43; _44 = other._44;
}

const CMatrix44 &CMatrix44::operator=(const CMatrix44 &other)
{
    _11 = other._11; _12 = other._12; _13 = other._13; _14 = other._14;
    _21 = other._21; _22 = other._22; _23 = other._23; _24 = other._24;
    _31 = other._31; _32 = other._32; _33 = other._33; _34 = other._34;
    _41 = other._41; _42 = other._42; _43 = other._43; _44 = other._44;

    return *this;
}

// Slower than target.Multiply(this, other)
const CMatrix44 CMatrix44::operator*(const CMatrix44 &other) const
{
    return CMatrix44(_11 * other._11 + _12 * other._21 + _13 * other._31 + _14 * other._41,
                     _11 * other._12 + _12 * other._22 + _13 * other._32 + _14 * other._42,
                     _11 * other._13 + _12 * other._23 + _13 * other._33 + _14 * other._43,
                     _11 * other._14 + _12 * other._24 + _13 * other._34 + _14 * other._44,

                     _21 * other._11 + _22 * other._21 + _23 * other._31 + _24 * other._41,
                     _21 * other._12 + _22 * other._22 + _23 * other._32 + _24 * other._42,
                     _21 * other._13 + _22 * other._23 + _23 * other._33 + _24 * other._43,
                     _21 * other._14 + _22 * other._24 + _23 * other._34 + _24 * other._44,

                     _31 * other._11 + _32 * other._21 + _33 * other._31 + _34 * other._41,
                     _31 * other._12 + _32 * other._22 + _33 * other._32 + _34 * other._42,
                     _31 * other._13 + _32 * other._23 + _33 * other._33 + _34 * other._43,
                     _31 * other._14 + _32 * other._24 + _33 * other._34 + _34 * other._44,

                     _41 * other._11 + _42 * other._21 + _43 * other._31 + _44 * other._41,
                     _41 * other._12 + _42 * other._22 + _43 * other._32 + _44 * other._42,
                     _41 * other._13 + _42 * other._23 + _43 * other._33 + _44 * other._43,
                     _41 * other._14 + _42 * other._24 + _43 * other._34 + _44 * other._44);
}

const CMatrix44 &CMatrix44::operator*=(const CMatrix44 &other)
{
  Set(_11 * other._11 + _12 * other._21 + _13 * other._31 + _14 * other._41,
      _11 * other._12 + _12 * other._22 + _13 * other._32 + _14 * other._42,
      _11 * other._13 + _12 * other._23 + _13 * other._33 + _14 * other._43,
      _11 * other._14 + _12 * other._24 + _13 * other._34 + _14 * other._44,

      _21 * other._11 + _22 * other._21 + _23 * other._31 + _24 * other._41,
      _21 * other._12 + _22 * other._22 + _23 * other._32 + _24 * other._42,
      _21 * other._13 + _22 * other._23 + _23 * other._33 + _24 * other._43,
      _21 * other._14 + _22 * other._24 + _23 * other._34 + _24 * other._44,

      _31 * other._11 + _32 * other._21 + _33 * other._31 + _34 * other._41,
      _31 * other._12 + _32 * other._22 + _33 * other._32 + _34 * other._42,
      _31 * other._13 + _32 * other._23 + _33 * other._33 + _34 * other._43,
      _31 * other._14 + _32 * other._24 + _33 * other._34 + _34 * other._44,

      _41 * other._11 + _42 * other._21 + _43 * other._31 + _44 * other._41,
      _41 * other._12 + _42 * other._22 + _43 * other._32 + _44 * other._42,
      _41 * other._13 + _42 * other._23 + _43 * other._33 + _44 * other._43,
      _41 * other._14 + _42 * other._24 + _43 * other._34 + _44 * other._44);

  return *this;
}

const CVector3f CMatrix44::operator*(const CVector3f &v) const
{
    //WARNING: I'm IGNORING THE W-COMPONENT THAT WOULD EXIST!!
    return CVector3f(_11 * v.m_X + _12 * v.m_Y + _13 * v.m_Z + _14,
                     _21 * v.m_X + _22 * v.m_Y + _23 * v.m_Z + _24,
                     _31 * v.m_X + _32 * v.m_Y + _33 * v.m_Z + _34);
}

bool CMatrix44::operator==(const CMatrix44 &other) const
{
    return _11 == other._11 && _12 == other._12 && _13 == other._13 && _14 == other._14
        && _21 == other._21 && _22 == other._22 && _23 == other._23 && _24 == other._24
        && _31 == other._31 && _32 == other._32 && _33 == other._33 && _34 == other._34
        && _41 == other._41 && _42 == other._42 && _43 == other._43 && _44 == other._44;
}

bool CMatrix44::operator!=(const CMatrix44 &other) const
{
    return _11 != other._11 || _12 != other._12 || _13 != other._13 || _14 != other._14
        || _21 != other._21 || _22 != other._22 || _23 != other._23 || _24 != other._24
        || _31 != other._31 || _32 != other._32 || _33 != other._33 || _34 != other._34
        || _41 != other._41 || _42 != other._42 || _43 != other._43 || _44 != other._44;
}

void CMatrix44::Set(const float &f)
{
    _11 = _12 = _13 = _14 =
    _21 = _22 = _23 = _24 =
    _31 = _31 = _33 = _34 =
    _41 = _42 = _43 = _44 = f;
}

void CMatrix44::Set(const float &f11, const float &f12, const float &f13, const float &f14,
                    const float &f21, const float &f22, const float &f23, const float &f24,
                    const float &f31, const float &f32, const float &f33, const float &f34,
                    const float &f41, const float &f42, const float &f43, const float &f44)
{
    _11 = f11; _12 = f12; _13 = f13; _14 = f14;
    _21 = f21; _22 = f22; _23 = f23; _24 = f24;
    _31 = f31; _32 = f32; _33 = f33; _34 = f34;
    _41 = f41; _42 = f42; _43 = f43; _44 = f44;
}

float CMatrix44::Determinant()
{
	return _14 * _23 * _32 * _41-_13 * _24 * _32 * _41-_14 * _22 * _33 * _41+_12 * _24 * _33 * _41 +
			_13 * _22 * _34 * _41-_12 * _23 * _34 * _41-_14 * _23 * _31 * _42+_13 * _24 * _31 * _42 +
			_14 * _21 * _33 * _42-_11 * _24 * _33 * _42-_13 * _21 * _34 * _42+_11 * _23 * _34 * _42 +
			_14 * _22 * _31 * _43-_12 * _24 * _31 * _43-_14 * _21 * _32 * _43+_11 * _24 * _32 * _43 +
			_12 * _21 * _34 * _43-_11 * _22 * _34 * _43-_13 * _22 * _31 * _44+_12 * _23 * _31 * _44 +
			_13 * _21 * _32 * _44-_11 * _23 * _32 * _44-_12 * _21 * _33 * _44+_11 * _22 * _33 * _44;
}

void CMatrix44::Identity()
{
    _11 = 1.0f; _12 = 0.0f; _13 = 0.0f; _14 = 0.0f;
    _21 = 0.0f; _22 = 1.0f; _23 = 0.0f; _24 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = 1.0f; _34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::Invert()
{
	Set(_23*_34*_42 - _24*_33*_42 + _24*_32*_43 - _22*_34*_43 - _23*_32*_44 + _22*_33*_44,
		_14*_33*_42 - _13*_34*_42 - _14*_32*_43 + _12*_34*_43 + _13*_32*_44 - _12*_33*_44,
		_13*_24*_42 - _14*_23*_42 + _14*_22*_43 - _12*_24*_43 - _13*_22*_44 + _12*_23*_44,
		_14*_23*_32 - _13*_24*_32 - _14*_22*_33 + _12*_24*_33 + _13*_22*_34 - _12*_23*_34,
		_24*_33*_41 - _23*_34*_41 - _24*_31*_43 + _21*_34*_43 + _23*_31*_44 - _21*_33*_44,
		_13*_34*_41 - _14*_33*_41 + _14*_31*_43 - _11*_34*_43 - _13*_31*_44 + _11*_33*_44,
		_14*_23*_41 - _13*_24*_41 - _14*_21*_43 + _11*_24*_43 + _13*_21*_44 - _11*_23*_44,
		_13*_24*_31 - _14*_23*_31 + _14*_21*_33 - _11*_24*_33 - _13*_21*_34 + _11*_23*_34,
		_22*_34*_41 - _24*_32*_41 + _24*_31*_42 - _21*_34*_42 - _22*_31*_44 + _21*_32*_44,
		_14*_32*_41 - _12*_34*_41 - _14*_31*_42 + _11*_34*_42 + _12*_31*_44 - _11*_32*_44,
		_12*_24*_41 - _14*_22*_41 + _14*_21*_42 - _11*_24*_42 - _12*_21*_44 + _11*_22*_44,
		_14*_22*_31 - _12*_24*_31 - _14*_21*_32 + _11*_24*_32 + _12*_21*_34 - _11*_22*_34,
		_23*_32*_41 - _22*_33*_41 - _23*_31*_42 + _21*_33*_42 + _22*_31*_43 - _21*_32*_43,
		_12*_33*_41 - _13*_32*_41 + _13*_31*_42 - _11*_33*_42 - _12*_31*_43 + _11*_32*_43,
		_13*_22*_41 - _12*_23*_41 - _13*_21*_42 + _11*_23*_42 + _12*_21*_43 - _11*_22*_43,
		_12*_23*_31 - _13*_22*_31 + _13*_21*_32 - _11*_23*_32 - _12*_21*_33 + _11*_22*_33);
	float d = 1.0f/Determinant();
	_11 *= d; _12 *= d; _13 *= d; _14 *= d;
	_21 *= d; _22 *= d; _23 *= d; _24 *= d;
	_31 *= d; _32 *= d; _33 *= d; _34 *= d;
	_41 *= d; _42 *= d; _43 *= d; _44 *= d;
}

void CMatrix44::Transpose()
{
  float f;

  f = _12;
  _12 = _21;
  _21 = f;

  f = _13;
  _13 = _31;
  _31 = f;

  f = _14;
  _14 = _41;
  _41 = f;

  f = _23;
  _23 = _32;
  _32 = f;

  f = _24;
  _24 = _42;
  _42 = f;

  f = _34;
  _34 = _43;
  _43 = f;
}

void CMatrix44::TransposeFrom(const CMatrix44 &m)
{
    _11 = m._11; _12 = m._21; _13 = m._31; _14 = m._41;
    _21 = m._12; _22 = m._22; _23 = m._32; _24 = m._42;
    _31 = m._13; _32 = m._23; _33 = m._33; _34 = m._43;
    _41 = m._14; _42 = m._24; _43 = m._34; _44 = m._44;
}


void CMatrix44::Multiply(const CMatrix44 &first, const CMatrix44 &second)
{
    _11 = first._11 * second._11 + first._12 * second._21 + first._13 * second._31 + first._14 * second._41;
    _12 = first._11 * second._12 + first._12 * second._22 + first._13 * second._32 + first._14 * second._42;
    _13 = first._11 * second._13 + first._12 * second._23 + first._13 * second._33 + first._14 * second._43;
    _14 = first._11 * second._14 + first._12 * second._24 + first._13 * second._34 + first._14 * second._44;

    _21 = first._21 * second._11 + first._22 * second._21 + first._23 * second._31 + first._24 * second._41;
    _22 = first._21 * second._12 + first._22 * second._22 + first._23 * second._32 + first._24 * second._42;
    _23 = first._21 * second._13 + first._22 * second._23 + first._23 * second._33 + first._24 * second._43;
    _24 = first._21 * second._14 + first._22 * second._24 + first._23 * second._34 + first._24 * second._44;

    _31 = first._31 * second._11 + first._32 * second._21 + first._33 * second._31 + first._34 * second._41;
    _32 = first._31 * second._12 + first._32 * second._22 + first._33 * second._32 + first._34 * second._42;
    _33 = first._31 * second._13 + first._32 * second._23 + first._33 * second._33 + first._34 * second._43;
    _34 = first._31 * second._14 + first._32 * second._24 + first._33 * second._34 + first._34 * second._44;

    _41 = first._41 * second._11 + first._42 * second._21 + first._43 * second._31 + first._44 * second._41;
    _42 = first._41 * second._12 + first._42 * second._22 + first._43 * second._32 + first._44 * second._42;
    _43 = first._41 * second._13 + first._42 * second._23 + first._43 * second._33 + first._44 * second._43;
    _44 = first._41 * second._14 + first._42 * second._24 + first._43 * second._34 + first._44 * second._44;
}

void CMatrix44::Scaling(const float &fX, const float &fY, const float &fZ)
{
    _11 = fX;   _12 = 0.0f; _13 = 0.0f; _14 = 0.0f;
    _21 = 0.0f; _22 = fY;   _23 = 0.0f; _24 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = fZ;   _34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::Scaling(const CVector3f &vScale)
{
    _11 = vScale.m_X;   _12 = 0.0f; _13 = 0.0f; _14 = 0.0f;
    _21 = 0.0f; _22 = vScale.m_Y;   _23 = 0.0f; _24 = 0.0f;
    _31 = 0.0f; _32 = 0.0f; _33 = vScale.m_Z;   _34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::Translation(const float &fX, const float &fY, const float &fZ)
{
    _11 = 1.0f; _12 = 0.0f; _13 = 0.0f; _14 = fX;
    _21 = 0.0f; _22 = 1.0f; _23 = 0.0f; _24 = fY;
    _31 = 0.0f; _32 = 0.0f; _33 = 1.0f; _34 = fZ;
    _41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::Translation(const CVector3f &vTranslation)
{
    _11 = 1.0f; _12 = 0.0f; _13 = 0.0f; _14 = vTranslation.m_X;
    _21 = 0.0f; _22 = 1.0f; _23 = 0.0f; _24 = vTranslation.m_Y;
    _31 = 0.0f; _32 = 0.0f; _33 = 1.0f; _34 = vTranslation.m_Z;
    _41 = 0.0f; _42 = 0.0f; _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::RotationX(const float &fRad)
{
    _11 = 1.0f; _12 = 0.0f;      _13 = 0.0f;      _14 = 0.0f;
    _21 = 0.0f; _22 = cos(fRad); _23 = sin(fRad); _24 = 0.0f;
    _31 = 0.0f; _32 = -_23;      _33 = _22;       _34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f;      _43 = 0.0f;      _44 = 1.0f;
}

void CMatrix44::RotationY(const float &fRad)
{
    _11 = cos(fRad); _12 = 0.0f; _13 = -sin(fRad); _14 = 0.0f;
    _21 = 0.0f;      _22 = 1.0f; _23 = 0.0f;       _24 = 0.0f;
    _31 = -_13;      _32 = 0.0f; _33 = _11;        _34 = 0.0f;
    _41 = 0.0f;      _42 = 0.0f; _43 = 0.0f;       _44 = 1.0f;
}

void CMatrix44::RotationZ(const float &fRad)
{
    _11 = cos(fRad); _12 = sin(fRad); _13 = 0.0f; _14 = 0.0f;
    _21 = -_12;      _22 = _11;       _23 = 0.0f; _24 = 0.0f;
    _31 = 0.0f;      _32 = 0.0f;      _33 = 1.0f; _34 = 0.0f;
    _41 = 0.0f;      _42 = 0.0f;      _43 = 0.0f; _44 = 1.0f;
}

void CMatrix44::RotationZYX(const float &fX, const float &fY, const float &fZ)
{
    float sinx = sin(fX);
    float siny = sin(fY);
    float sinz = sin(fZ);
    float cosx = cos(fX);
    float cosy = cos(fY);
    float cosz = cos(fZ);

    _11 = cosy*cosz; _12 = cosy*sinz; _13 = -siny; _14 = 0.0f;
    _21 = sinx*siny*cosz - sinz*cosx; _22 = sinx*siny*sinz+cosx*cosz; _23 = sinx*cosy; _24 = 0.0f;
    _31 = cosx*siny*cosz + sinx*sinz; _32 = cosx*siny*sinz - sinx*cosz;_33 = cosx*cosy;_34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f;      _43 = 0.0f;      _44 = 1.0f;
}

void CMatrix44::RotationZYX(const CVector3f &vRotation)
{
    float sinx = sin(vRotation.m_X);
    float siny = sin(vRotation.m_Y);
    float sinz = sin(vRotation.m_Z);
    float cosx = cos(vRotation.m_X);
    float cosy = cos(vRotation.m_Y);
    float cosz = cos(vRotation.m_Z);

    _11 = cosy*cosz; _12 = cosy*sinz; _13 = -siny; _14 = 0.0f;
    _21 = sinx*siny*cosz - sinz*cosx; _22 = sinx*siny*sinz+cosx*cosz; _23 = sinx*cosy; _24 = 0.0f;
    _31 = cosx*siny*cosz + sinx*sinz; _32 = cosx*siny*sinz - sinx*cosz;_33 = cosx*cosy;_34 = 0.0f;
    _41 = 0.0f; _42 = 0.0f;      _43 = 0.0f;      _44 = 1.0f;
}

void CMatrix44::RotationZXY(const float &fX, const float &fY, const float &fZ)
{
	float sinx = sin(fX);
	float siny = sin(fY);
	float sinz = sin(fZ);
	float cosx = cos(fX);
	float cosy = cos(fY);
	float cosz = cos(fZ);

	_11 = cosy*cosz - sinx*siny*sinz; _12 = cosy*sinz + sinx*siny*cosz; _13 = -cosx*siny; _14 = 0.0f;
	_21 =-cosx*sinz; _22 = cosx*cosz; _23 = sinx; _24 = 0.0f;
	_31 = siny*cosz + sinx*cosy*sinz; _32 = siny*sinz - sinx*cosy*cosz;_33 = cosx*cosy ;_34 = 0.0f;
	_41 = 0.0f; _42 = 0.0f;      _43 = 0.0f;      _44 = 1.0f;
}

void CMatrix44::RotationZXY(const CVector3f &vRotation)
{
	float sinx = sin(vRotation.m_X);
	float siny = sin(vRotation.m_Y);
	float sinz = sin(vRotation.m_Z);
	float cosx = cos(vRotation.m_X);
	float cosy = cos(vRotation.m_Y);
	float cosz = cos(vRotation.m_Z);

	_11 = cosy*cosz - sinx*siny*sinz; _12 = cosy*sinz + sinx*siny*cosz; _13 = -cosx*siny; _14 = 0.0f;
	_21 =-cosx*sinz; _22 = cosx*cosz; _23 = sinx; _24 = 0.0f;
	_31 = siny*cosz + sinx*cosy*sinz; _32 = siny*sinz - sinx*cosy*cosz;_33 = cosx*cosy ;_34 = 0.0f;
	_41 = 0.0f; _42 = 0.0f;      _43 = 0.0f;      _44 = 1.0f;
}

const CVector3f CMatrix44::GetZXYRotationVector()
{
	CVector3f rot;
	rot.m_Z = atan2(-_21, _22);
/*
	CMatrix33 invZ;
	invZ.RotationZ(-rot.m_Z);
	CMatrix33 noZ((*this) * invZ);
*/
	rot.m_X = asin(_23);
	rot.m_Y = atan2(-_13, _33);

	return rot;
}

void CMatrix44::Orthogonal(const float &fWidth, const float &fHeight, const float &fNearZ, const float &fFarZ)
{
//My own "lets try it out"-Version... :-) Works under OGL
    _11 = 2.0f / fWidth; _12 = 0.0f;            _13 = 0.0f;                    _14 = -1.0f;
    _21 = 0.0f;          _22 = -2.0f / fHeight; _23 = 0.0f;                    _24 = 1.0f;
    _31 = 0.0f;          _32 = 0.0f;            _33 = 2.0f / (fNearZ - fFarZ); _34 = (fNearZ + fFarZ) / (fFarZ - fNearZ);
    _41 = 0.0f;          _42 = 0.0f;            _43 = 0.0f;                    _44 = 1.0f;
/*
//DirectX-Version, OGL seems to want it different
    _11 = 2.0f / fWidth; _12 = 0.0f;           _13 = 0.0f;                 _14 = 0.0f;
    _21 = 0.0f;          _22 = 2.0f / fHeight; _23 = 0.0f;                 _24 = 0.0f;
    _31 = 0.0f;          _32 = 0.0f;           _33 = 1 / (fNearZ - fFarZ); _34 = 0.0f;
    _41 = 0.0f;          _42 = 0.0f;           _43 = fNearZ * _33;         _44 = 1.0f;
*/

}

void CMatrix44::Perspective(const float &fFov, const float &fAspectRatio, const float &fNearZ, const float &fFarZ)
{    
    float tmp(fFov/2);
    _22 = 1.0f/tan(tmp);
    _11 = _22 / fAspectRatio; _12 = 0.0f; _13 = 0.0f;                                _14 =  0.0f;
    _21 = 0.0f;                           _23 = 0.0f;                                _24 =  0.0f;
    _31 = 0.0f;               _32 = 0.0f; _33 = (fFarZ + fNearZ) / (fNearZ - fFarZ); _34 =  fNearZ * _33 - fNearZ;
    _41 = 0.0f;               _42 = 0.0f; _43 = -1.0f;                               _44 =  0.0f;
}


void CMatrix44::Log(word wTargetLog) const
{
  LOG(wTargetLog, "(%.2f, %.2f, %.2f, %.2f)\n"
                   "(%.2f, %.2f, %.2f, %.2f)\n"
                   "(%.2f, %.2f, %.2f, %.2f)\n"
                   "(%.2f, %.2f, %.2f, %.2f)\n",     _11, _12, _13, _14,
                                                     _21, _22, _23, _24,
                                                     _31, _32, _33, _34,
                                                     _41, _42, _43, _44);
}

CVector4f operator*(const CVector4f &v, const CMatrix44 &m)
{
    return CVector4f(m._11 * v.m_X + m._12 * v.m_Y + m._13 * v.m_Z + m._14 * v.m_W,
                     m._21 * v.m_X + m._22 * v.m_Y + m._23 * v.m_Z + m._24 * v.m_W,
                     m._31 * v.m_X + m._32 * v.m_Y + m._33 * v.m_Z + m._34 * v.m_W,
                     m._41 * v.m_X + m._42 * v.m_Y + m._43 * v.m_Z + m._44 * v.m_W);
}

CVector4f &operator*=(CVector4f &v, const CMatrix44 &m)
{
    v.Set(m._11 * v.m_X + m._12 * v.m_Y + m._13 * v.m_Z + m._14 * v.m_W,
          m._21 * v.m_X + m._22 * v.m_Y + m._23 * v.m_Z + m._24 * v.m_W,
          m._31 * v.m_X + m._32 * v.m_Y + m._33 * v.m_Z + m._34 * v.m_W,
          m._41 * v.m_X + m._42 * v.m_Y + m._43 * v.m_Z + m._44 * v.m_W);
    return v;
}