/****************************************************************************\
*                                                                            *
* 05.05.2006                                                                 *
* TBox3 - Header File                                                        *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
* Represents a box in R3, represented by one edge and 3 orthonormal vectors, *
* one along each side of the box.                                            *
\****************************************************************************/

#ifndef __TBOX3_H__
#define __TBOX3_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "TVector3.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<typename T>
class TBox3
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        TBox3() {}
        TBox3(const TVector3<T> &vPoint, const TVector3<T> &vA,
              const TVector3<T> &vB, const TVector3<T> &vC) : m_vPoint(vPoint), m_vA(vA), m_vB(vB), m_vC(vC) {}
        TBox3<T>(const TBox3<T> &other) : m_vPoint(other.m_vPoint), m_vA(other.m_vA), m_vB(other.m_vB), m_vC(other.m_vC) {}
        const TBox3<T> &operator =(const TBox3<T> &other) {m_vPoint = other.m_vPoint; m_vA = other.m_vA; m_vB = other.m_vB; m_vC = other.m_vC;
                                                           return *this;}

        const TBox3<T> operator+(const TVector3<T> &vec) const {return TBox3<T>(m_vPoint + vec, m_vA, m_vB, m_vC);}
        const TBox3<T> &operator+=(const TVector3<T> &vec) {m_vPoint += vec; return *this;}

		const CVector3f Center() const {return (m_vPoint + m_vA/2.0f + m_vB/2.0f + m_vC/2.0f);}
    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                    *
    \**************************************************************************/
    public:
        TVector3<T> m_vPoint;
        TVector3<T> m_vA, m_vB, m_vC;
        
};

typedef TBox3<float> CBox3f;

#endif
