/****************************************************************************\
*                                                                            *
* 19.02.2006                                                                 *
* TVector2 - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
* If you are wondering about all the consts and references, read             *
* "Effective C++" and "More Effective C++" by Scott Meyers                   *
\****************************************************************************/

#ifndef __TVECTOR2_H__
#define __TVECTOR2_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <math.h>

/****************************************************************************\
* DEFINITIONS	                                                               *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<typename T>
struct TVector2
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
public:
    TVector2() {}     //No initialization for speed purposes
    TVector2(const T &x, const T &y) {m_X = x; m_Y = y;}
    TVector2(const TVector2<T> &other) {m_X = other.m_X; m_Y = other.m_Y;}

    const TVector2<T> &operator=(const TVector2<T> &other) {m_X = other.m_X; m_Y = other.m_Y; return *this;}

    const TVector2<T> operator+(const TVector2<T> &other) const {return TVector2<T>(m_X + other.m_X, m_Y + other.m_Y);}
    const TVector2<T> &operator+=(const TVector2<T> &other) {m_X += other.m_X; m_Y += other.m_Y; return *this;}

    const TVector2<T> operator-(const TVector2<T> &other) const {return TVector2<T>(m_X - other.m_X, m_Y - other.m_Y);}
    const TVector2<T> &operator-=(const TVector2<T> &other) {m_X -= other.m_X; m_Y -= other.m_Y; return *this;}

    const TVector2<T> operator*(const T &factor) const {return TVector2<T>(m_X * factor, m_Y * factor);}
    const TVector2<T> &operator*=(const T &factor) {m_X *= factor; m_Y *= factor; return *this;}

    const TVector2<T> operator/(const T &quot) const {return TVector2<T>(m_X / quot, m_Y / quot);}
    const TVector2<T> &operator/=(const T &quot) {m_X /= quot; m_Y /= quot; return *this;}

    bool operator==(const TVector2<T> &other) {return m_X == other.m_X && m_Y == other.m_Y;}
    bool operator!=(const TVector2<T> &other) {return m_X != other.m_X || m_Y != other.m_Y;}

    T &operator[](const byte &i) {assert(i < 2 && "Error: Invalid index for TVector2::operator[]!");
                                  return *(&m_X + i);}

    const T &operator[](const byte &i) const {assert(i < 2 && "Error: Invalid index for TVector2::operator[]!");
                                  return *(&m_X + i);}

    void Set(const T &x, const T &y) {m_X = x; m_Y = y;}
    void SetRPhi(const T &r, const T &phi) {m_X = 0; m_Y = r;/*ToDo!!!*/}

    const double Length() const {return sqrt(m_X * m_X + m_Y * m_Y);}
    const T LengthSqr() const {return m_X * m_X + m_Y * m_Y;}

    void Normalize() {assert(LengthSqr() && "Error: TVector2::Normalize called for a zero vector!");
                      float fL = 1 / Length(); m_X *= fL; m_Y *= fL;}

    const T Dot(const TVector2<T> &other) const {return m_X * other.m_X + m_Y * other.m_Y}

    void Log(word wTargetLog) const {LOG(wTargetLog, "(%.2f, %.2f)", m_X, m_Y);}

    /**************************************************************************\
    * PUBLIC CLASS MEMBERS                                                     *
    \**************************************************************************/
    public:
        T m_X, m_Y;
};

typedef TVector2<float> CVector2f;
typedef TVector2<double> CVector2d;
typedef TVector2<sword> CVector2sw;
typedef TVector2<sdword> CVector2sdw;

#endif
