/****************************************************************************\
*                                                                            *
* 22.05.2005                                                                 *
* TSmartPointer - Header File                                                *
*                                                                            *
* (c) Raphael Charwot                                                        *
*     Thx to Photon                                                          *
*                                                                            *
\****************************************************************************/

#ifndef __TSMARTPOINTER_H__
#define __TSMARTPOINTER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template<class T>
class TSmartPointer
{
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    TSmartPointer() : m_pObject(NULL) {};
    TSmartPointer(T *pObject) : m_pObject(pObject) {AddRef();};
    TSmartPointer(const TSmartPointer &other) : m_pObject(other.m_pObject) {AddRef();}

    virtual ~TSmartPointer() {Release();}

    TSmartPointer &operator =(const TSmartPointer &other)
    {
      return *this = other.m_pObject;
    }

    TSmartPointer &operator =(T *pObject)
    {
      if(m_pObject != pObject)
      {
        Release();
        m_pObject = pObject;
        AddRef();
      }
      return *this;
    }

    T &operator *() const {assert(m_pObject && "Error: NULL SmartPointer dereferenced! (*)"); return *m_pObject;}
    T *operator ->() const {assert(m_pObject && "Error: NULL SmartPointer dereferenced! (->)"); return m_pObject;}

    operator T*() const {return m_pObject;}
    operator const T*() const {return m_pObject;}

    operator bool () const {return m_pObject != 0;}

    bool operator !() const {return !(m_pObject);}
    bool operator ==(const TSmartPointer &other) const {return m_pObject == other.m_pObject;};
    bool operator ==(const T &other) const {return m_pObject == pObject;};
    bool operator !=(const TSmartPointer &other) const {return m_pObject != other.m_pObject;};
    bool operator !=(const T &other) const {return m_pObject != pObject;};

  /**************************************************************************\
  * PRIVATE CLASS FUNCTIONS                                                  *
  \**************************************************************************/
  private:
    void AddRef()
    {
      if(m_pObject)
      {
        m_pObject->AddRef();
      }
    }

    void Release()
    {
      if(m_pObject)
      {
        if(m_pObject->Release())
        {
            delete m_pObject;
            m_pObject = NULL;
        }
      }
    }
  /**************************************************************************\
  * PRIVATE CLASS MEMBERS                                                    *
  \**************************************************************************/
  private:
    T *m_pObject;    
};

#endif
