/****************************************************************************\
*                                                                            *
* 30.06.2003                                                                 *
* TSingleton - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __TSINGLETON_H__
#define __TSINGLETON_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <assert.h>

#include <Memory/mmgr.h>

/****************************************************************************\
* DEFINITIONS	                                                              *
\****************************************************************************/

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
template <typename T>
class TSingleton
{
  /**************************************************************************\
  * PRIVATE CLASS MEMBERS                                                    *
  \**************************************************************************/
  private:
    static T *sm_pInstance;
    
  /**************************************************************************\
  * PROTECTED CLASS MEMBERS                                                  *
  \**************************************************************************/
  protected:
    
  /**************************************************************************\
  * PUBLIC CLASS MEMBERS                                                     *
  \**************************************************************************/
  public:
    
  /**************************************************************************\
  * PRIVATE CLASS FUNCTIONS                                                  *
  \**************************************************************************/
  private:
    TSingleton(const TSingleton &other);
    TSingleton &operator=(const TSingleton &other);
    
  /**************************************************************************\
  * PROTECTED CLASS FUNCTIONS                                                *
  \**************************************************************************/
  protected:
    TSingleton() {}
    
  /**************************************************************************\
  * PUBLIC CLASS FUNCTIONS                                                   *
  \**************************************************************************/
  public:
    static void CreateInstance()
    {
      if(!sm_pInstance)
      {
        sm_pInstance = new T;
      }
    }

    static void CreateInstance(T *pInstance)
    {
      assert(!sm_pInstance && "Error: Instance already created in TSingleton::CreateInstance(...)!")
      if(!sm_pInstance)
      {
        sm_pInstance = pInstance;
      }
    }

    static void DestroyInstance()
    {
      if(sm_pInstance)
      {
        delete sm_pInstance;
        sm_pInstance = NULL;
      }
    }

    static T *InstancePtr()
    {
      assert(sm_pInstance && "Error: Call to TSingleton::InstancePtr before creation");
      return sm_pInstance;
    }

    static T &Instance()
    {
      assert(sm_pInstance && "Error: Call to TSingleton::Instance before creation");

      return *sm_pInstance;
    }

    static bool IsCreated() {return sm_pInstance != NULL;}

    virtual ~TSingleton() {}
};

template <typename T>
T *TSingleton<T>::sm_pInstance = NULL;

#endif
