/****************************************************************************\
*                                                                            *
* 03.07.2003                                                                 *
* Tools - Source File                                                        *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "Tools.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
namespace Tools
{
  dword ReadWord(const char *pMem, dword dwAvailable, string &sDest)
  {
    dword dwFirst, dwLast;

    for(dwFirst = 0; pMem[dwFirst] == ' ' || pMem[dwFirst] == '\n'; ++dwFirst);

    for(dwLast = dwFirst; pMem[dwLast] != ' ' && pMem[dwLast] != '\n' && dwLast != dwAvailable; ++dwLast);

    if(dwLast - dwFirst)
	    sDest.assign(pMem + dwFirst, dwLast - dwFirst);

    return dwLast;
  }

  dword ReadLine(const char *pMem, dword dwAvailable, string &sDest)
  {
    dword dwCopied, dwSkip = 0;
    const char *pFound;

	  pFound = (const char *) ::memchr(pMem, '\n', dwAvailable);
	  if(pFound == NULL)
	  {
		  dwCopied = dwAvailable;
	  }
	  else
	  {
		  dwCopied = (dword) (pFound - pMem);
		  ++dwSkip;

		  if(pFound[-1] == '\r')
		  {
			  --dwCopied;
			  ++dwSkip;
		  }
	  }

	  sDest.assign(pMem, dwCopied);

	  return dwCopied + dwSkip;
  }

  void AppendBackslash(string &sStr)
  {
    if(!sStr.empty())
    {
	    char c = *sStr.rbegin();
	    if(c != '\\' && c != '/')
	    {
		    sStr += '\\';
	    }
    }
  }

  void RemoveEndBackslash(string &sStr)
  {
    if(!sStr.empty())
    {
		  const char *s = sStr.c_str();
		  const char *c = s + sStr.length() - 1;

		  while(*c == '\\' || *c == '/')
		  {
			  --c;
		  }

		  sStr.erase((c - s) + 1);
    }
  }

  const char *GetFileName(const char *sPath)
  {
    if(sPath && sPath[0] != '\0')
    {
      word i = (word) (strlen(sPath) - 1);
      
      while(i && sPath[i] != '\\' && sPath[i] != '/')
      {
        --i;
      }

      if(i)
        ++i;

      return sPath + i;
    }
    return NULL;
  }

  const char *GetFirstDir(const char *sPath, string &sDest)
  {
    if(sPath && sPath[0] != '\0')
    {
      word i, length;
      
      for(i = 0, length = (word) strlen(sPath); i < length && sPath[i] != '\\' && sPath[i] != '/'; ++i);

      sDest.assign(sPath, i);

      if(i < length)
      {
        return sPath + i + 1;
      }

      return NULL;
    }

    return NULL;
  }

  string GetFileExtension(const string &sFileName)
  {
    string::size_type pos = sFileName.rfind('.');

    if(pos != string::npos) // Falls Endung gefunden
    {
      return sFileName.substr(pos); // gib die Endung inklusive Punkt zurck
                                    //(um den Fall "kein Punkt im Dateinamen" unterscheiden zu knnen)
    }
    
    return "";
  }

  void TryFileExtension(string &sFileName, const char *sExtension)
  {
    if(!sFileName.empty())
    {
		  const char *s = sFileName.c_str();
		  const char *c = s + sFileName.length() - 1;

		  while(c != s && *c != '.')
		  {
			  --c;
		  }
      if(c == s)
      {
        sFileName += '.';
        sFileName += sExtension;
      }
    }
  }

  void ForceFileExtension(string &sFileName, const char *sExtension)
  {
    if(!sFileName.empty())
    {
		  const char *s = sFileName.c_str();
		  const char *c = s + sFileName.length() - 1;

		  while(c != s && *c != '.')
		  {
			  --c;
		  }
      if(c == s)
      {
        sFileName += '.';
        sFileName += sExtension;
      }
      else
      {
  		  sFileName.erase((c - s) + 1);
        sFileName += '.';
        sFileName += sExtension;
      }
    }
  }
}
