/****************************************************************************\
*                                                                            *
* 01.06.2006                                                                 *
* COGLCGSimpleBump - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "COGLCGSimpleBump.h"
#include "Renderer/COGLTriangleGroup.h"
#include "Renderer/COGLModelGeometry.h"
#include "Renderer/COGLRenderInfo.h"
#include "Display/Camera/CCamera.h"
#include "Renderer/COGLRenderer.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
#define SIMPLE_BUMP_SHADER_FILE    "Data/shader/simpleBump.cg"
#define SIMPLE_BUMP_FRAGMENT_ENTRY "mainFragment"
#define SIMPLE_BUMP_VERTEX_ENTRY   "mainVertex"

// DEBUG
//float          g_fRotAngle   = MY_PI/3;
//float          g_fLightSpeed = 0.5f;
//CVector3f      g_vLightPos;
//GLUquadricObj *g_lightSphere = NULL;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

COGLCGSimpleBump::COGLCGSimpleBump(COGLTriangleGroup *pTri, COGLRenderer *pRenderer)
	:COGLCGProgram(pTri,pRenderer){

		m_NormalMapTexID  = 0;
		m_ColorMapTexID   = 0;
		m_pIndices        = m_pGeometry->getIndexArray(); 
		m_pCoords         = m_pGeometry->getCoordArray();
		m_pNormals        = m_pGeometry->getNormalArray();
		m_pUvs            = m_pGeometry->getTexLayer(0)->getUvArray();
		m_pAmbientColor   = &m_pGeometry->getModel()->getMaterial(pTri->getMaterialIndex())->m_ambientC;
		m_pSpecularColor  = &m_pGeometry->getModel()->getMaterial(pTri->getMaterialIndex())->m_specularC;
		m_exp             = &m_pGeometry->getModel()->getMaterial(pTri->getMaterialIndex())->m_exp;

		m_pVertexTangents = NULL;
			
		// DEBUG
		//g_lightSphere = gluNewQuadric();
}

bool COGLCGSimpleBump::Create(dword normalMapTexID,dword colorMapTexID){

	if(glIsTexture(normalMapTexID) && glIsTexture(colorMapTexID)){
        
		if(m_pIndices && m_pCoords && m_pNormals && m_pUvs){
				
			m_pVertexProgInfo   = CompileProgram(ePROGRAM_VERTEX,SIMPLE_BUMP_SHADER_FILE,SIMPLE_BUMP_VERTEX_ENTRY);
			m_pFragmentProgInfo = CompileProgram(ePROGRAM_FRAGMENT,SIMPLE_BUMP_SHADER_FILE,SIMPLE_BUMP_FRAGMENT_ENTRY);
			
			if( m_pVertexProgInfo && m_pFragmentProgInfo){

				m_NormalMapTexID = normalMapTexID;
				m_ColorMapTexID  = colorMapTexID;

				cgVp_camPos            = cgGetNamedParameter(m_pVertexProgInfo->m_Program,   "vCamPos");
				cgVp_modelViewMatrix   = cgGetNamedParameter(m_pVertexProgInfo->m_Program,   "modelViewProjMatrix");
				cgVp_lightDirection    = cgGetNamedParameter(m_pVertexProgInfo->m_Program,   "vLightDirection");
				cgFp_lightDiffuseColor = cgGetNamedParameter(m_pFragmentProgInfo->m_Program, "vLightDiffuseColor");
				cgFp_ambientColor      = cgGetNamedParameter(m_pFragmentProgInfo->m_Program, "vAmbientColor");
				cgFp_specularColor     = cgGetNamedParameter(m_pFragmentProgInfo->m_Program, "vSpecularColor");
				cgFp_exp               = cgGetNamedParameter(m_pFragmentProgInfo->m_Program, "fExp"); 

				if(!m_pGeometry->getShaderData()->m_pVertexTangent){
					int vertexCount   = (int)m_pGeometry->getNumIndices();
					m_pVertexTangents = m_pGeometry->getShaderData()->m_pVertexTangent = new CVector4f[vertexCount];
					CalculateVertexTangents(vertexCount,m_pIndices,m_pCoords,m_pNormals,m_pUvs,m_pVertexTangents);
				}else{
					m_pVertexTangents = m_pGeometry->getShaderData()->m_pVertexTangent;
				}
				return true;
			}
		}
	}
	return false;
}

void COGLCGSimpleBump::SetupAndBind(){

	/* DEBUG LightSource
	glPushMatrix();
		glColor3f(1.0f, 1.0f, 0.3f);
		g_fRotAngle += (float)MY_PI * 0.02f * g_fLightSpeed;
		g_vLightPos.m_X = cos(g_fRotAngle) * 5.0f;
		g_vLightPos.m_Z = sin(g_fRotAngle) * 5.0f;
		g_vLightPos.m_Y = 1.0f; 
		
		// Position and render the light sphere
        glTranslatef(g_vLightPos.m_X, g_vLightPos.m_Y + 1.2f, g_vLightPos.m_Z);
		gluSphere(g_lightSphere, 0.5f, 20, 20);
	glPopMatrix();*/

	// Enable the Programs
	cgGLBindProgram(m_pVertexProgInfo->m_Program);
	cgGLBindProgram(m_pFragmentProgInfo->m_Program);
	cgGLEnableProfile(m_pVertexProgInfo->m_Profile);
	cgGLEnableProfile(m_pFragmentProgInfo->m_Profile);

	
	// Bring campos into object space
	glGetFloatv(GL_MODELVIEW_MATRIX, &m_mat._11);
	m_mat.Invert();
	m_vCamPos = m_mat * m_vec;
	
	// Parameters
	cgGLSetStateMatrixParameter(cgVp_modelViewMatrix,CG_GL_MODELVIEW_PROJECTION_MATRIX,CG_GL_MATRIX_IDENTITY);
	cgGLSetParameter3f(cgVp_camPos, m_vCamPos.m_X, m_vCamPos.m_Y, m_vCamPos.m_Z);
	cgGLSetParameter3fv(cgVp_lightDirection,&m_pRenderer->GetLightDir().m_X);
	cgGLSetParameter3f(cgFp_lightDiffuseColor,1.0f,1.0f,1.0f);
    cgGLSetParameter4fv(cgFp_ambientColor,(float*)&(m_pAmbientColor->ElementMultiply(m_pRenderer->getAmbientLightColor(0))));
	cgGLSetParameter4fv(cgFp_specularColor,(float*)m_pSpecularColor);
	cgGLSetParameter1f(cgFp_exp,((*m_exp)/10));
	
	// Disable Lighting
	glDisable(GL_LIGHTING);
	
	// Enable and bind the texture
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,m_ColorMapTexID);

	// Enable and bind the normal map
	glActiveTextureARB(GL_TEXTURE1_ARB);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,m_NormalMapTexID);

	glBegin(GL_TRIANGLES);

		for(int i = 0; i < (int)m_pGeometry->getNumIndices() ; ++i)
		{
			
			//Specify the normal vector
			glNormal3fv((float*)&(m_pNormals[m_pIndices[i]]));
			//Specify the uvs
			glMultiTexCoord2fv(GL_TEXTURE0,(float*)&(m_pUvs[m_pIndices[i]]));
			//Specify the tangent vector
			glMultiTexCoord4fv(GL_TEXTURE1,(float*)&m_pVertexTangents[i]);
			
			glVertex3fv((float*)&(m_pCoords[m_pIndices[i]]));

			++i;
			//Specify the normal vector
			glNormal3fv((float*)&(m_pNormals[m_pIndices[i]]));
			//Specify the uvs
			glMultiTexCoord2fv(GL_TEXTURE0,(float*)&(m_pUvs[m_pIndices[i]]));
			//Specify the tangent vector
			glMultiTexCoord4fv(GL_TEXTURE1,(float*)&m_pVertexTangents[i]);

			glVertex3fv((float*)&(m_pCoords[m_pIndices[i]]));

			++i;
			//Specify the normal vector
			glNormal3fv((float*)&(m_pNormals[m_pIndices[i]]));
			//Specify the uvs
			glMultiTexCoord2fv(GL_TEXTURE0,(float*)&(m_pUvs[m_pIndices[i]]));
			//Specify the tangent vector
			glMultiTexCoord4fv(GL_TEXTURE1,(float*)&m_pVertexTangents[i]);

			glVertex3fv((float*)&(m_pCoords[m_pIndices[i]]));
		}

	glEnd();
		
	// Disable textures
	glActiveTextureARB(GL_TEXTURE0_ARB);
	glDisable(GL_TEXTURE_2D);

	glActiveTextureARB(GL_TEXTURE1_ARB);
	glDisable(GL_TEXTURE_2D);

}

void COGLCGSimpleBump::Unbind(){

	cgGLDisableProfile(m_pVertexProgInfo->m_Profile);
	cgGLDisableProfile(m_pFragmentProgInfo->m_Profile);

}

void COGLCGSimpleBump::Destroy(){

}

COGLCGSimpleBump::~COGLCGSimpleBump(){
	
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
