/****************************************************************************\
*                                                                            *
* 09..06.2006                                                                 *
* COGLGlobalRenderSettings - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLGLOBALRENDERSETTINGS_H__
#define __COGLGLOBALRENDERSETTINGS_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Renderer/IGlobalRenderSettings.h>
#include <Renderer/COGLRenderInfo.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class COGLRenderer;
class COGLModelFactory;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class COGLGlobalRenderSettings : public IGlobalRenderSettings{

	friend class COGLRenderer;
	friend class COGLModelFactory;

public:

	virtual bool IsGlobalRenderStateSet();

	virtual void DisableAllRenderFeatures();
	virtual void EnableAllRenderFeatures();

	virtual void StoreGlobalRenderState();
	virtual void RestoreGlobalRenderState();

	virtual void EnableVertexColors(bool bEnable);
	virtual void EnableLighting(bool bEnable);
	virtual void EnableTexturing(bool bEnable);
	virtual void EnableShader(bool bEnable);

	virtual void SetMipMappingOverride(bool bEnable);
	virtual void SetTextureQualityOverride(EOGLTextureQuality eQuality, bool bEnable);

	virtual bool SetRenderModeOverride(EOGLRenderMode eMode, bool bEnable);
	virtual bool SetCacheModeOverride(EOGLCacheMode eMode, bool bEnable);

	virtual void SetStipplePolygon(bool bEnable);
	virtual void SetWireFrame(bool bEnable,bool bUniformColor);
	virtual void SetDrawColor(CVector4f &);

	virtual void EnableBlendingEffects(bool bEnable);
	
	~COGLGlobalRenderSettings(){m_pRenderer = NULL;}

protected:

	COGLGlobalRenderSettings(IRenderer *pRenderer);

	COGLRenderer * m_pRenderer;
	CVector4f      m_globalDrawColor;
	CVector4f      m_bgColor;

	dword          m_GlobalRenderState;
	dword          m_StoredGlobalRenderState;
	bool           m_bGlobalRenderStateIsActive;

	dword          m_GlobalTextureState;	
	bool           m_bGlobalTextureStateIsActive;

	EOGLRenderMode m_RenderModeOverride;
	bool           m_bGlobalRenderModeStateIsActive;

	EOGLCacheMode  m_CacheModeOverride;
	bool           m_bGlobalCacheModeStateIsActive;

	bool           m_bUseCurrentOverride;
};

#endif