/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* COGLModelGeometry - Header File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLMODELGEOMETRY_H__
#define __COGLMODELGEOMETRY_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/COGLTriangleGroup.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

class COGLRenderer;
class COGLModelFactory;
class CGeometryInfo;

//****************************************************************************

class COGLMaterial{
public:
	TVector4<float> m_ambientC;
	TVector4<float> m_diffuseC;
	TVector4<float> m_selfIllumC;
	TVector4<float> m_specularC;
	float           m_exp;
	float           m_opacity;
	float           m_reflectivity;
	bool            m_useLighting;
};

//****************************************************************************

class COGLTexture{

	friend class COGLModelFactory;
	friend class COGLRenderer;

public:
	COGLTexture();
	~COGLTexture(){}
	
	dword getTexBufID();
	const std::string & getFilename();
	
private:
	void setTexBufID(dword);
	void setFileName(const std::string &);

	std::string  m_fileName;
	dword        m_glTexBufID;
	dword        m_target;
	dword        m_minFilter;
	dword        m_maxFilter;
	bool         m_bMipMap;
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class COGLModelGeometry : public IModelGeometry
{
	friend class COGLRenderer;
	friend class COGLModelFactory;

	/**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		COGLModelGeometry(){};

        bool Create(COGLRenderer *pRenderer, COGLModelFactory *pHost, CGeometryInfo *pGeoInfo);
        virtual bool RayIntersection(const CVector3f &vPos, const CVector3f &vRot, const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual void Render();
        virtual const CSpheref GetBoundingSphere();
        virtual const CAABox3f GetAABoundingBox();

		COGLTexture * const getTexture(dword nmb);
		COGLMaterial * const getMaterial(dword nmb);
		COGLTriangleGroup * const getTriangleGroup(dword nmb);

		COGLRenderer * const getRenderer(){return m_pRenderer;}
		
		virtual ~COGLModelGeometry();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
	       
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        COGLModelGeometry(const COGLModelGeometry &);
        COGLModelGeometry &operator =(const COGLModelGeometry &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
		std::vector<COGLTriangleGroup *> m_TriangleGroups;
		std::vector<COGLMaterial *>      m_Materials;
		std::vector<COGLTexture *>       m_Textures;
		
	/**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        COGLRenderer *m_pRenderer;
		COGLModelFactory *m_pHost;
};

#endif