/****************************************************************************\
*                                                                            *
* 18.03.2006                                                                 *
* COGLRenderer - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLRENDERER_H__
#define __COGLRENDERER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Renderer/IRenderer.h>
#include <Renderer/COGLFont.h>
#include <Renderer/COGLModelGeometry.h>
#include <Renderer/COGLStockGeometry.h>
#include <Loader/ITextureLoader.h>
#include <Renderer/COGLGlobalRenderSettings.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class COGLCGProgram;
/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class COGLRenderer : public IRenderer, private IWindowHandler
{
	/**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		COGLRenderer(){Clear();m_pGlobalRenderSettings = new COGLGlobalRenderSettings(this);}
        virtual ~COGLRenderer() {Destroy();}
        
		virtual bool Create(const TSmartPointer<IWindow> &pWindow);
        
		virtual TSmartPointer<IFont> GetFontObject(const std::string &sFontName, byte bSize, bool bBold, bool bItalic);
		virtual TSmartPointer<IGlobalRenderSettings> GetGlobalSettings(){return (IGlobalRenderSettings *)(COGLGlobalRenderSettings*)m_pGlobalRenderSettings;}
        
        virtual void SetBgColor(dword dwColor);
				
		virtual void SetMatrixMode(EMatrixMode mode);
        virtual void SetMatrix(const CMatrix44 &mat);
        virtual void MultMatrix(const CMatrix44 &mat);
        virtual void SaveMatrix();
        virtual void RestoreMatrix();
        
		virtual void EnableZTest(float fEnable);
		virtual void EnableZWrite(float fEnable);
		virtual void EnableAlphaTest(float fEnable);
		virtual void EnableBlending(float fEnable);

		virtual void SetLightDir(const CVector3f &vDir);
		virtual const CVector4f &GetLightDir() const;
		void SetLightColor(byte lightNmb,const CVector4f & ambientColor,const CVector4f &diffuseColor);
		virtual CVector4f const & getAmbientLightColor(byte lightNmb);
	

		virtual void EnableBlendEffects(bool bEnable){m_bBlendEnableFlag = bEnable;}

		virtual void StartRendering();
		virtual void EndRendering();

       	virtual bool CompileList(COGLTriangleGroup *,COGLModelGeometry *);
		virtual bool ComplieVBO(COGLTriangleGroup *);
		virtual bool CreateTexture(const CImageInfo * const,COGLTexture *);

		void Render(COGLModelGeometry *);
		void RenderStockObject(COGLStockGeometry *);
		void RenderText(COGLFont *pFont, SText *pText);

		void Free(COGLModelGeometry *obj);
		void Free(COGLStockGeometry *obj);
       
        virtual void Destroy();
             
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		void RenderMegaSlow();
		void RenderVertexArrays();
		void RenderVertexBufferObjects();
		void RenderShader();
		void ReportErrors();
		
	/**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        void OnWndSize(word wWidth, word wHeight);
		void PushState();
		void PopState();

        //Private copy constructor and assignment operator to prevent mistakes:
        COGLRenderer(const COGLRenderer &);
        COGLRenderer &operator =(const COGLRenderer &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		typedef std::vector<TSmartPointer<COGLFont> > FontObjectList;

		TSmartPointer<COGLGlobalRenderSettings> m_pGlobalRenderSettings;
		FontObjectList           m_FontObjects;

        TSmartPointer<IWindow> m_pWindow;
        HDC                    m_hDC;
        HGLRC                  m_hGLRC;
        
		EMatrixMode m_MatrixMode;  
		dword       m_eGlError;
		int         m_i,m_j,m_k,m_l;

		bool m_bVertexBufferSupported;
		bool m_bTextureCubeMapSupported;
		bool m_bBlendEnableFlag;

		COGLModelGeometry    * m_pActiveMG;
		COGLTriangleGroup    * m_pActiveTriGroup;
		dword                * m_pActiveIndexArray;
		CVector3f            * m_pActiveCoordPtr;
		CVector3f            * m_pActiveNormalPtr;
		CVector4f            * m_pActiveColorPtr;
		COGLMaterial         * m_pActiveMat;
		int                    m_ActiveNmbOfTexLayers;
		word                   m_ActiveState;

		byte                 * m_pPattern;

		CVector4f m_vLightDir;
		CVector4f m_vLightDiffuseColor;
		CVector4f m_vLightAmbientColor;
};

#endif
