/****************************************************************************\
*                                                                            *
* 24.05.2006                                                                 *
* COGLTriangleGroup - Source File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "COGLTriangleGroup.h"
#include "COGLModelGeometry.h"
#include "COGLRenderer.h"
#include "COGLRenderInfo.h"
/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

COGLTriangleGroup::COGLTriangleGroup(COGLModelGeometry *pMg,dword numVertices, dword numIndices, byte nmbOfTexLayers, bool vertexColors){

	m_pModelGeometry = pMg;
	m_NumVertices    = numVertices;
	m_NumIndices     = numIndices;
	m_RenderInfo     = new COGLRenderInfo();

	if(m_NumIndices > 0){

		m_MaterialIndex = 0;
		m_pIndexArray.m_DataArray  = new dword[numIndices];
		m_pCoordArray.m_DataArray  = new CVector3f[numVertices];
		m_pNormalArray.m_DataArray = new CVector3f[numVertices];
		if(vertexColors){
			m_pVertexColorArray.m_DataArray  = new CVector4f[numVertices];
		}else{
			m_pVertexColorArray.m_DataArray  = NULL;
		}
		if(nmbOfTexLayers > 0)
		{
			for(byte i = 0 ; i < nmbOfTexLayers ; ++i){
				COGLTexLayer *newLayer = new COGLTexLayer();
				newLayer->createUVArray(numVertices);
				m_TextureLayers.push_back(newLayer);
			}
		}
		m_pGlobalReflTexLayer = NULL;

	}else{
		m_MaterialIndex     = -1;
		m_pIndexArray.m_DataArray        = NULL;
		m_pCoordArray.m_DataArray        = NULL;
		m_pNormalArray.m_DataArray       = NULL;
		m_pVertexColorArray.m_DataArray  = NULL;
	}
}

void COGLTriangleGroup::setMaterialIndex(sword matIndex){
	if( matIndex >= 0 ) m_MaterialIndex = matIndex;
}

sword COGLTriangleGroup::getMaterialIndex(){
	return m_MaterialIndex;
}

dword * const COGLTriangleGroup::getIndexArray(){
	return m_pIndexArray.m_DataArray ;
}
CVector3f * const COGLTriangleGroup::getCoordArray(){
	return m_pCoordArray.m_DataArray ;
}
CVector3f * const COGLTriangleGroup::getNormalArray(){
	return m_pNormalArray.m_DataArray ;
}
CVector4f * const COGLTriangleGroup::getVertexColorArray(){
	return m_pVertexColorArray.m_DataArray ;
}
COGLTexLayer * const COGLTriangleGroup::getTexLayer(byte nmb){
	if( nmb >= 0 && nmb < (byte) m_TextureLayers.size()){
		return m_TextureLayers[nmb];
	}else{
		return NULL;
	}
}

bool COGLTriangleGroup::Build(EOGLRenderMode eMode,bool bOverrideMode){
	switch(eMode){
		case OGL_RENDER_MODE_VERTEXARRAY:
			return BuildVertexArrays(bOverrideMode);
		case OGL_RENDER_MODE_MEGASLOW:
			return BuildMegaSlow(bOverrideMode);
		case OGL_RENDER_MODE_VERTEXBUFFEROBJECT:
			return BuildVertexBufferObjects(bOverrideMode);
		case OGL_RENDER_MODE_SHADER:
			return BuildShaderMode(bOverrideMode);
		default:
			LOG(TL_ERROR, "Error: Unknown model typ in CCOGLTriangleGroup::Build()");
			return false;
	}
}

bool COGLTriangleGroup::BuildMegaSlow(bool bOverrideMode)
{
	if(bOverrideMode){
		m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_MEGASLOW;
	}else{
		m_RenderInfo->m_eRenderMode = m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_MEGASLOW;
	}
	return true;
}

bool COGLTriangleGroup::BuildVertexArrays(bool bOverrideMode)
{
	if(bOverrideMode){
		m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXARRAY;
	}else{
		m_RenderInfo->m_eRenderMode = m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXARRAY;
	}
	return true;
}
	
bool COGLTriangleGroup::BuildVertexBufferObjects(bool bOverrideMode)
{
	if(!m_RenderInfo->m_bIsVertexBuffered){
		if(m_pModelGeometry->getRenderer()->ComplieVBO(this)){
			if(bOverrideMode){
				m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXBUFFEROBJECT;
			}else{
				m_RenderInfo->m_eRenderMode = m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXBUFFEROBJECT;
			}
			m_RenderInfo->m_bIsVertexBuffered = true;
			return true;
		}else{
			LOG(TL_ERROR,"Could not create VBO in COGLModelGeometry::BuildVertexBufferObjects()");
			return false;
		}
	}else{
		if(bOverrideMode){
			m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXBUFFEROBJECT;
		}else{
			m_RenderInfo->m_eRenderMode = m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_VERTEXBUFFEROBJECT;
		}
		return true;
	}
}

bool COGLTriangleGroup::BuildShaderMode(bool bOverrideMode)
{
	if(bOverrideMode){
		m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_SHADER;
	}else{
		m_RenderInfo->m_eRenderMode = m_RenderInfo->m_eCurrentRenderMode = OGL_RENDER_MODE_SHADER;
	}
	return true;
}

bool COGLTriangleGroup::Compile(EOGLRenderMode eMode,bool bOverrideMode)
{
	if(Build(eMode,bOverrideMode)){
		if(m_pModelGeometry->getRenderer()->CompileList(this,m_pModelGeometry)){
			if(bOverrideMode){
				m_RenderInfo->m_eCurrentCacheMode = OGL_CACHE_MODE_EXECUTE;
			}else{
				m_RenderInfo->m_eCacheMode = OGL_CACHE_MODE_EXECUTE;
			}
			return true;
		}else{
			if(bOverrideMode){
				m_RenderInfo->m_eCurrentCacheMode = OGL_CACHE_MODE_COMPILE_ERROR;
			}else{
				m_RenderInfo->m_eCacheMode = OGL_CACHE_MODE_COMPILE_ERROR;
			}
			return false;
		}
	}else{
		return false;
	}
}

bool COGLTriangleGroup::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fClosestHit) const
{
	CVector3f v0, edge1, edge2;
	CVector3f vCross;
	float det, u, v, t;
	CVector3f tvec;
	bool bHit = false;

	for(dword i = 0; i < m_NumIndices; ++i)
	{
		v0 = m_pCoordArray.m_DataArray[m_pIndexArray.m_DataArray[i]];
		++i;
		edge1 = m_pCoordArray.m_DataArray[m_pIndexArray.m_DataArray[i]] - v0;
		++i;
		edge2 = m_pCoordArray.m_DataArray[m_pIndexArray.m_DataArray[i]] - v0;
		vCross = vDir * edge2;
		det = edge1.Dot(vCross);
		if(det < 0.0001f && det > -0.0001f) //Lies in triangle plane
			continue;

		if(det > 0.0f)
		{
			tvec = vOrigin - v0;
		}
		else
		{
			tvec = v0 - vOrigin;
			det = -det;
		}

		// Calculate U parameter and test bounds
		u = tvec.Dot(vCross);
		if(u < 0.0f || u > det)
			continue;

		// Prepare to test V parameter
		CVector3f qvec;
		qvec = tvec * edge1;
		// Calculate V parameter and test bounds
		v = vDir.Dot(qvec);
		if(v < 0.0f || u + v > det)
			continue;

		// Calculate t, scale parameters, ray intersects triangle
		t = edge2.Dot(qvec);
		det = 1.0f / det;
		t *= det;
		u *= det;
		v *= det;
		if(t < fClosestHit)
		{
			bHit = true;
			fClosestHit = t;
			vIntersectionPos = v0 + edge1 * u + edge2 * v;
		}
	}

	return bHit;
}

COGLReflectionMapTexLayer * const COGLTriangleGroup::createGlobalReflTexLayer(){
	if(m_pGlobalReflTexLayer == NULL){
		SAFE_DELETE(m_pGlobalReflTexLayer); 
		m_pGlobalReflTexLayer = new COGLReflectionMapTexLayer();
	}
	return m_pGlobalReflTexLayer;
}

COGLTriangleGroup::~COGLTriangleGroup()
{

	delete m_RenderInfo;

	SAFE_DELETE_ARRAY(m_pIndexArray.m_DataArray);
	SAFE_DELETE_ARRAY(m_pCoordArray.m_DataArray);
	SAFE_DELETE_ARRAY(m_pNormalArray.m_DataArray);
	SAFE_DELETE_ARRAY(m_pVertexColorArray.m_DataArray);

	for(std::vector<COGLTexLayer *>::iterator it = m_TextureLayers.begin(); it != m_TextureLayers.end(); ++it)
	{
		SAFE_DELETE((*it));
	}
	SAFE_DELETE(m_pGlobalReflTexLayer);

	
}


/****************************************************************************\
* COGLTEXLAYER                                                               *
\****************************************************************************/

COGLTexLayer::COGLTexLayer()
{ 
	m_pUvs.m_DataArray = NULL; 
	m_textureIndex = -1;
}

dword COGLTexLayer::getTextureIndex()
{
	return m_textureIndex;
}

void COGLTexLayer::setTextureIndex(sword texIndex)
{
	if(texIndex >= 0)
	{
		m_textureIndex = texIndex;
	}
	else
	{
		m_textureIndex = -1;
	}
}

CVector2f * const COGLTexLayer::getUvArray()
{
	return m_pUvs.m_DataArray;
}

CVector2f * COGLTexLayer::createUVArray(dword nmbOfUvs)
{
	SAFE_DELETE_ARRAY(m_pUvs.m_DataArray);
	m_pUvs.m_DataArray = new CVector2f[nmbOfUvs];
	return m_pUvs.m_DataArray;
}

COGLTexLayer::~COGLTexLayer()
{
	SAFE_DELETE_ARRAY(m_pUvs.m_DataArray);
}