/****************************************************************************\
*                                                                            *
* 24.05.2006                                                                 *
* COGLTriangleGroup - Header File                                            *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __COGLTRIANGLEGROUP_H__
#define __COGLTRIANGLEGROUP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Math/TVector2.h>
#include <Math/TVector3.h>
#include <Math/TVector4.h>
#include <Math/CMatrix33.h>
#include <Renderer/COGLRenderInfo.h>
/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

class COGLModelGeometry;
class COGLRenderInfo;

//****************************************************************************

template <class T>
class COGLRenderData{

public:
    T *m_DataArray;
	unsigned int m_ID;
};

//****************************************************************************

enum EOGLTextureBlendMode{
	e_TBM_REPLACE  = 0x1E01,    
	e_TBM_MODULATE = 0x2100,  
	e_TBM_ADD      = 0x0104,
	e_TBM_COMBINE  = 0x8570
};

class COGLTexLayer{

	friend class COGLRenderer;
	friend class COGLTriangleGroup;
	friend class COGLModelFactory;

public:
	COGLTexLayer();
	~COGLTexLayer();

	void setTextureIndex(sword texIndex);
	dword getTextureIndex();

	void setBlendMode(EOGLTextureBlendMode eMode){m_texureBlendMode = eMode;}
	EOGLTextureBlendMode getBlendMode(){return m_texureBlendMode;}

	CVector2f * createUVArray(dword nmbOfUvs);
	CVector2f * const getUvArray();
	
private:
	sword                     m_textureIndex;
	EOGLTextureBlendMode      m_texureBlendMode;
	COGLRenderData<CVector2f> m_pUvs;
};

//****************************************************************************

struct TBNMatrix{CVector3f col[3];};

struct SShaderData{

	SShaderData(){m_pTBNMatrices = NULL;m_pVertexTangent = NULL;}
	~SShaderData(){SAFE_DELETE_ARRAY(m_pTBNMatrices);SAFE_DELETE_ARRAY(m_pVertexTangent);}

	TBNMatrix * m_pTBNMatrices;
	CVector4f * m_pVertexTangent;
};

//****************************************************************************

enum EOGLReflectionMapType{
	eRefl_Sphere,
	eRefl_Cube,
	eRefl_None
};

class COGLReflectionMapTexLayer{

	friend class COGLModelFactory;
	friend class COGLTriangleGroup;
	friend class COGLRenderer;

private:
	COGLReflectionMapTexLayer(){m_Type = eRefl_None; m_TexBufID = 0;}
	void setCube(){m_Type = eRefl_Cube; m_TexBufID = 0;}
	void setSphere(dword texBufID){m_Type = eRefl_Sphere; m_TexBufID = texBufID;}
	void setReflectivity(float refl){m_mixColor[3] = 1.0f - refl;}

	EOGLReflectionMapType m_Type;
	dword                 m_TexBufID;
	CVector4f             m_mixColor;
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/

class COGLTriangleGroup{

	friend class COGLRenderer;
	friend class COGLModelGeometry;
	friend class COGLCGProgram;

public:

	bool Build(EOGLRenderMode eMode,bool bOverrideMode);
	bool BuildMegaSlow(bool bOverrideMode);
	bool BuildVertexArrays(bool bOverrideMode);
	bool BuildVertexBufferObjects(bool bOverrideMode);
	bool BuildShaderMode(bool bOverrideMode);
	bool Compile(EOGLRenderMode eMode,bool bOverrideMode);

	void  setMaterialIndex(sword matIndex);
	sword getMaterialIndex();
	
	inline dword getNumIndices(){return m_NumIndices;}
	inline dword getNumTexLayers(){return (dword)m_TextureLayers.size();}

	dword        * const getIndexArray();
	CVector3f    * const getCoordArray();
	CVector3f    * const getNormalArray();
	CVector4f    * const getVertexColorArray();
	COGLTexLayer * const getTexLayer(byte nmb);

	COGLModelGeometry * getModel(){return m_pModelGeometry;}

	SShaderData    * const getShaderData(){return &m_shaderData;}
	COGLRenderInfo * const getRenderInfo(){return m_RenderInfo;}

	COGLReflectionMapTexLayer * const createGlobalReflTexLayer();

	bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fClosestHit) const;

protected:
	COGLTriangleGroup(COGLModelGeometry *pMg,dword numVertices, dword numIndices, byte nmbOfTexLayers, bool vertexColors);
	~COGLTriangleGroup();

private:
	sword m_MaterialIndex;
	dword m_NumVertices;
	dword m_NumIndices;
	
	COGLRenderData<dword>        m_pIndexArray;
	COGLRenderData<CVector3f>    m_pCoordArray;
	COGLRenderData<CVector3f>    m_pNormalArray;
	COGLRenderData<CVector4f>    m_pVertexColorArray;

	std::vector<COGLTexLayer *> m_TextureLayers;
	COGLReflectionMapTexLayer * m_pGlobalReflTexLayer;
	SShaderData                 m_shaderData;

	COGLRenderInfo    *m_RenderInfo;
	COGLModelGeometry *m_pModelGeometry;
};

#endif