/****************************************************************************\
*                                                                            *
* 27.03.2006                                                                 *
* IModelGeometry - Header File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __IMODELGEOMETRY_H__
#define __IMODELGEOMETRY_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Math/TAABox3.h>
#include <Math/TSphere.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
const float cg_fMaxIntersectionDist = 10000000000000000000000.0f;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IModelGeometry : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
		IModelGeometry(){}
        
        virtual void Render() = 0;
        virtual const CSpheref GetBoundingSphere() = 0;
        virtual const CAABox3f GetAABoundingBox() = 0;
		virtual bool RayIntersection(const CVector3f &vPos, const CVector3f &vRot, const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const = 0;

		virtual ~IModelGeometry(){}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        IModelGeometry(const IModelGeometry &);
        IModelGeometry &operator =(const IModelGeometry &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        
};

#endif
