/****************************************************************************\
*                                                                            *
* 25.04.2006                                                                 *
* CFModSoundMgr - Source File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CFModSoundMgr.h"
#include "CFModSound.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/

CFModSoundMgr::CFModSoundMgr(){
	m_pSystem = NULL;
}

bool CFModSoundMgr::Create(){

	if(!m_pSystem){

		FMOD_RESULT result; 

		//Create the main system object.
		result = FMOD::System_Create(&m_pSystem);	

		if (result != FMOD_OK)
		{
			LOG(TL_ERROR, "Error: Creating System Object failed in CFModSoundMgr::Create");
			return false;
		}

		result = m_pSystem->init(100, FMOD_INIT_NORMAL, 0);

		if (result != FMOD_OK)
		{
			LOG(TL_ERROR, "Error: Initializing FMod failed in CFModSoundMgr::Create");
			return false;
		}

	}
	return true;

}

TSmartPointer<ISound> CFModSoundMgr::LoadSound(ESoundType sType, std::string const & fileName){

	TSmartPointer<CFModSound> newSound = new CFModSound(this);

	std::map<std::string,FMOD::Sound *>::iterator tIt;
    tIt = m_currentSounds.find(fileName);
	
	FMOD_RESULT result = FMOD_OK;
	FMOD_MODE   mode   = FMOD_CREATESAMPLE;

	switch(sType){
		case eDECOMPRESSED_SAMPLE:{
			if(tIt != m_currentSounds.end()){
				newSound->m_pSound = (*tIt).second;
			}else{
				result = m_pSystem->createSound(fileName.c_str(), FMOD_CREATESAMPLE , 0,&newSound->m_pSound);
				result = newSound->m_pSound->getMode(&newSound->m_pMode);
			}
		}
		break;
		case eCOMPRESSED_SAMPLE:{
			if(tIt != m_currentSounds.end()){
				newSound->m_pSound = (*tIt).second;
			}else{
				result = m_pSystem->createSound(fileName.c_str(), FMOD_DEFAULT | FMOD_CREATECOMPRESSEDSAMPLE , 0,&newSound->m_pSound);
				result = newSound->m_pSound->getMode(&newSound->m_pMode);
			}
		}
		break;
		case eSTREAM:{
			if(tIt != m_currentSounds.end()){
				newSound->m_pSound = (*tIt).second;
			}else{
				result = m_pSystem->createStream(fileName.c_str(), FMOD_DEFAULT | FMOD_LOOP_NORMAL, 0,&newSound->m_pSound);
				result = newSound->m_pSound->getMode(&newSound->m_pMode);
			}
		}
		break;
	}

	if(result == FMOD_OK){
		m_currentSounds.insert(std::make_pair(fileName,newSound->m_pSound));
		return (ISound *)(CFModSound *)newSound;
	}else{
		return NULL;
	}
}

void CFModSoundMgr::Play(CFModSound *soundObj){

	m_pSystem->playSound(FMOD_CHANNEL_REUSE, soundObj->m_pSound, true, &soundObj->m_pChannel);
	soundObj->m_pChannel->setVolume(soundObj->GetVolume());
	soundObj->m_pChannel->setLoopCount(soundObj->GetLoopCount());
	soundObj->m_pChannel->setPaused(false);
	
/*
	//DEBUG
	int nmb;
	char sbuf[5];
	char sname[100];
	m_pSystem->getChannelsPlaying(&nmb);
	soundObj->m_pSound->getName(sname,100);
	itoa(nmb,sbuf,10);
	LOG(TL_DEBUG,sname);
	LOG(TL_DEBUG,sbuf);
	//END DEBUG
*/
}

void CFModSoundMgr::Destroy(){
	m_pSystem->release();
	m_pSystem = NULL;
}
/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


