/****************************************************************************\
*                                                                            *
* 24.02.2006                                                                 *
* CClock - Header File                                                       *
*                                                                            *
* (c) Raphael Charwot                                                        *
* Parts (c) Noel Llopis                                                      *
*                                                                            *
\****************************************************************************/

#ifndef __CCLOCK_H__
#define __CCLOCK_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <deque>
#include <vector>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>
#include "ITimeSource.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IObserver;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CClock : public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CClock(const TSmartPointer<ITimeSource> &pTimeSource);

        void SetTimeSource(const TSmartPointer<ITimeSource> &pTimeSource);

        void FrameStep();

        inline double GetTime() const {return m_dCurrentTime;}
        inline double GetFrameDuration() const {return m_dFrameTime;}
        inline float GetFrameRate() const {return 1.0f/(float)m_dFrameTime;}

        void SetFiltering (byte bFrameWindow, double dFrameDefault = 0.030, double dFrameLimit = 0.2);

        void AddObserver(IObserver *pObserver);
        void RemoveObserver(IObserver *pObserver);

        virtual ~CClock();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        double GetExactLastFrameDuration();
        void AddToFrameHistory(double dExactFrameDuration);
        double GetPredictedFrameDuration() const;

        //Private copy constructor and assignment operator to prevent mistakes:
        CClock(const CClock &);
        CClock &operator =(const CClock &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<ITimeSource> m_pTimeSource;
        double m_dCurrentTime;
        double m_dFrameTime;
        double m_dFrameLimit;

        double m_dSourceLastValue;

        byte   m_bFrameFilteringWindow;
        double m_dFrameDefaultTime;
        std::deque<double> m_FrameDurationHistory;

        typedef std::vector<IObserver *> ObserverList;
        ObserverList m_Observers;
};

#endif
