/****************************************************************************\
*                                                                            *
* 24.02.2006                                                                 *
* CTimer - Header File                                                       *
*                                                                            *
* (c) Raphael Charwot                                                        *
*                                                                            *
\****************************************************************************/

#ifndef __CTIMER_H__
#define __CTIMER_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Misc/IObserver.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CClock;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTimer : private IObserver, public IManagedObject
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTimer(CClock * const pClock);
        
        inline double GetTime() const {return m_dCurrentTime;}
        double GetFrameDuration() const;
        float GetFrameRate() const;

        void Enable(bool bEnable) {m_bEnabled = bEnable;}
        void SetScale(float fScale) {m_fScale = fScale;}
        void Reset();

        inline bool IsPaused () const {return !m_bEnabled;}
        inline float GetScale () const {return m_fScale;}

        virtual ~CTimer();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Notify();
        friend class CClock;

        //Private copy constructor and assignment operator to prevent mistakes:
        CTimer(const CTimer &);
        CTimer &operator =(const CTimer &);
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CClock *m_pClock;

        double  m_dCurrentTime;
        double  m_dFrameTime;

        bool    m_bEnabled;
        float   m_fScale;
};

#endif
