/****************************************************************************\
*                                                                            *
* 15.03.2006                                                                 *
* CSettings - Source File                                                    *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CSettings.h"
#include "FileSystem/CIniFile.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

using namespace std;

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
CSettings::CSettings(){
	m_settingsFileName = "";
	p_iniFile          = new CIniFile();
}

CSettings::~CSettings(){
	SAFE_DELETE(p_iniFile);
}

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CSettings::Load(char * filename){
	
	m_settingsFileName = filename;
	p_iniFile->LoadFile(filename);
		
	switch (p_iniFile->m_status){
		case 0:
			return true;
		case 1:
			LOG(TL_ERROR,"Error: Cannot open the \"settings.ini\" file !");
			return false;
		case 2:
			LOG(TL_ERROR,"Error: Cannot parse the \"settings.ini\" file !");
			return false;
		default:
			return false;
		}
}

bool CSettings::GetValue(const string &sCategory, const string &sName, string &sValue){
	if(p_iniFile->m_status == 0){
		if(p_iniFile->SetCategory(sCategory)==1){
			sValue = p_iniFile->GetEntryValueByName(sName);
			if(sValue.size()){
				return true;
			}
		}
	}
	return false;
}

bool CSettings::SetValue(const string &sCategory, const string &sName, const string &sValue){
	if(p_iniFile->m_status == 0){
		if(p_iniFile->SetCategory(sCategory)==1){
			if(p_iniFile->SetEntry(sName)==1){
				p_iniFile->SetValue(sValue);
			}else{
				p_iniFile->AddEntry(sName,sValue);
			}
		}else{
			p_iniFile->AddCategory(sCategory);
			p_iniFile->AddEntry(sName,sValue);
		}
		return true;
	}
	return false;
}

void CSettings::Clear(){
	p_iniFile->ClearAll();
}

bool CSettings::SaveAs(char * filename){
	if(p_iniFile->m_status == 0){
		p_iniFile->SaveToFile(filename);
		return true;
	}else{
		return false;
	}
}