/****************************************************************************\
*                                                                            *
* 30.04.2006                                                                 *
* CAiKamikazeCtrl - Source File                                             *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CAiKamikazeCtrl.h"
#include "CEngineFactory.h"
#include "CBirdObj.h"
#include "CGameCtrl.h"
#include "Objects/Buildings/CBuildingObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fAttackDistanceSqr = 25.0f;
const float cg_fTurnDistance = 25.0f;
const float cg_fTurnHeight = 15.0f;
const float cg_fTurnPointDistanceSqr = 2.0f;
const float cg_fAttackHeight = 2.0f;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CAiKamikazeCtrl::CAiKamikazeCtrl(CGameCtrl *pGameCtrl) : m_pGameCtrl(pGameCtrl), m_pNextTarget(NULL),
    m_vRot(0.0f, 0.0f, 0.0f), m_vAttackPoint(0.0f, 0.0f, 0.0f)
{
}

void CAiKamikazeCtrl::Update(float dt)
{
	if(m_pNextTarget == NULL)
	{
		m_pNextTarget = m_pGameCtrl->GetRandomBuilding();
		if(!m_pNextTarget)
		{
			LOG(TL_DEBUG, "Warning: CAiKamikazeCtrl: No building found");
            m_vRot = m_pBird->GetRot();
            m_vRot.m_Y += MY_PI/2.5f;
			return;
		}
		m_pNextTarget->AddDeleteObserver(this);
	}
	CVector3f vDir;
	CVector3f vIn;

    m_vAttackPoint = GetAttackPoint();
	m_vRot.LookAlongRot(m_vAttackPoint - m_pBird->GetPos());
}

void CAiKamikazeCtrl::GetDir(CVector3f &vDir)
{
    vDir = m_pBird->GetIn();
}

bool CAiKamikazeCtrl::GetRot(CVector3f &vRot)
{
	vRot.ShortestRot(m_pBird->GetRot(), m_vRot);

    return true;
}

bool CAiKamikazeCtrl::Shoot()
{
	return false;
}

CAiKamikazeCtrl::~CAiKamikazeCtrl()
{
	if(m_pNextTarget)
	{
		m_pNextTarget->RemoveDeleteObserver(this);
		m_pNextTarget = NULL;
	}
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
const CVector3f CAiKamikazeCtrl::GetAttackPoint()
{
    CSpheref sphere = m_pNextTarget->GetBoundingSphere();
    return sphere.m_vCenter;
}

void CAiKamikazeCtrl::Notify()
{
	//Ok, our target has been destroyed
	m_pNextTarget = NULL;
}
