/****************************************************************************\
*                                                                            *
* 24.04.2006                                                                 *
* CBirdObj - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBIRDOBJ_H__
#define __CBIRDOBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include "Objects/ICharacterObj.h"
#include "IBirdCtrl.h"
#include <Misc/Functors.h>
#include <Sound/ISound.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/

enum EBirdState
{
	EBS_LIVING,
	EBS_FALLING,
	EBS_DEAD,
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBirdObj : public ICharacterObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CBirdObj(CGameCtrl *pGameCtrl, const TSmartPointer<IFunctor> &pBirdDestroyedCb);

		virtual void CollidedWith(IGameObj *pOther);
		virtual void LevelDone();

        virtual void SetHitpoints(float fHitpoints);
        inline float GetHitpoints() const {return m_fHitpoints;}

        void SetBirdCtrl(const TSmartPointer<IBirdCtrl> &pBirdCtrl);
		inline EBirdState GetState() const {return m_State;}

        virtual void Destroy();

        virtual ~CBirdObj() {Destroy();}

    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		void SetState(EBirdState state);

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void SubUpdate(float dt);
        //Private copy constructor and assignment operator to prevent mistakes:
        CBirdObj(const CBirdObj &);
        const CBirdObj &operator =(const CBirdObj &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<IBirdCtrl> m_pBirdCtrl;
		EBirdState m_State;
		float m_fStateTime;
		float m_fAttackDelay;

        float m_fHitpoints;

        TSmartPointer<IFunctor> m_pBirdDestroyedCb;
		TSmartPointer<ISound> m_pDieSound;
};

#endif
