/****************************************************************************\
*                                                                            *
* 18.04.2006                                                                 *
* CBirdRep - Header File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBIRDREP_H__
#define __CBIRDREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class ISceneGraph;
class CBirdObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBirdRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CBirdRep();

        bool Create(ISceneGraph *pSceneGraph, CBirdObj *pBirdObj);

        virtual const CVector3f &GetPos() const;
        virtual const CVector3f &GetRot() const;
        virtual const CVector3f &GetScale() const;
        virtual const CMatrix44 &GetTransformation();
        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual IGameObj * const GetGameObj() const;

        void Show(bool bShow);

        virtual ~CBirdRep();

    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:

    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CBirdRep(const CBirdRep &);
        const CBirdRep &operator =(const CBirdRep &);

    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:

    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CBirdObj *m_pBirdObj;
        TSmartPointer<IModelGeometry> m_pModelGeometry;
};

#endif
