/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CBuildingObj - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBUILDINGOBJ_H__
#define __CBUILDINGOBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "Objects/ICollidableObj.h"
#include "CGameCtrl.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
enum EBuildingState
{
	EBS_VALIDPOS,
	EBS_INVALIDPOS,
	EBS_DESTROYED,
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBuildingObj : public ICollidableObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CBuildingObj(CGameCtrl *pGameCtrl, const std::string &sModel, float fMaxHitpoints,
			float fMaxValue, ECollisionGroup cg);

		virtual void SetHitpoints(float fHitpoints);
		inline float GetHitpoints() const {return m_fHitpoints;}

		virtual void CollidedWith(IGameObj *pOther);
		virtual void LevelDone();

        bool Build();
		void UpdateState();

		inline const EBuildingState GetState() const {return m_State;}

        virtual ~CBuildingObj();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		virtual void SubUpdate(float dt);
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
		void AdjustHeight(const CVector3f &vCorner, const CVector3f &vCenter, float &fMinHeight, float &fMaxHeight);

        //Private copy constructor and assignment operator to prevent mistakes:
        CBuildingObj(const CBuildingObj &);
        const CBuildingObj &operator =(const CBuildingObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		EBuildingState m_State;
		ECollisionGroup m_CollisionGroup;

		float m_fMaxHitpoints;
		float m_fHitpoints;
		float m_fMaxValue;

		bool m_bReadyForDestroy;
};

#endif
