/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CBuildingRep - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBuildingRep.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/IRenderer.h>
#include <CEngineFactory.h>
#include "CBuildingObj.h"
#include <Renderer/COGLGlobalRenderSettings.h>


/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBuildingRep::CBuildingRep() : m_pBuildingObj(NULL), m_pModelGeometry(NULL)
{

}

bool CBuildingRep::Create(ISceneGraph *pSceneGraph, const std::string &sModel, CBuildingObj *pBuildingObj)
{
    m_pBuildingObj = pBuildingObj;
	m_pGlobalRenderSettings = CEngineFactory::InstancePtr()->GetRenderer()->GetGlobalSettings();

    m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(sModel);
	m_pModelGeometryDestroyed = CEngineFactory::InstancePtr()->GetModelFactory()->getModel("Data/models/buildings/destroyed.fbx");

	float fMinScale = 1.0f;
	float fScale;
	CAABox3f origBox(m_pModelGeometry->GetAABoundingBox());
	CAABox3f destroyedBox(m_pModelGeometryDestroyed->GetAABoundingBox());
	if(origBox.SizeX() < destroyedBox.SizeX())
	{
		fScale = origBox.SizeX() / destroyedBox.SizeX();
		if(fScale < fMinScale)
			fMinScale = fScale;
	}
	if(origBox.SizeY() < destroyedBox.SizeY())
	{
		fScale = origBox.SizeY() / destroyedBox.SizeY();
		if(fScale < fMinScale)
			fMinScale = fScale;
	}
	if(origBox.SizeZ() < destroyedBox.SizeZ())
	{
		fScale = origBox.SizeZ() / destroyedBox.SizeZ();
		if(fScale < fMinScale)
			fMinScale = fScale;
	}
	m_DestroyedScale.Scaling(CVector3f(fMinScale, fMinScale, fMinScale));

    if(!m_pModelGeometry)
    {
        LOG(TL_ERROR, "Error: getModel failed in CBuildingRep::Create");
        return false;
    }

    IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

    pSceneGraph->AddObject(this);

    return true;
}

const CVector3f &CBuildingRep::GetPos() const
{
    return m_pBuildingObj->GetPos();
}

const CVector3f &CBuildingRep::GetRot() const
{
    return m_pBuildingObj->GetRot();
}

const CVector3f &CBuildingRep::GetScale() const
{
    return m_pBuildingObj->GetScale();
}

const CMatrix44 &CBuildingRep::GetTransformation()
{
    return m_pBuildingObj->GetTransformation();
}

IGameObj * const CBuildingRep::GetGameObj() const
{
    return m_pBuildingObj;
}

bool CBuildingRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

CBuildingRep::~CBuildingRep()
{
    m_pBuildingObj = NULL;
	m_pModelGeometry = NULL;
	m_pModelGeometryDestroyed = NULL;
	m_pGlobalRenderSettings = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CBuildingRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
	switch(m_pBuildingObj->GetState()){

		case EBS_VALIDPOS:
			m_pModelGeometry->Render();
			return;

		case EBS_DESTROYED:
			CEngineFactory::InstancePtr()->GetRenderer()->SaveMatrix();
			CEngineFactory::InstancePtr()->GetRenderer()->MultMatrix(m_DestroyedScale);
			m_pModelGeometryDestroyed->Render();
			CEngineFactory::InstancePtr()->GetRenderer()->RestoreMatrix();
			return;

		default:
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->StoreGlobalRenderState();
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->DisableAllRenderFeatures();
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetDrawColor(CVector4f(1.0f,0.0f,0.0f,1.0f));
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetStipplePolygon(true);
			m_pModelGeometry->Render();
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetStipplePolygon(false);
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->SetDrawColor(CVector4f(1.0f,1.0f,1.0f,1.0f));
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->EnableAllRenderFeatures();
			((COGLGlobalRenderSettings*)(IGlobalRenderSettings*)m_pGlobalRenderSettings)->RestoreGlobalRenderState();
	}
	
}