/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CBuildingRep - Header File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBUILDINGREP_H__
#define __CBUILDINGREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>
#include <Renderer/IGlobalRenderSettings.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class ISceneGraph;
class CBuildingObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBuildingRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CBuildingRep();
        
        bool Create(ISceneGraph *pSceneGraph, const std::string &sModel, CBuildingObj *pBuildingObj);

        virtual const CVector3f &GetPos() const;
        virtual const CVector3f &GetRot() const;
        virtual const CVector3f &GetScale() const;
        virtual const CMatrix44 &GetTransformation();
        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual IGameObj * const GetGameObj() const;

        virtual ~CBuildingRep();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CBuildingRep(const CBuildingRep &);
        const CBuildingRep &operator =(const CBuildingRep &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CBuildingObj *m_pBuildingObj;
        TSmartPointer<IModelGeometry> m_pModelGeometry;
		TSmartPointer<IModelGeometry> m_pModelGeometryDestroyed;
		TSmartPointer<IGlobalRenderSettings> m_pGlobalRenderSettings;

		CMatrix44 m_DestroyedScale;
};

#endif
