/****************************************************************************\
*                                                                            *
* 03.06.2006                                                                 *
* CTowerObj - Source File                                          *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTowerObj.h"
#include "CGameCtrl.h"
#include "Objects/CProjectileObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
const float cg_fAttackDelay = 2.0f;
const float cg_fTowerProjSpeed = 60.0f;
const CVector3f cg_vTowerShootOffset = CVector3f(0.0f, 5.0f, 0.0f);

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CTowerObj::CTowerObj(CGameCtrl *pGameCtrl)
	: CBuildingObj(pGameCtrl, "Data/models/buildings/tower.fbx", 150.0f, 0.0f, ECG_TOWER), m_pTowerCtrl(NULL), m_fAttackDelay(0.0f)
{
}

void CTowerObj::SetTowerCtrl(const TSmartPointer<ITowerCtrl> &pTowerCtrl)
{
	if(m_pTowerCtrl)
	{
		m_pTowerCtrl->SetTower(NULL);
	}
	m_pTowerCtrl = pTowerCtrl;
	if(m_pTowerCtrl)
	{
		m_pTowerCtrl->SetTower(this);
	}
}

const CVector3f CTowerObj::GetShootingPos() const
{
    return m_TrObj.GetPos() + cg_vTowerShootOffset;
}

CTowerObj::~CTowerObj()
{

}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void CTowerObj::SubUpdate(float dt)
{	
	CBuildingObj::SubUpdate(dt);
	if(GetState() == EBS_VALIDPOS)
	{
		if(m_pTowerCtrl)
		{
			m_pTowerCtrl->Update(dt);
			CVector3f rot;
			m_pTowerCtrl->GetRot(rot);
			m_fAttackDelay -= dt;
			if(m_fAttackDelay <= 0.0f)
			{
				if(m_pTowerCtrl->Shoot())
				{
					m_fAttackDelay = cg_fAttackDelay;
					CProjectileObj *pProjectile = new CProjectileObj(m_pGameCtrl, EOT_ARROW, ECG_TOWER_PROJECTILE, "Data/models/pfeil.fbx");
					pProjectile->SetPos(GetShootingPos());
					pProjectile->SetRot(m_TrObj.GetRot() + rot);
					pProjectile->AddVelocity(pProjectile->GetIn() * cg_fTowerProjSpeed);
					m_pGameCtrl->AddObject(pProjectile);
				}
			}
		}
	}
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/

