/****************************************************************************\
*                                                                            *
* 01.05.2006                                                                 *
* CDummyRep - Header File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CDUMMYREP_H__
#define __CDUMMYREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>
#include <Memory/mmgr.h>
#include <Display/IRepresentation.h>
#include <Memory/TSmartPointer.h>
#include <Renderer/IModelGeometry.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class TQTNode;
/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CDummyRep : public IRepresentation
{
	/**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:

	CDummyRep(const std::string & modelName, const CVector3f &vPos);

	virtual const CVector3f &GetPos() const;
	virtual const CVector3f &GetRot() const;
	virtual const CVector3f &GetScale() const;
	virtual const CMatrix44 &GetTransformation();

    virtual IGameObj * const GetGameObj() const;
    virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

	virtual void Destroy();

	virtual ~CDummyRep() {Destroy();}

	/**************************************************************************\
	* PROTECTED CLASS FUNCTIONS                                                *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	virtual void RenderSub(const CFrustum &frustum, bool bVerify);

    //Private copy constructor and assignment operator to prevent mistakes:
	CDummyRep(const CDummyRep &);
	CDummyRep &operator =(const CDummyRep &);

	/**************************************************************************\
	* PROTECTED CLASS MEMBERS                                                  *
	\**************************************************************************/
protected:
	
	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:

	CVector3f m_pos;
	CVector3f m_rot;
	CVector3f m_scale;
	CMatrix44 m_transform;
	CSpheref  m_Sphere;
	CAABox3f  m_Box;

	TSmartPointer<IModelGeometry> m_pModelGeometry;
};

#endif
