/****************************************************************************\
*                                                                            *
* 30.04.2006                                                                 *
* CHumanPlayerCtrl - Source File                                             *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CHumanPlayerCtrl.h"
#include "CEngineFactory.h"
#include "CPlayerObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CHumanPlayerCtrl::CHumanPlayerCtrl(word wInputPriority)
{
    SetPriority(wInputPriority);
}

void CHumanPlayerCtrl::SetPlayer(CPlayerObj *pPlayer)
{
    if(!m_pPlayer && pPlayer)
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->RegisterHandler(this);
    }
    else if(m_pPlayer && !pPlayer)
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->UnregisterHandler(this);
    }
    IPlayerCtrl::SetPlayer(pPlayer);
    m_bForward = false;
    m_bBackward = false;
    m_bLeft = false;
    m_bRight = false;
    m_bJump = false;
	m_bShoot = false;
	m_bRun = false;
	m_bRotChanged = false;
    m_vRot.Set(0.0f, 0.0f, 0.0f);
}

void CHumanPlayerCtrl::GetDirection(CVector3f &vDirection)
{
    vDirection.Set(0.0f, 0.0f, 0.0f);
    if(m_bForward)
    {
        vDirection += m_pPlayer->GetIn();
    }
    if(m_bBackward)
    {
        vDirection -= m_pPlayer->GetIn();
    }

    if(m_bRight)
    {
        vDirection += m_pPlayer->GetRight();
    }
    if(m_bLeft)
    {
        vDirection -= m_pPlayer->GetRight();
    }
}

bool CHumanPlayerCtrl::GetRot(CVector3f &vRot)
{
    if(m_bRotChanged)
    {
        vRot = m_vRot;
        m_vRot.Set(0.0f, 0.0f, 0.0f);
        m_bRotChanged = false;
        return true;
    }

    return false;
}

bool CHumanPlayerCtrl::Shoot()
{
    if(m_bShoot)
	{
		m_bShoot = false;
		return true;
	}

    return false;
}

bool CHumanPlayerCtrl::Jump()
{
    return m_bJump;
}

bool CHumanPlayerCtrl::Run()
{
	return m_bRun;
}

CHumanPlayerCtrl::~CHumanPlayerCtrl()
{
    if(m_pPlayer)
    {
        CEngineFactory::InstancePtr()->GetInputMgr()->UnregisterHandler(this);
    }
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
bool CHumanPlayerCtrl::OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ)
{
    if(sdwX || sdwY)
    {
        m_vRot.m_X += ((float) sdwY) / 150.0f;
        m_vRot.m_Y += ((float) sdwX) / 150.0f;
        m_bRotChanged = true;

        return true;
    }
    return false;
}

bool CHumanPlayerCtrl::OnMouseKeyDown(byte bKey)
{
	if(bKey == 0)
	{
		m_bShoot = true;
		return true;
	}
	return false;
}

bool CHumanPlayerCtrl::OnKeyDown(dword dwKey)
{
    switch(dwKey)
    {
        case cg_dwKeyW:
            m_bForward = true;
            return true;
        case cg_dwKeyS:
            m_bBackward = true;
            return true;
        case cg_dwKeyA:
            m_bLeft = true;
            return true;
        case cg_dwKeyD:
            m_bRight = true;
            return true;
        case cg_dwKeySpace:
            m_bJump = true;
            return true;
		case cg_dwKeyLShift:
			m_bRun = true;
			return true;
    }

    return false;
}

bool CHumanPlayerCtrl::OnKeyUp(dword dwKey)
{
    switch(dwKey)
    {
        case cg_dwKeyW:
            m_bForward = false;
            return true;
        case cg_dwKeyS:
            m_bBackward = false;
            return true;
        case cg_dwKeyA:
            m_bLeft = false;
            return true;
        case cg_dwKeyD:
            m_bRight = false;
            return true;
        case cg_dwKeySpace:
            m_bJump = false;
            return true;
		case cg_dwKeyLShift:
			m_bRun = false;
			return true;
    }

    return false;

}