/****************************************************************************\
*                                                                            *
* 24.04.2006                                                                 *
* CPlayerObj - Header File                                                   *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CPLAYEROBJ_H__
#define __CPLAYEROBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "ICharacterObj.h"
#include <Sound/ISound.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IPlayerCtrl;

enum EPlayerState
{
    EPS_NORMAL = 0,
    EPS_MOVING = 1,
    EPS_JUMPING = 2,    //Flag!
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CPlayerObj : public ICharacterObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CPlayerObj(CGameCtrl *pGameCtrl);

        virtual void SetPos(const CVector3f &vPos);

        //Redefinitions, because X-Z-Rotation shouldn't affect the player's movement directions
        virtual void SetRot(const CVector3f &vRot);
        virtual void LookAt(const CVector3f &vPoint, const CVector3f &vUp);
        virtual void RotateX(float angle);
        virtual void RotateY(float angle);
        virtual void RotateZ(float angle);
        virtual const CVector3f &GetIn();
        virtual const CVector3f &GetUp();
        virtual const CVector3f &GetRight();

		virtual void CollidedWith(IGameObj *pOther);
		virtual const CSpheref GetBoundingSphere();

        void SetPlayerCtrl(IPlayerCtrl *pPlayerCtrl);

        inline word GetState() const {return m_wState;}
        virtual ~CPlayerObj();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        void SetState(word wState);
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void SubUpdate(float dt);

        void UpdateVectors();
        //Private copy constructor and assignment operator to prevent mistakes:
        CPlayerObj(const CPlayerObj &);
        const CPlayerObj &operator =(const CPlayerObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        IPlayerCtrl *m_pPlayerCtrl;
        word m_wState;

        CVector3f m_vIn;
        CVector3f m_vRight;
        CVector3f m_vUp;
        bool m_bVectorsValid;

		TSmartPointer<ISound> m_pShootSound;
		TSmartPointer<ISound> m_pJumpSound;
};

#endif
