/****************************************************************************\
*                                                                            *
* 22.05.2006                                                                 *
* CProjectileRep - Header File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CPROJECTILEREP_H__
#define __CPROJECTILEREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class ISceneGraph;
class CProjectileObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CProjectileRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CProjectileRep();
        
		bool Create(ISceneGraph *pSceneGraph, const std::string &sModel, CProjectileObj *pProjectileObj);

		virtual const CVector3f &GetPos() const;
		virtual const CVector3f &GetRot() const;
		virtual const CVector3f &GetScale() const;
		virtual const CMatrix44 &GetTransformation();
		virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

		virtual IGameObj * const GetGameObj() const;
        
        virtual ~CProjectileRep();
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
		virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CProjectileRep(const CProjectileRep &);
        const CProjectileRep &operator =(const CProjectileRep &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
		CProjectileObj *m_pProjectileObj;
		TSmartPointer<IModelGeometry> m_pModelGeometry;
};

#endif
