/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CStartRep - Source File                                                 *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CStartRep.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include <Renderer/IRenderer.h>
#include <CEngineFactory.h>
#include "CStartObj.h"
#include <Renderer/COGLGlobalRenderSettings.h>


/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CStartRep::CStartRep() : m_pStartObj(NULL), m_pModelGeometry(NULL)
{

}

bool CStartRep::Create(ISceneGraph *pSceneGraph, const std::string &sModel, CStartObj *pStartObj)
{
    m_pStartObj = pStartObj;

    m_pModelGeometry = CEngineFactory::InstancePtr()->GetModelFactory()->getModel(sModel);
    if(!m_pModelGeometry)
    {
        LOG(TL_ERROR, "Error: getModel failed in CStartRep::Create");
        return false;
    }

    IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

    pSceneGraph->AddObject(this);

    return true;
}

const CVector3f &CStartRep::GetPos() const
{
    return m_pStartObj->GetPos();
}

const CVector3f &CStartRep::GetRot() const
{
    return m_pStartObj->GetRot();
}

const CVector3f &CStartRep::GetScale() const
{
    return m_pStartObj->GetScale();
}

const CMatrix44 &CStartRep::GetTransformation()
{
    return m_pStartObj->GetTransformation();
}

IGameObj * const CStartRep::GetGameObj() const
{
    return m_pStartObj;
}

bool CStartRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

CStartRep::~CStartRep()
{
    m_pStartObj = NULL;
	m_pModelGeometry = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CStartRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
	m_pModelGeometry->Render();
}