/****************************************************************************\
*                                                                            *
* 13.05.2006                                                                 *
* ICharacterObj - Header File                                                *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __ICHARACTEROBJ_H__
#define __ICHARACTEROBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "IGameObj.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CPhysXCharacter;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class ICharacterObj : public IGameObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        ICharacterObj(CGameCtrl *pGameCtrl, EObjectType type);

        virtual void Update(float dt);

        //Movement
        virtual void SetPos(const CVector3f &vPos);
        virtual void Move(const CVector3f &vDistance, float dt);

		inline void SetCollisionGroups(dword dwCollisionGroups) {m_dwCollisionGroups = dwCollisionGroups;}
        void UseGravity(bool bUseGravity);
		void SetGravityFactor(float fGravityFactor);
        inline bool IsInAir() const {return m_bInAir;}
        inline float GetAirTime() const {return m_fAirTime;}

        virtual void Destroy();
        
        virtual ~ICharacterObj() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        void SetCharacter(CPhysXCharacter *pCharacter);
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        ICharacterObj(const ICharacterObj &);
        const ICharacterObj &operator =(const ICharacterObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CPhysXCharacter *m_pPhysXChar;
		dword m_dwCollisionGroups;
        bool m_bUseGravity;
		float m_fGravityFactor;
        bool m_bInAir;
        float m_fAirTime;
		bool m_bMoved;
};

#endif
