/****************************************************************************\
*                                                                            *
* 13.05.2006                                                                 *
* ICollidableObj - Source File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "ICollidableObj.h"
#include <Collision/CPhysXActor.h>
#include <CEngineFactory.h>

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
ICollidableObj::ICollidableObj(CGameCtrl *pGameCtrl, EObjectType type) : IGameObj(pGameCtrl, type), m_pActor(NULL)
{

}

void ICollidableObj::Update(float dt)
{
	if(m_pActor)
	{
		IGameObj::SetRot(m_pActor->GetRot());
		IGameObj::SetPos(m_pActor->GetPos());
	}
    IGameObj::Update(dt);
}

void ICollidableObj::SetPos(const CVector3f &vPos)
{
	if(m_pActor)
	{
		m_pActor->SetPos(vPos);
	}
	IGameObj::SetPos(vPos);
}

void ICollidableObj::SetRot(const CVector3f &vRot)
{
	if(m_pActor)
	{
		m_pActor->SetRot(vRot);
	}
	IGameObj::SetRot(vRot);
}

void ICollidableObj::SetScale(const CVector3f &vScale)
{
	assert(!m_pActor && "Error: m_pActor not NULL in ICollidableObj::SetScale! This is not yet supported.");

	IGameObj::SetScale(vScale);
}

void ICollidableObj::LookAt(const CVector3f &vPoint)
{
	IGameObj::LookAt(vPoint);
	SetPos(GetPos());
	SetRot(GetRot());
}

void ICollidableObj::LookAt(const CVector3f &vPoint, const CVector3f &vUp)
{
    LOG(TL_ERROR, "Error: ICollidableObj::LookAt not implemented");
}

void ICollidableObj::AddVelocity(const CVector3f &vForce)
{
	if(m_pActor)
	{
		m_pActor->AddForce(vForce, EFM_VELOCITY);
	}
}

void ICollidableObj::Destroy()
{
    if(m_pActor)
    {
		if(CEngineFactory::InstancePtr())
		{
			CEngineFactory::InstancePtr()->GetPhysicsMgr()->DeleteActor(m_pActor);
		}
        m_pActor = NULL;
    }
    IGameObj::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/
void ICollidableObj::SetActor(CPhysXActor *pActor)
{
	if(m_pActor != pActor)
	{
		if(m_pActor)
		{
			CEngineFactory::InstancePtr()->GetPhysicsMgr()->DeleteActor(m_pActor);
		}
		m_pActor = pActor;
		if(m_pActor)
		{
			m_pActor->SetObject(this);
		}
	}
}

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


