/****************************************************************************\
*                                                                            *
* 13.05.2006                                                                 *
* ICollidableObj - Header File                                               *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __ICOLLIDABLEOBJ_H__
#define __ICOLLIDABLEOBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include "IGameObj.h"

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CPhysXActor;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class ICollidableObj : public IGameObj
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        ICollidableObj(CGameCtrl *pGameCtrl, EObjectType type);
        
        void Update(float dt);

        virtual void SetPos(const CVector3f &vPos);
        virtual void SetRot(const CVector3f &vRot);
		virtual void SetScale(const CVector3f &vScale);
		virtual void LookAt(const CVector3f &vPoint);
        virtual void LookAt(const CVector3f &vPoint, const CVector3f &vUp);

		void AddVelocity(const CVector3f &vForce);
        
		virtual void Destroy();

		virtual ~ICollidableObj() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        void SetActor(CPhysXActor *pActor);
		inline CPhysXActor *GetActor() const {return m_pActor;}
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        //Private copy constructor and assignment operator to prevent mistakes:
        ICollidableObj(const ICollidableObj &);
        const ICollidableObj &operator =(const ICollidableObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CPhysXActor *m_pActor;
};

#endif
