/****************************************************************************\
*                                                                            *
* 26.04.2006                                                                 *
* IGameObj - Source File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <algorithm>
#include "IGameObj.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
IGameObj::IGameObj(CGameCtrl *pGameCtrl, EObjectType type) : m_pGameCtrl(pGameCtrl), m_bTransformationChanged(true), m_Type(type), m_bDestroyed(false)
{
}

void IGameObj::Update(float dt)
{
    SubUpdate(dt);
    if(m_bTransformationChanged && (bool) m_pRepresentation)
    {
        m_pRepresentation->ObjTransformationChanged();
        m_bTransformationChanged = false;
    }
}

void IGameObj::SetPos(const CVector3f &vPos)
{
    m_TrObj.SetPos(vPos);
    m_bTransformationChanged = true;
}

void IGameObj::SetRot(const CVector3f &vRot)
{
    m_TrObj.SetRot(vRot);
    m_bTransformationChanged = true;
}

void IGameObj::SetScale(const CVector3f &vScale)
{
	m_TrObj.SetScale(vScale);
	m_bTransformationChanged = true;
}

void IGameObj::LookAt(const CVector3f &vPoint)
{
	m_TrObj.LookAt(vPoint);
	m_bTransformationChanged = true;
}

void IGameObj::LookAt(const CVector3f &vPoint, const CVector3f &vUp)
{
    m_TrObj.LookAt(vPoint, vUp);
    m_bTransformationChanged = true;
}

void IGameObj::Move(const CVector3f &vDistance) 
{
    m_TrObj.Move(vDistance);
    m_bTransformationChanged = true;
}

void IGameObj::RotateX(float angle)
{
    m_TrObj.RotateX(angle);
    m_bTransformationChanged = true;
}

void IGameObj::RotateY(float angle)
{
    m_TrObj.RotateY(angle);
    m_bTransformationChanged = true;
}

void IGameObj::RotateZ(float angle)
{
    m_TrObj.RotateZ(angle);
    m_bTransformationChanged = true;
}

const CSpheref IGameObj::GetBoundingSphere()
{
    if(m_pRepresentation)
    {
        return m_pRepresentation->GetBoundingSphere();
    }
    LOG(TL_ERROR, "Error: m_pRepresentation is NULL in IGameObj::GetBoundingSphere()");
    return CSpheref();
}

const CBox3f IGameObj::GetBoundingBox()
{
	if(m_pRepresentation)
	{
		return m_pRepresentation->GetBoundingBox();
	}
	LOG(TL_ERROR, "Error: m_pRepresentation is NULL in IGameObj::GetBoundingBox()");
	return CBox3f();
}

bool IGameObj::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
	if(m_pRepresentation)
	{
		return m_pRepresentation->RayIntersection(vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
	}
	LOG(TL_ERROR, "Warning: RayIntersection called for object with NULL m_pRepresentation");
	return false;
}

void IGameObj::AddDeleteObserver(IObserver *pObserver)
{
	if (pObserver != NULL)
		m_Observers.push_back(pObserver);
}

void IGameObj::RemoveDeleteObserver(IObserver *pObserver)
{
	m_Observers.erase(std::remove(m_Observers.begin(),m_Observers.end(), pObserver), m_Observers.end());
}

void IGameObj::NotifyAllDeleteObservers()
{
	ObserverList::iterator it;
	for (it = m_Observers.begin(); it != m_Observers.end(); ++it)
	{
		(*it)->Notify();
	}
	m_Observers.clear();
}

void IGameObj::Destroy()
{
	NotifyAllDeleteObservers();
	m_bDestroyed = true;

    if(m_pRepresentation)
    {
        m_pRepresentation->Destroy();
        m_pRepresentation = NULL;
    }

    m_pGameCtrl = NULL;
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/


