/****************************************************************************\
*                                                                            *
* 21.04.2006                                                                 *
* IGameObj - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __IGAMEOBJ_H__
#define __IGAMEOBJ_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/IManagedObject.h>
#include <Memory/TSmartPointer.h>
#include <Display/CTransformableObj.h>
#include <Display/IRepresentation.h>
#include <Misc/IObserver.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CGameCtrl;
const float cg_fMaxSoundDistance = 2500.0f;

enum EObjectType
{
	EOT_UNKNOWN,
	EOT_TERRAIN,
	EOT_PLAYER,
	EOT_BUILDING,
	EOT_BIRD,
	EOT_ARROW,
	EOT_BIRDPROJECTILE,
	EOT_VEGETATION,
	EOT_STARTPOS,
};

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class IGameObj : public IManagedObject, public IObserver
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        IGameObj(CGameCtrl *pGameCtrl, EObjectType type);

        virtual void Update(float dt);
        
        //Movement
        virtual void SetPos(const CVector3f &vPos);
        virtual void SetRot(const CVector3f &vRot);
		virtual void SetScale(const CVector3f &vScale);
		virtual void LookAt(const CVector3f &vPoint);
        virtual void LookAt(const CVector3f &vPoint, const CVector3f &vUp);
        virtual void Move(const CVector3f &vDistance); //ToDo: Entfernen!!!
         //Rotations (use Move_ before Rotate_ to prevent multiple calculations of matrices)
        virtual void RotateX(float angle);
        virtual void RotateY(float angle);
        virtual void RotateZ(float angle);
        //State request:
        inline const CVector3f &GetPos() const {return m_TrObj.GetPos();}
        inline const CVector3f &GetRot() const {return m_TrObj.GetRot();}
        inline const CVector3f &GetScale() const {return m_TrObj.GetScale();}
        virtual const CVector3f &GetIn() {return m_TrObj.GetIn();}
        virtual const CVector3f &GetUp() {return m_TrObj.GetUp();}
        virtual const CVector3f &GetRight() {return m_TrObj.GetRight();}
		inline EObjectType GetType() const {return m_Type;}

		virtual void CollidedWith(IGameObj *pOther) {}
		virtual void LevelDone() {}

		inline const CMatrix44 &GetTransformation() {return m_TrObj.GetTransformation();}
        virtual const CSpheref GetBoundingSphere();
		virtual const CBox3f GetBoundingBox();
		virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

		void AddDeleteObserver(IObserver *pObserver);
		void RemoveDeleteObserver(IObserver *pObserver);
		void NotifyAllDeleteObservers();
		virtual void Notify() {}

        virtual void Destroy();
		inline bool IsDestroyed() const {return m_bDestroyed;}

		CGameCtrl * const getGameControl(){return m_pGameCtrl;}

        virtual ~IGameObj() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
		inline void SetType(EObjectType type) {m_Type = type;}
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual void SubUpdate(float dt) = 0;
        //Private copy constructor and assignment operator to prevent mistakes:
        IGameObj(const IGameObj &);
        const IGameObj &operator =(const IGameObj &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        CGameCtrl *m_pGameCtrl;
        CTransformableObj m_TrObj;
        TSmartPointer<IRepresentation> m_pRepresentation;
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        bool m_bTransformationChanged;
		EObjectType m_Type;

		typedef std::vector<IObserver *> ObserverList;
		ObserverList m_Observers;
		bool m_bDestroyed;
};

#endif
