/****************************************************************************\
*                                                                            *
* 18.04.2006                                                                 *
* CTerrainPartRep - Source File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CTerrainPartRep.h"
#include <Display/ISceneGraph.h>
#include <Renderer/IModelGeometry.h>
#include "CTerrain.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
bool CTerrainPartRep::Create(CTerrainRep *pTerrainRep, const TSmartPointer<IModelGeometry> pModelGeometry,
                             ISceneGraph *pSceneGraph, CTerrain *pTerrain)
{
    m_pTerrain = pTerrain;
    m_pTerrainRep = pTerrainRep;
    m_pModelGeometry = pModelGeometry;

    IRepresentation::Create(pSceneGraph, m_pModelGeometry->GetBoundingSphere(), m_pModelGeometry->GetAABoundingBox());

    m_pSceneGraph->AddObject(this);
    
    return true;
}

const CVector3f &CTerrainPartRep::GetPos() const
{
    return m_pTerrain->GetPos();
}

const CVector3f &CTerrainPartRep::GetRot() const
{
    return m_pTerrain->GetRot();
}

const CVector3f &CTerrainPartRep::GetScale() const
{
    return m_pTerrain->GetScale();
}

const CMatrix44 &CTerrainPartRep::GetTransformation()
{
    return m_pTerrain->GetTransformation();
}

IGameObj * const CTerrainPartRep::GetGameObj() const
{
    return m_pTerrain;
}

bool CTerrainPartRep::RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const
{
    return m_pModelGeometry->RayIntersection(GetPos(), GetRot(), vOrigin, vDir, vIntersectionPos, fIntersectionDistance);
}

void CTerrainPartRep::Destroy()
{
    Clear();
    IRepresentation::Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
void CTerrainPartRep::Clear()
{
    m_pTerrain = NULL;
    m_pTerrainRep = NULL;
    m_pModelGeometry = NULL;
}

void CTerrainPartRep::RenderSub(const CFrustum &frustum, bool bVerify)
{
    m_pModelGeometry->Render();
}

