/****************************************************************************\
*                                                                            *
* 18.04.2006                                                                 *
* CTerrainPartRep - Header File                                              *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTERRAINPARTREP_H__
#define __CTERRAINPARTREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CTerrainRep;
class IModelGeometry;
class ISceneGraph;
class CTerrain;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTerrainPartRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTerrainPartRep() {Clear();}
        
        bool Create(CTerrainRep *pTerrainRep, const TSmartPointer<IModelGeometry> pModelGeometry,
                    ISceneGraph *pSceneGraph, CTerrain *pTerrain);

        virtual const CVector3f &GetPos() const;
        virtual const CVector3f &GetRot() const;
        virtual const CVector3f &GetScale() const;
        virtual const CMatrix44 &GetTransformation();

        virtual IGameObj * const GetGameObj() const;
        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual void Destroy();
        
        virtual ~CTerrainPartRep() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CTerrainPartRep(const CTerrainPartRep &);
        const CTerrainPartRep &operator =(const CTerrainPartRep &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CTerrain *m_pTerrain;
        CTerrainRep *m_pTerrainRep;
        TSmartPointer<IModelGeometry> m_pModelGeometry;
        
        
};

#endif
