/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* CTerrainRep - Header File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTERRAINREP_H__
#define __CTERRAINREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CTerrainPartRep;
class ISceneGraph;
class CTerrainInfo;

class CTerrain;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTerrainRep : public IRepresentation
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CTerrainRep() {Clear();}
        
        bool Create(const std::string &sLevel, CTerrainInfo *pTerrainInfo, ISceneGraph *pSceneGraph,
                    CTerrain *pTerrain);

        virtual const CVector3f &GetPos() const;
        virtual const CVector3f &GetRot() const;
        virtual const CVector3f &GetScale() const;
        virtual const CMatrix44 &GetTransformation();

        virtual IGameObj * const GetGameObj() const;
        virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

        virtual void Destroy();

        virtual ~CTerrainRep() {Destroy();}
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        void Clear();
        virtual void RenderSub(const CFrustum &frustum, bool bVerify);

        //Private copy constructor and assignment operator to prevent mistakes:
        CTerrainRep(const CTerrainRep &);
        const CTerrainRep &operator =(const CTerrainRep &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        CTerrain *m_pTerrain;
		dword m_dwNumParts;
        TSmartPointer<CTerrainPartRep> *m_pTerrainParts;
        
};

#endif
