/****************************************************************************\
*                                                                            *
* 07.05.2006                                                                 *
* CTreeRep - Header File                                                     *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CTREEREP_H__
#define __CTREEREP_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <Memory/mmgr.h>
#include <Memory/TSmartPointer.h>
#include <Display/IRepresentation.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class IModelGeometry;
class ISceneGraph;
class CTreeObj;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CTreeRep : public IRepresentation
{
	/**************************************************************************\
	* PUBLIC CLASS FUNCTIONS                                                   *
	\**************************************************************************/
public:
	CTreeRep();

	bool Create(ISceneGraph *pSceneGraph, const std::string &sModel, CTreeObj *pTreeObj);

	virtual const CVector3f &GetPos() const;
	virtual const CVector3f &GetRot() const;
	virtual const CVector3f &GetScale() const;
	virtual const CMatrix44 &GetTransformation();
	virtual bool RayIntersection(const CVector3f &vOrigin, const CVector3f &vDir, CVector3f &vIntersectionPos, float &fIntersectionDistance) const;

	virtual IGameObj * const GetGameObj() const;

	virtual ~CTreeRep();

	/**************************************************************************\
	* PROTECTED CLASS FUNCTIONS                                                *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS FUNCTIONS                                                  *
	\**************************************************************************/
private:
	virtual void RenderSub(const CFrustum &frustum, bool bVerify);

	//Private copy constructor and assignment operator to prevent mistakes:
	CTreeRep(const CTreeRep &);
	const CTreeRep &operator =(const CTreeRep &);

	/**************************************************************************\
	* PROTECTED CLASS MEMBERS                                                  *
	\**************************************************************************/
protected:

	/**************************************************************************\
	* PRIVATE CLASS MEMBERS                                                    *
	\**************************************************************************/
private:

	CTreeObj *m_pTreeObj;
	TSmartPointer<IModelGeometry> m_pModelGeometry;

};

#endif
