/****************************************************************************\
*                                                                            *
* 16.04.2006                                                                 *
* CBuildState - Source File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include "CBuildState.h"
#include "CGameCtrl.h"
#include "CEngineFactory.h"
#include "CSettings.h"
#include "CBattleState.h"
#include <Display/Camera/CScrollXZCamCtrl.h>
#include "Objects/Buildings/CHqBuildingObj.h"
#include "Objects/Buildings/CTowerObj.h"
#include "Objects/Buildings/CAiTowerCtrl.h"

/****************************************************************************\
* PRIVATE DEFINITIONS                                                        *
\****************************************************************************/
enum EBuildingTypes
{
    EBT_HQ,
    EBT_CITIZEN1,
	EBT_CITIZEN2,
	EBT_MARKET,
	EBT_CHURCH,
	EBT_TOILET,
	EBT_TOWER,
};

const dword cg_dwBuildingTextColor = 0xD0D0D000;

/****************************************************************************\
* PUBLIC CLASS FUNCTIONS                                                     *
\****************************************************************************/
CBuildState::CBuildState()
{
    Clear();
}

bool CBuildState::Create(CGameCtrl *pGameCtrl)
{
    if(!IGameState::Create(pGameCtrl))
        return false;

	std::string sValue;
	//Building costs:
	float fCost;
	//Citizen1
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Citizen1", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Citizen1\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_CITIZEN1, fCost));
	//Citizen2
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Citizen2", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Citizen2\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_CITIZEN2, fCost));
	//Market
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Market", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Market\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_MARKET, fCost));
	//Church
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Church", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Church\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_CHURCH, fCost));
	//Toilet
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Toilet", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Toilet\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_TOILET, fCost));
	//Tower
	if(!CSettings::InstancePtr()->GetValue("COSTS", "Tower", sValue))
	{
		LOG(TL_ERROR, "Error: Couldn't get \"Tower\"-Value from \"COSTS\" in CBuildState::Enter");
		fCost = 0.0f;
	}
	else
	{
		fCost = (float) strtod(sValue.c_str(), NULL);
	}
	m_Costs.insert(std::make_pair(EBT_TOWER, fCost));
	//ToDo: Other buildings

    m_pTimer = new CTimer(pGameCtrl->GetClock());

	//Scroll-Kamerasteuerung
    m_pCamCtrl = new CScrollXZCamCtrl(1, CAABox3f(300, 50, 200, 600, 100, 600), 50, m_pTimer, 1500);

    TSmartPointer<CGuiText> pText;
    TSmartPointer<CGuiButton> pButton;
    //Get Pointer to gui mgr
    m_pGuiMgr = CEngineFactory::InstancePtr()->GetGuiMgr();
    //Create own gui root object
    m_pGuiRoot = m_pGuiMgr->CreateContainer();

	//Create the prebuild gui window
	m_pPreBuildWindow = m_pGuiMgr->CreateWnd("Data\\windows.tga", CVector2f(0.0f, 0.0f), CVector2f(1.0f, 0.25f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f), EBM_ALPHA);
	m_pPreBuildWindow->SetScale(CVector3f(1000.0f, 150.0f, 1.0f));
	//PreBuild-Text text
	m_pPreBuildText = m_pGuiMgr->CreateText();
	m_pPreBuildText->SetColor(0xFF000000);
	m_pPreBuildText->SetPos(CVector3f(845.0f, 40.0f, 1.0f));
	m_pPreBuildText->SetText("Additional");
	m_pPreBuildWindow->AddChild((IGuiObject *)((CGuiText *)m_pPreBuildText));
	m_pPreBuildText = m_pGuiMgr->CreateText();
	m_pPreBuildText->SetColor(0xFF000000);
	m_pPreBuildText->SetPos(CVector3f(845.0f, 60.0f, 1.0f));
	m_pPreBuildText->SetText("buildings");
	m_pPreBuildWindow->AddChild((IGuiObject *)((CGuiText *)m_pPreBuildText));
	m_pPreBuildText = m_pGuiMgr->CreateText();
	m_pPreBuildText->SetColor(0xFF000000);
	m_pPreBuildText->SetPos(CVector3f(845.0f, 80.0f, 1.0f));
	m_pPreBuildWindow->AddChild((IGuiObject *)((CGuiText *)m_pPreBuildText));
	//Add the pre-building buttons
	//HQ
	m_pHqButton = m_pGuiMgr->CreateButton();
	m_pHqButton->SetButtonId(EBT_HQ);
	m_pHqButton->SetToggleButton(true);
	m_pHqButton->SetPos(CVector3f(10.0f, 10.0f, 1.0f));
	m_pHqButton->SetScale(CVector3f(50.0f, 60.0f, 1.0f));
	m_pHqButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pPreBuildWindow->AddChild((IGuiObject *)((CGuiButton *)m_pHqButton));
	m_PreBuildingButtons.push_back(m_pHqButton);
	//Citizen
	m_pCitizenButton = m_pGuiMgr->CreateButton();
	m_pCitizenButton->SetButtonId(EBT_CITIZEN1);
	m_pCitizenButton->SetToggleButton(true);
	m_pCitizenButton->SetPos(CVector3f(10.0f, 80.0f, 1.0f));
	m_pCitizenButton->SetScale(CVector3f(50.0f, 60.0f, 1.0f));
	m_pCitizenButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pPreBuildWindow->AddChild((IGuiObject *)((CGuiButton *)m_pCitizenButton));
	m_PreBuildingButtons.push_back(m_pCitizenButton);
	//Add the window to the gui
	m_pGuiRoot->AddChild(m_pPreBuildWindow);

	//Create the top gui window
    m_pBuildWindow = m_pGuiMgr->CreateWnd("Data\\windows.tga", CVector2f(0.0f, 0.0f), CVector2f(1.0f, 0.25f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f), EBM_ALPHA);
    m_pBuildWindow->SetScale(CVector3f(1000.0f, 150.0f, 1.0f));
	//Money text
	m_pMoneyText = m_pGuiMgr->CreateText();
	m_pMoneyText->SetColor(0xFF000000);
	m_pMoneyText->SetPos(CVector3f(845.0f, 30.0f, 1.0f));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiText *)m_pMoneyText));
	//Time left
    m_pTimeLeft = m_pGuiMgr->CreateText();
    m_pTimeLeft->SetColor(0xFF000000);
    m_pTimeLeft->SetPos(CVector3f(845.0f, 55.0f, 1.0f));
    m_pBuildWindow->AddChild((IGuiObject *)((CGuiText *)m_pTimeLeft));
	//Skip button
	pButton = m_pGuiMgr->CreateButton();
	pButton->SetPos(CVector3f(840.0f, 80.0f, 1.0f));
	pButton->SetScale(CVector3f(150.0f, 50.0f, 1.0f));
	pButton->SetText("Skip", 0xD0D0D000);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnSkip));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
    //Add the building buttons
	char sText[32];
	//Citizen1
    pButton = m_pGuiMgr->CreateButton("Data/button_citizen1.tga");
    pButton->SetButtonId(EBT_CITIZEN1);
    pButton->SetToggleButton(true);
    pButton->SetPos(CVector3f(10.0f, 20.0f, 1.0f));
    pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_CITIZEN1)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
    pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
    m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
    m_BuildingButtons.push_back(pButton);
	//Citizen2
	pButton = m_pGuiMgr->CreateButton("Data/button_citizen2.tga");
	pButton->SetButtonId(EBT_CITIZEN2);
	pButton->SetToggleButton(true);
	pButton->SetPos(CVector3f(80.0f, 20.0f, 1.0f));
	pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_CITIZEN2)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
	m_BuildingButtons.push_back(pButton);
	//Market
	pButton = m_pGuiMgr->CreateButton("Data/button_market.tga");
	pButton->SetButtonId(EBT_MARKET);
	pButton->SetToggleButton(true);
	pButton->SetPos(CVector3f(150.0f, 20.0f, 1.0f));
	pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_MARKET)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
	m_BuildingButtons.push_back(pButton);
	//Church
	pButton = m_pGuiMgr->CreateButton("Data/button_church.tga");
	pButton->SetButtonId(EBT_CHURCH);
	pButton->SetToggleButton(true);
	pButton->SetPos(CVector3f(220.0f, 20.0f, 1.0f));
	pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_CHURCH)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
	m_BuildingButtons.push_back(pButton);
	//Toilet
	pButton = m_pGuiMgr->CreateButton("Data/button_toilet.tga");
	pButton->SetButtonId(EBT_TOILET);
	pButton->SetToggleButton(true);
	pButton->SetPos(CVector3f(290.0f, 20.0f, 1.0f));
	pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_TOILET)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
	m_BuildingButtons.push_back(pButton);
	//Tower
	pButton = m_pGuiMgr->CreateButton("Data/button_tower.tga");
	pButton->SetButtonId(EBT_TOWER);
	pButton->SetToggleButton(true);
	pButton->SetPos(CVector3f(360.0f, 20.0f, 1.0f));
	pButton->SetScale(CVector3f(60.0f, 80.0f, 1.0f));
	sprintf(sText, "%.0f,-", (*m_Costs.find(EBT_TOWER)).second);
	pButton->SetText(sText, cg_dwBuildingTextColor);
	pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnBuildButton));
	m_pBuildWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
	m_BuildingButtons.push_back(pButton);
	//Add the window to the gui
    m_pGuiRoot->AddChild(m_pBuildWindow);

    //Create the battle message box
    m_pBattleWindow = m_pGuiMgr->CreateWnd("Data\\buttonTex.tga", CVector2f(0.0f, 0.0f), CVector2f(1.0f, 0.25f), CVector4f(1.0f, 1.0f, 1.0f, 1.0f));
    m_pBattleWindow->Show(false);
    m_pBattleWindow->SetPos(CVector3f(330.0f, 425.0f, 1.0f));
    m_pBattleWindow->SetScale(CVector3f(340.0f, 150.0f, 1.0f));
    //Add info text
    pText = m_pGuiMgr->CreateText();
    pText->SetColor(0xFF000000);
    pText->SetPos(CVector3f(30.0f, 30.0f, 1.0f));
    pText->SetText("Build time is up. Incoming Birds!");
    m_pBattleWindow->AddChild((IGuiObject *)((CGuiText *)pText));
    //Add button
    pButton = m_pGuiMgr->CreateButton();
    pButton->SetPos(CVector3f(95.0f, 80.0f, 1.0f));
    pButton->SetScale(CVector3f(150.0f, 50.0f, 1.0f));
    pButton->SetText("To battle!", 0xD0D0D000);
    pButton->SetOnClickHandler(new TParamFunctor<CBuildState>(this, OnEnterBattle));
    m_pBattleWindow->AddChild((IGuiObject *)((CGuiButton *)pButton));
    m_pGuiRoot->AddChild(m_pBattleWindow);

    //Set input priority (fairly low)
    SetPriority(2000);

	//Load background sound
	m_pBgSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eSTREAM, "Data/sounds/bg_build.mp3");
	m_pBgSound->SetLoopCount(20);
	m_pBgSound->SetVolume(1.0f);
	m_pBuildSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE, "Data/sounds/build.mp3");
	m_pBuildSound->SetVolume(0.4f);
	m_pNoBuildSound = CEngineFactory::InstancePtr()->GetSoundMgr()->LoadSound(eDECOMPRESSED_SAMPLE, "Data/sounds/nobuild.mp3");
	m_pNoBuildSound->SetVolume(0.4f);

    return true;
}

void CBuildState::Enter()
{
	m_pBgSound->Play();

    std::string sValue;
    //Get required parameters
	if(m_pGameCtrl->GetLevel() == 1)
	{
		//Start money
		if(!CSettings::InstancePtr()->GetValue("GAME", "StartMoney", sValue))
		{
			LOG(TL_ERROR, "Error: Couldn't get \"StartMoney\"-Value from \"GAME\" in CBuildState::Enter");
			m_fMoney = 0.0f;
		}
		else
		{
			m_fMoney = (float) strtod(sValue.c_str(), NULL);
		}
		//Disable too expensive buttons, etc.
		MoneyChanged();
	}

    if(!CSettings::InstancePtr()->GetValue(m_pGameCtrl->GetLevelName(), "BuildTime", sValue))
    {
        LOG(TL_ERROR, "Error: Couldn't get \"BuildTime\"-Value from \"%s\" in CBuildState::Enter", m_pGameCtrl->GetLevelName().c_str());
        m_fBuildTime = 10.0f;
    }
    else
    {
        m_fBuildTime = (float) strtod(sValue.c_str(), NULL);
    }
	//Get prebuild counts
	m_wPrebuildTotal = 0;
	if(CSettings::InstancePtr()->GetValue(m_pGameCtrl->GetLevelName(), "PreHq", sValue))
	{
		m_wPrebuildTotal += m_wPrebuildHq = (word) strtoul(sValue.c_str(), NULL, 10);
	}
	else
	{
		m_wPrebuildHq = 0;
	}
	UpdateButton(m_pHqButton, m_wPrebuildHq);
	if(CSettings::InstancePtr()->GetValue(m_pGameCtrl->GetLevelName(), "PreCitizen", sValue))
	{
		m_wPrebuildTotal += m_wPrebuildCitizen = (word) strtoul(sValue.c_str(), NULL, 10);
	}
	else
	{
		m_wPrebuildCitizen = 0;
	}
	UpdateButton(m_pCitizenButton, m_wPrebuildCitizen);
	char sText[32];
	sprintf(sText, "necessary: %02u", m_wPrebuildTotal);
	m_pPreBuildText->SetText(sText);

	m_pPreBuildWindow->Show(m_wPrebuildTotal > 0);
	m_pBuildWindow->Show(m_wPrebuildTotal == 0);

    m_pTimer->Enable(true);
    m_pGameCtrl->SetCamCtrl(m_pCamCtrl);
	m_pGameCtrl->SetCameraPos(m_pGameCtrl->GetPlayerStartPos() + CVector3f(0.0f, 50.0f, 25.0f));
    m_pGuiMgr->SetRootObject(m_pGuiRoot);
    CEngineFactory::InstancePtr()->GetInputMgr()->RegisterHandler(this);
}

bool CBuildState::Update()
{
	if(m_wPrebuildTotal)
	{
	}
	else
	{
		if(m_fBuildTime >= 0.0f)
		{
			m_fBuildTime -= (float) m_pTimer->GetFrameDuration();

			if(m_fBuildTime < 0.0f)
			{
				m_fBuildTime = 0.0f;
				//Remove Building from cursor:
				OnMouseKeyDown(1);
				//Show "To battle" message box and make it modal
				m_pBattleWindow->Show(true);
				m_pGuiMgr->GrabInput(m_pBattleWindow);

				m_pTimer->Enable(false);
			}
			char sText[32];
			float mins = floorf(m_fBuildTime / 60.0f);
			sprintf(sText, "Time left: %02.0f:%02.0f", mins, m_fBuildTime - 60 * mins);
			m_pTimeLeft->SetText(sText);
		}
	}
    
    m_pGameCtrl->UpdateGameObjects((float) m_pTimer->GetFrameDuration());

    return true;
}

void CBuildState::Leave()
{
    CEngineFactory::InstancePtr()->GetInputMgr()->UnregisterHandler(this);
    m_pGuiMgr->SetRootObject(NULL);
    m_pGameCtrl->SetCamCtrl(NULL);
    m_pTimer->Enable(false);

	m_pBgSound->Stop();
}

void CBuildState::AddMoney(float fAmount)
{
	m_fMoney += fAmount;
	MoneyChanged();
}

void CBuildState::RemoveMoney(float fAmount)
{
	m_fMoney -= fAmount;
	MoneyChanged();
}

void CBuildState::Destroy()
{
    IGameState::Destroy();
    Clear();
}

dword CBuildState::OnEnterBattle(dword dwButtonId)
{
    m_pGuiMgr->ReleaseInput(m_pBattleWindow);
    m_pBattleWindow->Show(false);
    m_pGameCtrl->SetCurrentState(m_pGameCtrl->GetBattleState());
    return 0;
}

dword CBuildState::OnBuildButton(dword dwButtonId)
{
	ButtonList::iterator it;
	if(m_wPrebuildTotal)
	{
		for(it = m_PreBuildingButtons.begin(); it != m_PreBuildingButtons.end(); ++it)
		{
			if((*it)->GetButtonId() != dwButtonId)
			{
				(*it)->SetToggleState(false);
			}
		}
	}
	else
	{
		for(it = m_BuildingButtons.begin(); it != m_BuildingButtons.end(); ++it)
		{
			if((*it)->GetButtonId() != dwButtonId)
			{
				(*it)->SetToggleState(false);
			}
		}
	}

	if(m_pBuildingBrush)
	{
		m_pBuildingBrush->Destroy();
		m_pBuildingBrush = NULL;
	}

	//Get building costs:
	float fCosts;
	CostMap::iterator cmit;
	cmit = m_Costs.find(m_dwBrushObject);
	if(cmit == m_Costs.end())
	{
		fCosts = 0.0f;
	}
	else
	{
		fCosts = (*cmit).second;
	}

	CTowerObj *pObj;
	switch(dwButtonId)
	{
		case EBT_CITIZEN1:
			m_pBuildingBrush = new CBuildingObj(m_pGameCtrl, "Data/models/buildings/haus1.fbx", 120, fCosts, ECG_BUILDING);
			break;
		case EBT_CITIZEN2:
			m_pBuildingBrush = new CBuildingObj(m_pGameCtrl, "Data/models/buildings/haus2.fbx", 200, fCosts, ECG_BUILDING);
			break;
		case EBT_MARKET:
			m_pBuildingBrush = new CBuildingObj(m_pGameCtrl, "Data/models/buildings/marktstand.fbx", 200, fCosts, ECG_BUILDING);
			break;
		case EBT_CHURCH:
			m_pBuildingBrush = new CBuildingObj(m_pGameCtrl, "Data/models/buildings/kirche.fbx", 300, fCosts, ECG_BUILDING);
			break;
		case EBT_TOILET:
			m_pBuildingBrush = new CBuildingObj(m_pGameCtrl, "Data/models/buildings/klo.fbx", 50, fCosts, ECG_BUILDING);
			break;
		case EBT_TOWER:
			pObj = new CTowerObj(m_pGameCtrl);
			pObj->SetTowerCtrl(new CAiTowerCtrl(m_pGameCtrl));
			m_pBuildingBrush = pObj;
			break;
		default:
			LOG(TL_ERROR, "Warning: Unknown BuildButton clicked: %lu", dwButtonId);
	}

	if(m_pBuildingBrush)
	{
		m_pGameCtrl->AddObject((IGameObj *)((CBuildingObj *)m_pBuildingBrush));
		m_dwBrushObject = dwButtonId;
	}

    return 0;
}

dword CBuildState::OnSkip(dword dwButtonId)
{
	m_fBuildTime = 0.0f;
	return 0;
}

CBuildState::~CBuildState()
{
    Destroy();
}

/****************************************************************************\
* PROTECTED CLASS FUNCTIONS                                                  *
\****************************************************************************/

/****************************************************************************\
* PRIVATE CLASS FUNCTIONS                                                    *
\****************************************************************************/
bool CBuildState::OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ)
{
    if(m_pBuildingBrush)
    {
        TSmartPointer<IRepresentation> pRep;
		CVector3f vPos;
        CVector3f vOrigin(m_pGameCtrl->GetCameraPos());
        CVector3f vDir(m_pGameCtrl->GetCameraRay());
        m_pGameCtrl->GetSceneGraph()->PickObject(vOrigin, vDir, pRep, vPos);
        if(pRep)
        {
            if(pRep->GetGameObj() == m_pGameCtrl->GetTerrain())
            {
                //Terrain hit
                //Set object brush to this location
                m_pBuildingBrush->SetPos(vPos);
				m_pBuildingBrush->UpdateState();
            }
        }
    }

    return true;
}

bool CBuildState::OnMouseKeyDown(byte bKey)
{
	if(bKey == 0)
    {
        if(m_pBuildingBrush)
        {
            if(m_pBuildingBrush->Build())
			{
				m_pBuildingBrush = NULL;
				//Keep track of prebuild counts
				if(m_wPrebuildTotal)
				{
					--m_wPrebuildTotal;
					char sText[32];
					sprintf(sText, "necessary: %02u", m_wPrebuildTotal);
					m_pPreBuildText->SetText(sText);
					switch(m_dwBrushObject)
					{
						case EBT_HQ:
							--m_wPrebuildHq;
							UpdateButton(m_pHqButton, m_wPrebuildHq);
							break;
						case EBT_CITIZEN1:
							--m_wPrebuildCitizen;
							UpdateButton(m_pCitizenButton, m_wPrebuildCitizen);
							break;
						default:
							LOG(TL_ERROR, "Warning: Unknown m_dwBrushObject: %lu", m_dwBrushObject);
					}
					if(!m_wPrebuildTotal)
					{
						m_pPreBuildWindow->Show(false);
						m_pBuildWindow->Show(true);
					}
				}
				else
				{
					//Pay building costs
					CostMap::iterator it;
					it = m_Costs.find(m_dwBrushObject);
					if(it == m_Costs.end())
					{
						LOG(TL_ERROR, "Error: Couldn't find costs for %lu", m_dwBrushObject);
					}
					else
					{
						RemoveMoney((*it).second);
					}
				}
				m_pBuildSound->Play();
			}
			else
			{
				m_pNoBuildSound->Play();
			}
		}
    }
    else
    {
        if(m_pBuildingBrush)
        {
			m_pBuildingBrush->Destroy();
            m_pBuildingBrush = NULL;
        }
    }
	if(!m_pBuildingBrush)
	{
		ButtonList::iterator it;
		for(it = m_PreBuildingButtons.begin(); it != m_PreBuildingButtons.end(); ++it)
		{
			(*it)->SetToggleState(false);
		}
		for(it = m_BuildingButtons.begin(); it != m_BuildingButtons.end(); ++it)
		{
			(*it)->SetToggleState(false);
		}
	}

    return true;
}

void CBuildState::UpdateButton(const TSmartPointer<CGuiButton> &pButton, word wBuildCount)
{
	char sText[32];
	pButton->Enable(wBuildCount > 0);
	sprintf(sText, "%u", wBuildCount);
	pButton->SetText(sText, cg_dwBuildingTextColor);
}

void CBuildState::MoneyChanged()
{
	ButtonList::iterator it;
	CostMap::iterator cmi;

	for(it = m_BuildingButtons.begin(); it != m_BuildingButtons.end(); ++it)
	{
		cmi = m_Costs.find((*it)->GetButtonId());
		assert(cmi != m_Costs.end() && "Error: Building ID not found in cost map");
		(*it)->Enable((*cmi).second <= m_fMoney);
	}

	char sText[32];
	sprintf(sText, "Gold: %.0f", m_fMoney);
	m_pMoneyText->SetText(sText);
}

void CBuildState::Clear()
{
    m_pTimer = NULL;
    m_pGuiRoot = NULL;
}


