/****************************************************************************\
*                                                                            *
* 15.04.2006                                                                 *
* CBuildState - Header File                                                  *
*                                                                            *
* (c) Raphael Charwot, Robert Hausmair                                       *
*                                                                            *
\****************************************************************************/

#ifndef __CBUILDSTATE_H__
#define __CBUILDSTATE_H__

/****************************************************************************\
* REQUIRED HEADERS                                                           *
\****************************************************************************/
#include <Definitions.h>

#include <map>

#include <Memory/mmgr.h>
#include "IGameState.h"
#include "Objects/Buildings/CBuildingObj.h"
#include <Display/Gui/CGuiText.h>
#include <Display/Camera/ICamCtrl.h>
#include <Timer/CTimer.h>

/****************************************************************************\
* DEFINITIONS                                                                *
\****************************************************************************/
class CGameCtrl;
class CGuiButton;

/****************************************************************************\
* CLASS                                                                      *
\****************************************************************************/
class CBuildState : public IGameState, private IInputHandler
{
    /**************************************************************************\
    * PUBLIC CLASS FUNCTIONS                                                   *
    \**************************************************************************/
    public:
        CBuildState();
        
        virtual bool Create(CGameCtrl *pGameCtrl);
        virtual void Enter();
        virtual bool Update();
        virtual void Leave();
        virtual void Destroy();

		void AddMoney(float fAmount);
		void RemoveMoney(float fAmount);

        virtual ~CBuildState();

        dword OnEnterBattle(dword dwButtonId);
        dword OnBuildButton(dword dwButtonId);
		dword OnSkip(dword dwButtonId);
        
    /**************************************************************************\
    * PROTECTED CLASS FUNCTIONS                                                *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS FUNCTIONS                                                  *
    \**************************************************************************/
    private:
        virtual bool OnMouseMove(sdword sdwX, sdword sdwY, sdword sdwZ);
        virtual bool OnMouseKeyDown(byte bKey);
		void UpdateButton(const TSmartPointer<CGuiButton> &pButton, word wBuildCount);
		void MoneyChanged();

        void Clear();
        //Private copy constructor and assignment operator to prevent mistakes:
        CBuildState(const CBuildState &);
        const CBuildState &operator =(const CBuildState &);
        
    /**************************************************************************\
    * PROTECTED CLASS MEMBERS                                                  *
    \**************************************************************************/
    protected:
        
    /**************************************************************************\
    * PRIVATE CLASS MEMBERS                                                    *
    \**************************************************************************/
    private:
        TSmartPointer<CTimer> m_pTimer;
        TSmartPointer<ICamCtrl> m_pCamCtrl;

		word m_wPrebuildTotal;
		word m_wPrebuildHq;
		TSmartPointer<CGuiButton> m_pHqButton;
		word m_wPrebuildCitizen;
		TSmartPointer<CGuiButton> m_pCitizenButton;

        float m_fBuildTime;

		float m_fMoney;

        TSmartPointer<IGuiObject> m_pGuiRoot;
		TSmartPointer<IGuiObject> m_pPreBuildWindow;
		TSmartPointer<CGuiText> m_pPreBuildText;
		TSmartPointer<IGuiObject> m_pBuildWindow;
		TSmartPointer<CGuiText> m_pMoneyText;
		TSmartPointer<CGuiText> m_pTimeLeft;
		TSmartPointer<IGuiObject> m_pBattleWindow;
        typedef std::map<dword, float> CostMap;
		CostMap m_Costs;
		typedef std::vector< TSmartPointer<CGuiButton> > ButtonList;
		ButtonList m_PreBuildingButtons;
        ButtonList m_BuildingButtons;

        TSmartPointer<CBuildingObj> m_pBuildingBrush;   //The object that should be placed with the mouse
		dword m_dwBrushObject;

        //For quicker access
        TSmartPointer<CGuiMgr> m_pGuiMgr;

		TSmartPointer<ISound> m_pBgSound;
		TSmartPointer<ISound> m_pBuildSound;
		TSmartPointer<ISound> m_pNoBuildSound;
};

#endif
